/*
 * linux/arch/arm/boot/compressed/head.S
 *
 * Copyright (C) 1996 Russell King
 */
		.text
/*
 * sort out different calling conventions
 */
start:

/*
 * this is position independent code.
 * the idea is that it copies itself to another location
 * (namely 0xc0100000), then jumps to where it was linked
 * to (newstart)
 */


/*
 * r0 is a pointer to params
 * r1 is a pointer to the architecture type
 */
		mov	r5, r0    @ preserve location of params
		mov	r4, r1
		b	skip_clksleep
	
/*----------------------------------------------------*/

/*
 *	Inputs:  r2: MDCNFG, r3: MDCAS0, r4: MDCAS1, r5: MDCAS2. 
 *		r1: &MDCNFG.
 *		lr has return location.  
 */
ClkSleep:	
	stmia	r1, {r2-r5}
	mov	pc, lr

skip_clksleep:	
		add	r1, pc, #(start - . - 8) @ get location we are executing at
	
	
/*
 * r0 = to, r1 = from, r2 = length
 */
	
@		mov	r0, #0xC0100000        @ where to copy compressed kernel to
		mov	r0,     #0xC0000000
		add	r0, r0, #0x00100000

		mov	r2, #0x70000  @ overkill
	
		add	r3, r1, r2
1:		ldmia	r1!, {r6 - r12, lr}
		stmia	r0!, {r6 - r12, lr}
		cmp	r1, r3
		blt	1b

		@ call kernel correctly (restore saved args)
		mov	r0, r5
		mov	r1, r4
		add	r2, pc, #(ClkSleep - . - 8)

		mov	r5, #(newstart - start)
@		mov	r6, #0xC0100000        @ where to copy compressed kernel to
		mov	r6,     #0xC0000000
		add	r6, r6, #0x00100000
		add	pc, r6, r5             @ this jumps to the relocated kernel
	
newstart:		
		teq	r0, #0   @ this basically means we're going to lose...
		beq	noparams
		mov	r4, #0xc0000000         @ PARAMS_BASE
		add	r4, r4, #0x0100
		mov	r3, #0x4000
		sub	r3, r3, #4
1:		ldmia	r0!, {r5 - r12}
		stmia	r4!, {r5 - r12}
		subs	r3, r3, #32
		bpl	1b
noparams:	mov	r8, r2   @ move clksleep to r8
		adr	r2, LC0
		ldmia	r2, {r2, r3, r4, r5, r7, sp}
/*
 * Clear zero-init
 */
		mov	r6, #0
1:	
		tst	r2, #3			@ deal with bss_start unaligned case
		strneb	r6, [r2], #1
		bne	1b	
	
1:		str	r6, [r2], #4
		cmp	r2, r3
		blt	1b
		str	r1, [r5]		@ save architecture
		str	r8, [r7]		@ save clksleep

/*
 * Uncompress the kernel
 */
		mov	r1, #0x8000
		add	r2, r2, r1, lsl #1	@ Add 64k for malloc
		sub	r1, r1, #1
		add	r2, r2, r1
		bic	r5, r2, r1		@ decompress kernel to after end of the compressed
		mov	r0, r5
		bl	decompress_kernel
		add	r0, r0, #7
		bic	r2, r0, #7

/*
 * Now move the kernel to the correct location (r5 -> r4, len r0)
 */
		mov	r0, r4			@ r0 = start of real kernel
		mov	r1, r5			@ r1 = start of kernel image
		add	r3, r5, r2		@ r3 = end of kernel
		adr	r4, movecode
		adr	r5, movecodeend
1:		ldmia	r4!, {r6 - r12, lr}
		stmia	r3!, {r6 - r12, lr}
		cmp	r4, r5
		blt	1b
		ldr	r5, LC0 + 12		@ get architecture pointer
		add	pc, r1, r2		@ Call move code

/*
 * r0 = length, r1 = to, r2 = from
 */
movecode:	add	r3, r1, r2
		mov	r4, r0
1:		ldmia	r1!, {r6 - r12, lr}
		stmia	r0!, {r6 - r12, lr}
		cmp	r1, r3
		blt	1b
		mov	r0, #0			@ call kernel correctly
		ldr	r1, [r5]		@ get architecture
		ldr	r2, [r5, #4]		@ pass in clksleep address
		add	pc, r4, #0		@ call via EXEC entry

movecodeend:
LC0:		.word	__bss_start
		.word	_end
		.word	LOADADDR
		.word	architecture
		.word   clock_sleep
		.word	user_stack+4096


		.bss
architecture:	.word	0        @ these two words must be consecutive
clock_sleep:	.word	0


