
/*
 *	FILE    	lnxstart.c
 *
 *	Version 	1.2
 *	Author  	Marc A. Viredaz
 *	        	Compaq Western Research Laboratory, Palo Alto, CA
 *	Date    	January 1999 (December 1998)
 *	System  	Itsy versions 1.0/1.1 and 1.5 (StrongARM SA-1100)
 *	Language	C
 *	Purpose 	This program updates the Linux parameter structure,
 *	        	according to the current run-time configuration, and
 *	        	start Linux.
 *
 * Copyright (c) Compaq Computer Corporation, 1998, 1999
 */

/* avoid sucking in linux mmap defs */
#define	VirtAdd(x)	(x)

/* XXX #undef __linux__ */

#include <asm/arch/ARMv4.h>
#include <asm/arch/SA-1100.h>
#include <asm/arch/Itsy.h>
#include "monpar.h"
#include "../arch/arm/kernel/params.h"

extern
struct param_struct	brutus_params ;


/*
 * PUBLIC PROCEDURE: Start
 *
 * Purpose
 *    The procedure "Start" is the main routine of the program "lnxstart".
 *
 * Input
 *    LnxStartAdd	Linux start address.
 *    RAMParamPtr	Pointer in RAM where to store the updated Linux
 *              	parameter structure.
 *    Arch      	Linux architecture code.
 *
 * Output       	-
 */

void Start (
   void         	(*LnxStartAdd) (struct param_struct *, Word),
   struct param_struct	*RAMParamPtr,
   Word         	Arch)
{
   unsigned int 	Size, BnkNb, TotPage, NbWord ;
   Word         	*WordPtr1, *WordPtr2, FstWord ;

   WordPtr1 = (Word *) &brutus_params ;
   WordPtr2 = (Word *) RAMParamPtr ;
   for (Size = 0 ; Size < sizeof (struct param_struct) ; Size += sizeof (Word))
      *(WordPtr2++) = *(WordPtr1++) ;
   RAMParamPtr->itsy_version = ItsyPar.ItsyVersion ;
   RAMParamPtr->mon_version = ItsyPar.MonVersion ;
   for (BnkNb = 0 ; BnkNb < 4 ; BnkNb++)
   {
     RAMParamPtr->class_id [BnkNb] = (ItsyPar.StMemPar [BnkNb]).Ident ;
     RAMParamPtr->staticmem_info_pointers [BnkNb] =
        (param_u32) (ItsyPar.StMemPar [BnkNb]).CtrlTbl ;
   }
   TotPage = 0 ;
   for (BnkNb = 0 ; BnkNb < 4 ; BnkNb++)
      if (MDCNFG & MDCNFG_DE (BnkNb))
      {
         FstWord = *(WordPtr1 = (Word *) DRAMBnk [BnkNb]) ;
         for (NbWord = 1 ; NbWord < DRAMBnkSp/sizeof (Word) ; NbWord *= 2)
            if (WordPtr1 [NbWord] == FstWord)
            {
               *WordPtr1 = ~FstWord ;
               if (WordPtr1 [NbWord] == ~FstWord)
               {
                  *WordPtr1 = FstWord ;
                  break ;
               }
               *WordPtr1 = FstWord ;
            }
         RAMParamPtr->pages_in_bank [BnkNb] =
            (param_u32) ((NbWord*sizeof (Word)) >> FSize (Add_SmPgIdx)) ;
         TotPage += (NbWord*sizeof (Word)) >> FSize (Add_SmPgIdx) ;
      }
      else
         RAMParamPtr->pages_in_bank [BnkNb] = 0 ;
   RAMParamPtr->nr_pages = TotPage ;
   RAMParamPtr->dram_info_pointer = (param_u32) ItsyPar.DRAMCtrlTbl ;
   (*LnxStartAdd) (RAMParamPtr, Arch) ;
}
