/*
 * linux/arch/arm/drivers/block/ide-ics.c
 *
 * Copyright (c) 1996,1997 Russell King.
 *
 * Changelog:
 *  08-06-1996	RMK	Created
 */

#include <linux/module.h>
#include <linux/malloc.h>
#include <linux/blkdev.h>
#include <linux/errno.h>

#include <asm/ecard.h>

#include "ide.h"

static const card_ids icside_cids[] = {
	{ MANU_ICS, PROD_ICS_IDE },
	{ 0xffff, 0xffff }
};

static struct expansion_card *ec[MAX_ECARDS];
static int result[MAX_ECARDS];

static inline int icside_register (struct expansion_card *ec)
{
	unsigned long port;

	port = ecard_address (ec, ECARD_MEMC, 0);
	ecard_claim (ec);

	return ide_register_port (port, ec->irq, 16);
}

int icside_init (void)
{
	int i;

	for (i = 0; i < MAX_ECARDS; i++)
		ec[i] = NULL;

	ecard_startfind ();

	for (i = 0; ; i++) {
		if ((ec[i] = ecard_find (0, icside_cids)) == NULL)
			break;

		result[i] = icside_register (ec[i]);
	}
	for (i = 0; i < MAX_ECARDS; i++)
		if (ec[i] && result[i] < 0) {
			ecard_release (ec[i]);
			ec[i] = NULL;
	}
	return 0;
}

#ifdef MODULE

int init_module (void)
{
	return icside_init();
}

void cleanup_module (void)
{
	int i;

	for (i = 0; i < MAX_ECARDS; i++)
		if (ec[i]) {
			unsigned long port;
			port = ecard_address (ec[i], ECARD_MEMC, 0);

			ide_unregister_port (port, ec[i]->irq, 16);
			ecard_release (ec[i]);
			ec[i] = NULL;
		}
}
#endif

