/*
 * DS2437 Smart Battery Monitor Commands
 * Copyright (c) Compaq Computer Corporation, 1999
 * Platform: Itsy V2.0 with SA-1100 processor
 * Author: Jennifer Anderson
 *
 * Interface to the DS2437 Smart Battery monitor.
 *		
 * IMPORTANT NOTE:
 * The DS2437 uses a timing-dependent protocol.  this code was timed to
 * work *only* with the SA-1100 at speeds of 59 MHz to 206.4 MHz.  
 */

	
#define LANGUAGE Assembly

#include <asm/arch/ARMv4.h>
#include <asm/arch/io_reg.h>
	
#define BATTERY_GPIO_NUM  9
#define BATTERY_GPIO      (0x00000001 << (BATTERY_GPIO_NUM))

#define GPDR_In 	  0       	/* input  */
#define GPDR_Out	  1       	/* output */
	
	.text
	.align
	.globl battmon_send_reset
	.globl battmon_write_zero
	.globl battmon_write_one
	.globl battmon_read_bit

@
@ Global Constants
@

@ Address of GPLR register.  We access all other GPIO registers as
@ offsets from this register to save on load instructions.

GPLRAddr:	.long	GPLR_V

@ Tables of constants for delay loops.  The table "WaitXXXCyclesY"
@ contains the number of iterations of a Y cycle loop 
@ that results in a delay of XXX us, for the processor
@ running at different frequencies.

Wait480Cycles3:	
		.long	9440          	@ 59.0 MHz
		.long	11792           @ 73.7 MHz
		.long   14160           @ 88.5 MHz
		.long	16512           @ 103.2 MHz
		.long	18880           @ 118.0 MHz
		.long	21231           @ 132.7 MHz
		.long	23600           @ 147.5 MHz
		.long	25952           @ 162.2 MHz
		.long	28304           @ 176.9 MHz
		.long	30672           @ 191.7 MHz
		.long	33024           @ 206.4 MHz
		.long	35392           @ 221.2 MHz
Wait1Cycles18:	
		.long	4               @ 59.0 MHz
		.long	5               @ 73.7 MHz
		.long	5               @ 88.5 MHz
		.long	6               @ 103.2 MHz
		.long	7               @ 118.0 MHz
		.long	8               @ 132.7 MHz
		.long	9               @ 147.5 MHz
		.long	10              @ 162.2 MHz
		.long	10              @ 176.9 MHz
		.long	11              @ 191.7 MHz
		.long	12              @ 206.4 MHz
		.long	13              @ 221.2 MHz
Wait480Cycles18:	
		.long	1574		@ 59.0 MHz
		.long	1966            @ 73.7 MHz
		.long	2360            @ 88.5 MHz
		.long	2752            @ 103.2 MHz
		.long	3147            @ 118.0 MHz
		.long	3539            @ 132.7 MHz
		.long	3934            @ 147.5 MHz
		.long	4326            @ 162.2 MHz
		.long	4718            @ 176.9 MHz
		.long	5112            @ 191.7 MHz
		.long	5504            @ 206.4 MHz
		.long	5899            @ 221.2 MHz
Wait1Cycles3:	
		.long	20		@ 59.0 MHz
		.long	25              @ 73.7 MHz
		.long	30              @ 88.5 MHz
		.long	35              @ 103.2 MHz
		.long	40              @ 118.0 MHz
		.long	45              @ 132.7 MHz
		.long	50              @ 147.5 MHz
		.long	55              @ 162.2 MHz
		.long	59              @ 176.9 MHz
		.long	64              @ 191.7 MHz
		.long	69              @ 206.4 MHz
		.long	74              @ 221.2 MHz
Wait60Cycles3:	
		.long	1180		@ 59.0 MHz
		.long	1474            @ 73.7 MHz
		.long	1770            @ 88.5 MHz
		.long	2064            @ 103.2 MHz
		.long	2360            @ 118.0 MHz
		.long	2654            @ 132.7 MHz
		.long	2950            @ 147.5 MHz
		.long	3244            @ 162.2 MHz
		.long	3538            @ 176.9 MHz
		.long	3834            @ 191.7 MHz
		.long	4128            @ 206.4 MHz
		.long	4424            @ 221.2 MHz
Wait59Cycles3:	
		.long	1161            @ 59.0 MHz
		.long	1450            @ 73.7 MHz
		.long	1741            @ 88.5 MHz
		.long	2030            @ 103.2 MHz
		.long	2321            @ 118.0 MHz
		.long	2610            @ 132.7 MHz
		.long	2901            @ 147.5 MHz
		.long	3190            @ 162.2 MHz
		.long	3480            @ 176.9 MHz
		.long	3771            @ 191.7 MHz
		.long	4060            @ 206.4 MHz
		.long	4351            @ 221.2 MHz
Wait10Cycles3:	
		.long	197		@ 59.0 MHz
		.long	246             @ 73.7 MHz
		.long	295             @ 88.5 MHz
		.long	344             @ 103.2 MHz
		.long	394             @ 118.0 MHz
		.long	443             @ 132.7 MHz
		.long	492             @ 147.5 MHz
		.long	541             @ 162.2 MHz
		.long	590             @ 176.9 MHz
		.long	639             @ 191.7 MHz
		.long	688             @ 206.4 MHz
		.long	738             @ 221.2 MHz
Wait49Cycles3:	
		.long	964		@ 59.0 MHz
		.long	1204            @ 73.7 MHz
		.long	1446            @ 88.5 MHz
		.long	1686            @ 103.2 MHz
		.long	1928            @ 118.0 MHz
		.long	2168            @ 132.7 MHz
		.long	2410            @ 147.5 MHz
		.long	2650            @ 162.2 MHz
		.long	2890            @ 176.9 MHz
		.long	3132            @ 191.7 MHz
		.long	3372            @ 206.4 MHz
		.long	3613            @ 221.2 MHz

/*	
 *  PUBLIC PROCEDURE: battmon_send_reset
 *
 *  Purpose
 *	The procedure "battmon_send_reset" transmits a reset pulse to the 
 *	to the DS2437 and watches for a presence pulse.  The sequence is:
 * 
 *	1. Send a reset pulse (low signal) 
 *	2. Hold for 480 - 980 us.   
 *	3. Line is pulled high, and the DS2437 detects the rising edge.
 *	4. The DS2437 waits 15 - 60 us.
 *	5. The DS2437 transmits a presence pulse (low signal) for 
 *	   60 - 240 us.
 *       6. Time slot is a minimum of 480 us after master releases line
 *
 *  C definition 	extern int battmon_send_reset(int); 
 *
 *  Input
 *	a1 (r0)         Clock frequency number (CCF<4:0> in PPCR)
 *	lr (r14)	Return address       
 *
 *  Output
 *	a1 (r0)			 
 *                       1 - presence pulse detected (success)
 *                       0 - presence pulse not detected (failure)
 *
 *  Modified
 * 	a1 (r0)		result register 
 *	a2 (r1)		used as scratch register
 *      a3 (r2)		used as scratch register
 *	a4 (r3)		holds GPDR with BATTERY_GPIO bit cleared
 *	v1 (r4)		holds clock frequency number
 *	ip (r12)	holds value of CPSR at entry to routine
 *	lr (r14)	holds address of GPLR
 */
battmon_send_reset:		
	MOV	ip, sp
	STMDB	sp!, {v1,fp,ip,lr,pc}
	SUB	fp, ip, #4
		
	@ Disable Interrupts
	MRS	ip, cpsr		
	ORR	a2, ip, #(PSR_F + PSR_I)	 
	MSR	cpsr, a2

	MOV	v1, a1			 
        MOV     a1, #0			@ Initialize routine result

	@ Configure BATTERY_GPIO as output
	LDR     lr, GPLRAddr		
	LDR     a4, [lr, #(GPDR_V - GPLR_V)]	
        BIC     a4, a4, #BATTERY_GPIO		
	ORR	a2, a4, #(BATTERY_GPIO * GPDR_Out)	
	STR     a2, [lr, #(GPDR_V - GPLR_V)]	

	@ Write to GPCR to clear BATTERY_GPIO
	MOV     a2, #BATTERY_GPIO
	STR     a2, [lr, #(GPCR_V - GPLR_V)]

	@ Hold low for 480 us
	ADR	a2, Wait480Cycles3	
	LDR	a2, [a2, v1, LSL #2]
HoldReset:	
	SUBS	a2, a2, #1	
	BNE	HoldReset

	@ Configure BATTERY_GPIO as input
	ORR	a2, a4, #(BATTERY_GPIO * GPDR_In)	
	STR     a2, [lr, #(GPDR_V - GPLR_V)]

	@ Wait 1 us for the line to go high
	ADR	a2, Wait1Cycles18	
	LDR	a2, [a2, v1, LSL #2]		 
WaitForHigh1:	
	LDR	a3, [lr, #(GPLR_V - GPLR_V)]
	ANDS    a3, a3, #BATTERY_GPIO	@ Once line is set, go look for
	BNE     LookForPresence	        @   presence pulse  
	SUBS	a2, a2, #1
	BNE     WaitForHigh1
	B	Done			@ fail if never see line high

	@ Look for the presence pulse
LookForPresence:					
	ADR	a2, Wait480Cycles18	@ wait out 480 us until end of slot
	LDR	a2, [a2, v1, LSL #2]    
WaitForPresence:	
	LDR	a3, [lr, #(GPLR_V - GPLR_V)]
	ANDS    a3, a3, #BATTERY_GPIO	
	MOVEQ   a1, #1			@ Saw presence pulse, success 
	SUBS	a2, a2, #1		
	BNE     WaitForPresence
Done:	
	MSR	cpsr, ip		@ restore CPSR
        LDMDB   fp, {v1,fp,sp,pc}	@ return   

		
/*	
 *  PUBLIC PROCEDURE: battmon_write_zero
 *
 *  Purpose
 *	The procedure "battmon_write_zero" sends a 0 bit to the DS2437.
 *	The sequence is:
 * 
 *	1. Wait 1 us recovery time
 *	2. Send a low signal
 *	3. Hold low for 60 us 
 *	4. The DS2437 checks the line 15 - 60 us after the line
 *          went low, and sees the line low
 *	5. Time slot is a total of 60 us. 	
 *
 *  C definition 	extern void battmon_write_zero(int); 
 *
 *  Input
 *	a1 (r0)          Clock frequency number (CCF<4:0> in PPCR)
 *	lr (r14)	 Return address       
 *
 *  Output
 *	None 
 *
 *
 *  Modified
 *	a2 (r1)		used as scratch register
 *	a4 (r3)		holds GPDR with BATTERY_GPIO bit cleared
 *	ip (r12)	holds value of CPSR at entry to routine
 *	lr (r14)	holds address of GPLR
 */	

battmon_write_zero:	
	MOV	ip, sp
	STMDB	sp!, {fp,ip,lr,pc}
	SUB	fp, ip, #4
		
	@ Disable Interrupts
	MRS	ip, cpsr		
	ORR	a2, ip, #(PSR_F + PSR_I)	 
	MSR	cpsr, a2

	@ Wait 1 us recovery time
	ADR	a2, Wait1Cycles3         
	LDR	a2, [a2, a1, LSL #2]
WaitForRecovery0:	
	SUBS	a2, a2, #1	
	BNE	WaitForRecovery0

	@ Configure BATTERY_GPIO as output
	LDR     lr, GPLRAddr		
	LDR     a4, [lr, #(GPDR_V - GPLR_V)]	
        BIC     a4, a4, #BATTERY_GPIO		
	ORR	a2, a4, #(BATTERY_GPIO * GPDR_Out)	
	STR     a2, [lr, #(GPDR_V - GPLR_V)]	

	@ Write to GPCR to clear BATTERY_GPIO
	MOV     a2, #BATTERY_GPIO
	STR     a2, [lr, #(GPCR_V - GPLR_V)]

	@ Hold line low for 60 us
	ADR	a2, Wait60Cycles3	
	LDR	a2, [a2, a1, LSL #2]
HoldLow0:	
	SUBS	a2, a2, #1	
	BNE	HoldLow0
	
	@ Configure BATTERY_GPIO as input
	@ to release the line
	ORR	a2, a4, #(BATTERY_GPIO * GPDR_In)	
	STR     a2, [lr, #(GPDR_V - GPLR_V)]
	
	MSR	cpsr, ip		@ restore CPSR
        LDMDB   fp, {fp,sp,pc}		@ return   	

	
/*	
 *  PUBLIC PROCEDURE: battmon_write_one
 *
 *  Purpose
 *	The procedure "battmon_write_one" sends a 1 bit to the DS2437.
 *	The sequence is:
 * 
 *	1. Wait 1 us recovery time
 *	2. Send a low signal
 *	3. Hold low for 1 us (must hold low for < 15 us)
 *	4. Line is pulled high
 *	5. The DS2437 checks the line 15 - 60 us after the line
 *	   went low, and now sees the line high.
 *	6. Time slot is a total of 60 us. 
 *
 *  C definition 	extern void battmon_write_one(int);
 *
 *  Input
 *	a1 (r0)		Clock frequency number (CCF<4:0> in PPCR)
 *	lr (r14)	Return address       
 *
 *  Output
 *	None 
 * 
 *  Modified
 *	a2 (r1)		used as scratch register
 *      a3 (r2)		used as scratch register
 *	a4 (r3)		holds GPDR with BATTERY_GPIO bit cleared
 *	ip (r12)	holds value of CPSR at entry to routine
 *	lr (r14)	holds address of GPLR
 */

battmon_write_one:	
	MOV	ip, sp
	STMDB	sp!, {fp,ip,lr,pc}
	SUB	fp, ip, #4
		
	@ Disable Interrupts
	MRS	ip, cpsr		
	ORR	a2, ip, #(PSR_F + PSR_I)	 
	MSR	cpsr, a2

	@ Wait 1 us recovery time
	ADR	a2, Wait1Cycles3         
	LDR	a2, [a2, a1, LSL #2]
	MOV	a3, a2
WaitForRecovery1:	
	SUBS	a2, a2, #1	
	BNE	WaitForRecovery1

	@ Configure BATTERY_GPIO as output
	LDR     lr, GPLRAddr		
	LDR     a4, [lr, #(GPDR_V - GPLR_V)]	
        BIC     a4, a4, #BATTERY_GPIO		
	ORR	a2, a4, #(BATTERY_GPIO * GPDR_Out)	
	STR     a2, [lr, #(GPDR_V - GPLR_V)]	

	@ Write to GPCR to clear BATTERY_GPIO
	MOV     a2, #BATTERY_GPIO
	STR     a2, [lr, #(GPCR_V - GPLR_V)]

	@ Hold line low for 1 us
HoldLow1:					
	SUBS	a3, a3, #1	       
	BNE	HoldLow1

	@ Configure BATTERY_GPIO as input
	ORR	a2, a4, #(BATTERY_GPIO * GPDR_In)	
	STR     a2, [lr, #(GPDR_V - GPLR_V)]

	@ Wait 59 us until end of slot
	ADR	a2, Wait59Cycles3	
	LDR	a2, [a2, a1, LSL #2]	
SlotDelay1:	
	SUBS	a2, a2,	#1	
	BNE	SlotDelay1

	MSR	cpsr, ip		@ restore CPSR
        LDMDB   fp, {fp,sp,pc}		@ return   		


/*	
 *  PUBLIC PROCEDURE: battmon_read_bit
 *
 *  Purpose
 *	The procedure "battmon_read_bit" reads a bit from the DS2437 and
 *	returns the bit read.  The sequence is:
 * 
 *	1. Wait 1 us recovery time
 *	2. Send a low signal
 *	3. Hold low for 1 us (must hold low for < 15 us)
 *	4. Output data is valid from the DS2437 valid within next 14 us max.
 *	5. Time slot is a total of 60 us 
 *
 *  C definition 	extern int battmon_read_bit(int);	
 *
 *  Input
 *	a1 (r0)         Clock frequency number (CCF<4:0> in PPCR)
 *	lr (a24)	Return address       
 *
 *  Output
 *	a1 (r0)		The bit read 
 *
 *  Modified
 *	a1 (r0)		result register 
 *	a2 (r1)		used as scratch register
 *      a3 (r2)		used as scratch register
 *	a4 (r3)		holds GPDR with BATTERY_GPIO bit cleared
 *      v1 (r4)		holds clock frequency number	
 *	ip (r12)	holds value of CPSR at entry to routine
 *	lr (r14)	holds address of GPLR
 */	

battmon_read_bit:	
	MOV	ip, sp
	STMDB	sp!, {v1,fp,ip,lr,pc}
	SUB	fp, ip, #4

	@ Disable Interrupts
	MRS	ip, cpsr		
	ORR	a2, ip, #(PSR_F + PSR_I)	 
	MSR	cpsr, a2

	MOV	v1, a1

	@ Wait 1 us recovery time
	ADR	a2, Wait1Cycles3	
	LDR	a2, [a2, v1, LSL #2]
	MOV	a3, a2
WaitForRecovery:	
	SUBS	a2, a2, #1	
	BNE	WaitForRecovery

	@ Configure BATTERY_GPIO as output
	LDR     lr, GPLRAddr		
	LDR     a4, [lr, #(GPDR_V - GPLR_V)]	
        BIC     a4, a4, #BATTERY_GPIO		
	ORR	a2, a4, #(BATTERY_GPIO * GPDR_Out)	
	STR     a2, [lr, #(GPDR_V - GPLR_V)]	

	@ Write to GPCR to clear BATTERY_GPIO
	MOV     a2, #BATTERY_GPIO
	STR     a2, [lr, #(GPCR_V - GPLR_V)]

	@ Hold line low for 1 us
HoldLow:					
	SUBS	a3, a3, #1	
	BNE	HoldLow

	@ Configure BATTERY_GPIO as input
	ORR	a2, a4, #(BATTERY_GPIO * GPDR_In)	
	STR     a2, [lr, #(GPDR_V - GPLR_V)]

	@ Wait 10 us before grabbing output
	ADR	a2, Wait10Cycles3	@ this gives plenty of time
	LDR	a2, [a2, v1, LSL #2]	@ for output from the DS2437
WaitForOutput:				@ to be ready
	SUBS	a2, a2,	#1		
	BNE	WaitForOutput

     	@ Grab the data
	LDR	a1, [lr, #(GPLR_V - GPLR_V)]
	AND     a1, a1, #BATTERY_GPIO
	MOV 	a1, a1, LSR #BATTERY_GPIO_NUM

	@ Wait another 49 us until end of slot
	ADR	a2, Wait49Cycles3
	LDR	a2, [a2, v1, LSL #2]
SlotDelay:	
	SUBS	a2, a2, #1	
	BNE	SlotDelay

	MSR	cpsr, ip		@ restore CPSR
        LDMDB   fp, {v1,fp,sp,pc}	@ return


