/*
 * SA1100 Clock Speed Device Driver
 * Copyright (c) Compaq Computer Corporation, 1998, 1999
 *
 * Author: Deborah A. Wallach
 *
 */

#define SUSPENDDEVICES

#include <linux/config.h>

#ifdef	MODULE
#include <linux/module.h>
#include <linux/version.h>
#endif

#include <linux/types.h>
#include <linux/fs.h>
#include <linux/powermgr.h>

#include <asm/io_reg.h>
#include <asm/io.h>

#include <linux/mm.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/hardware.h>
#include <asm/pgtable.h>
#include <asm/arch/mmu.h>
#include <asm/arch/mmap.h>
#include <linux/miscdevice.h>

#ifdef	CONFIG_PROC_FS
#include <linux/stat.h>
#include <linux/proc_fs.h>
#endif

#include <asm/arch/SA-1100.h>

#include <linux/clkspeed-sa1100.h>

extern int clock_switching_enabled;

static int static_memory_setup(int);

extern unsigned int dram_info_pointer;
extern unsigned int staticmem_info_pointers[4];
extern unsigned int params_version;

/*
 * debugging 
 *
 */

#define	CLKSPEED_DEBUG			(1)
#define	CLKSPEED_DEBUG_VERBOSE		(0)

/* names */
#define	CLKSPEED_NAME			"clkspeed-itsy"
#define	CLKSPEED_NAME_VERBOSE		"itsy clockspeed driver"
#define	CLKSPEED_VERSION_STRING		"$Revision: 2.8 $"

#ifdef	CONFIG_PROC_FS
static int clkspeed_get_info(char *, char **, off_t, int, int);
static struct proc_dir_entry clkspeed_proc_entry = {
  0, 10, "clockspeed", S_IFREG | S_IRUGO, 1, 0, 0, 0, NULL, clkspeed_get_info,
};
#endif	/* CONFIG_PROC_FS */

typedef struct {
  int mdcnfg;
  int mdcas0;
  int mdcas1;
  int mdcas2;
} dram_table_t;

static void disable_switching(void)
{
  int dummy1, dummy2;
  clock_switching_enabled = 0;
  __asm__("mcr        p15, 0, r0, c15, c2, 2
           mov        %0,     #0xe8000000
           add        %0, %0, #0x00050000
           ldr        %1, [%0]" : "=r" (dummy1), "=r" (dummy2) );
  return;
}
  
static void enable_switching(void)
{
  clock_switching_enabled = 1;
  __asm__("mcr        p15, 0, r0, c15, c1, 2"); 
  return;
}

static int clkspeed_change(int newspeed, int suspend)
{
  dram_table_t *dt, cdt;
  int currentspeed = PPCR_MASK & *(int *)PPCR_V;
  extern unsigned long sleep_function_virt, sleep_function_phys;
  extern void clock_call_dram_setup(int, int, dram_table_t *);
  int switching_enabled = 0;
  
  if (newspeed == currentspeed) return 0;

  if (suspend) {
    /* suspend devices */
    {
      int delay;
      if (powermgr_suspend_check(0, &delay)) {
	powermgr_suspend();
      }
      else {
	printk("suspend_check failed with delay %d\n", delay);
	return -EIO;
      }
    }
  }

  dt = (dram_table_t *) io_p2v(dram_info_pointer);  /* readonly table in flash */

  memcpy(&cdt, &(dt[newspeed]), sizeof(dram_table_t));
  cdt.mdcnfg = (cdt.mdcnfg & ~MDCNFG_DE_MASK) | (inl(MDCNFG_V) & MDCNFG_DE_MASK);

  /* done once per kernel boot lifecycle */
  if (sleep_function_virt == 0) {
    sleep_function_virt = io_p2v(sleep_function_phys);
  }

  if (newspeed > currentspeed) {
    static_memory_setup(newspeed);
    clock_call_dram_setup(sleep_function_virt, MDCNFG_V, &cdt);
  }

  if(clock_switching_enabled) {
      disable_switching();
      switching_enabled = 1;
  }

  outl(newspeed, PPCR_V);  

  if(switching_enabled) {
      enable_switching();
  }

  if (newspeed < currentspeed) {
    static_memory_setup(newspeed);
    clock_call_dram_setup(sleep_function_virt, MDCNFG_V, &cdt);
  }

  if (suspend) {
    powermgr_resume(inl(PSSR_V));
  }

  return(0);
}

static int clkspeed_ioctl(struct inode *inode,
			  struct file *file,
			  uint command,
			  ulong arg)
{
  if (CLKSPEED_DEBUG_VERBOSE)
    printk(CLKSPEED_NAME ": clkspeed_ioctl: command=%d, arg=%lu\n", command, arg);

  /* dispatch based on command */
  switch (command) {
    case CLKSPEED_SET_CLOCK: {
      return(clkspeed_change(arg, 0));
    }
    case CLKSPEED_GET_CLOCK: {
      return(0x0000001f & *(int *)PPCR_V);
    }
    case CLKSPEED_ENABLE_SWITCHING: {
      enable_switching();
      return(0);
    }
    case CLKSPEED_DISABLE_SWITCHING: {
      disable_switching();
      return(0);
    }
    case CLKSPEED_GET_SWITCHING: {
      return(clock_switching_enabled);
    }
    case CLKSPEED_SET_SPEED: {
      int switching = _CLKSPEED_GET_SWITCHING_PARAM(arg);
      int suspend = _CLKSPEED_GET_SUSPEND_PARAM(arg);
      int speed = _CLKSPEED_GET_BASE_PARAM(arg);
      if(switching!=clock_switching_enabled) {
	(switching?enable_switching():disable_switching());
      }
      return(clkspeed_change(speed, suspend));
    }
    case CLKSPEED_PRINT_PARAMS: {
      printk("dram: mdcnfg %08x\nmdcas[0-2] %08x %08x %08x\n", 
	     inl(MDCNFG_V), inl(MDCAS0_V), inl(MDCAS1_V), inl(MDCAS2_V));
      printk("sram: msc1 %08x msc0 %08x\n", inl(MSC1_V), inl(MSC0_V));
      printk("clock switching is %s\n", (clock_switching_enabled?"enabled":"disabled"));
      return(0x0000001f & *(int *)PPCR_V);
    }
    case CLKSPEED_TIME_IT: {
      int i;
      int bt, at;
      int suspend = _CLKSPEED_GET_SUSPEND_PARAM(arg);
      int iters = _CLKSPEED_GET_BASE_PARAM(arg);
      printk("arg %lx: suspend %d iters %d\n", arg, suspend, iters);
      clkspeed_change(10, suspend);  /* start at fastest speed */
      bt = inl(RCNR_V);
      for(i=0; i<iters; i++) {
	clkspeed_change(0, suspend);
	clkspeed_change(10, suspend);
      }
      at = inl(RCNR_V);
      printk("%d x two switches: %d sec\n", iters, at-bt);
      printk("%d: %d\n", iters, (at-bt)/iters);
      return(0);
    }
  }

  /* invalid command */
  return(-EINVAL);
}

static int clkspeed_open(struct inode *inode,struct file *file)
{
#ifdef MODULE
        MOD_INC_USE_COUNT;
#endif
        return 0;
}

static void clkspeed_close(struct inode *inode,struct file *file)
{
#ifdef MODULE
        MOD_DEC_USE_COUNT;
#endif
}

static struct file_operations clkspeed_fops =
{
  NULL,			/* no special lseek */
  NULL,			/* no special read */
  NULL,			/* no special write */
  NULL,			/* no special readdir */
  NULL,			/* no special select */
  clkspeed_ioctl,
  NULL,			/* no special mmap */
  clkspeed_open,
  clkspeed_close,
  NULL,			/* no special fsync */
  NULL,			/* no special fasync */
  NULL,			/* no special check_media_change */
  NULL			/* no special revalidate */
};

static struct miscdevice clkspeed_misc = {
        SA1100_CLKSPEED_MINOR, "clkspeed", &clkspeed_fops, NULL, NULL
};

int clkspeed_sa1100_init(void)
{
  if (CLKSPEED_DEBUG_VERBOSE)
    printk(CLKSPEED_NAME ": clkspeed_sa1100_init\n");

  if (((params_version & 0xffff0000) != 0xbabe0000) ||
      ((params_version & 0x0000ffff) == 0)) {
    printk(CLKSPEED_NAME ": wrong version (%08d) of the parameters file, can't init\n",
	   params_version);
    return(params_version);
  }

  /* register device */
  misc_register(&clkspeed_misc);

#ifdef	CONFIG_PROC_FS
  /* register procfs devices */
  proc_register_dynamic(&proc_root, &clkspeed_proc_entry);
#endif	/* CONFIG_PROC_FS */

  /* log device registration */
  printk(CLKSPEED_NAME_VERBOSE " initialized\n");
  if (CLKSPEED_DEBUG_VERBOSE)
    printk(CLKSPEED_NAME ": %s\n", CLKSPEED_VERSION_STRING);

  /* everything OK */
  return(0);
}

#ifdef	MODULE
int init_module(void)
{
  if (CLKSPEED_DEBUG_VERBOSE)
    printk(CLKSPEED_NAME ": init_module\n");

  return(clkspeed_sa1100_init());
}

void cleanup_module(void)
{
  int err;
#ifdef	CONFIG_PROC_FS
  /* unregister procfs entries */
  if ((err = proc_unregister(&proc_root, clkspeed_proc_entry.low_ino)) != 0)
    if (CLKSPEED_DEBUG_VERBOSE)
      printk(CLKSPEED_NAME ": cleanup_module: proc_unregister(clkspeed) %d\n", err);
#endif	/* CONFIG_PROC_FS */

  /* unregister driver */
  misc_deregister(&clkspeed_misc);

  /* log device unload */
  printk(CLKSPEED_NAME_VERBOSE " unloaded\n");
}
#endif	/* MODULE */


static int static_memory_setup(int newspeed)
{
  unsigned int msc0_new, msc0_old, msc1_new, msc1_old; 
  int config; 
  int i=0;
  unsigned int mscs[4];

  msc0_old = inl(MSC0_V);
  msc1_old = inl(MSC1_V); 
	
  mscs[0] = msc0_old & 0x0000ffff;
  mscs[1] = msc0_old >> 16;
  mscs[2] = msc1_old & 0x0000ffff;
  mscs[3] = msc1_old >> 16;

  for(i=0; i<4; i++) {
    /* if there is a static memory for this bank */
    if (staticmem_info_pointers[i]) {

      /* is it 16 or 32 bits wide? */
      if ((mscs[i] & MSC_RBW) == MSC_32BitStMem) {
	config = ((volatile int *) io_p2v (staticmem_info_pointers[i]))[newspeed/2];
	if (newspeed % 2) config = config & MSC_BANK_MASK; 
	else config = config >> 16; 
	mscs[i] = config;
      }
      else {
	config = ((volatile short *) io_p2v (staticmem_info_pointers[i]))[newspeed];
	mscs[i] = config;
      }

    }
    else {
      mscs[i] = MSC_INVALID;
    }
  }

  msc0_new = (mscs[1] << 16) | mscs[0];
  msc1_new = (mscs[3] << 16) | mscs[2];

  if (CLKSPEED_DEBUG_VERBOSE)
    printk("msc0_new: %x msc1_new: %x \n", msc0_new, msc1_new);

  outl(msc0_new, MSC0_V);
  outl(msc1_new, MSC1_V);
  return 0;
}



/*
 * procfs operations
 *
 */

#ifdef	CONFIG_PROC_FS
static int clkspeed_get_info(char *buf, 
			  char **start,
			  off_t offset,
			  int length,
			  int unused)
{
  int len;

  /* format device stats */
  len = sprintf(buf, 
		"clock: %2d\n"
		"switching is: %s\n"
		"dram: \n"
		"    mdcnfg:     %08x\n"
		"    mdcas[0-2]: %08x %08x %08x\n"
		"sram: \n"
		"    msc0:       %08x\n"
		"    msc1:       %08x\n",
		(0x0000001f & *(int *)PPCR_V),
		(clock_switching_enabled?"enabled":"disabled"),
		inl(MDCNFG_V), 
		inl(MDCAS0_V), inl(MDCAS1_V), inl(MDCAS2_V),
		inl(MSC0_V),
		inl(MSC1_V)
		);
  return(len);  
}

#endif	/* CONFIG_PROC_FS */
