/*
 * Internal .h file to ts.c (Touchscreen driver)
 *
 * Copyright (c) Compaq Computer Corporation, 1998
 *
 * Author: Larry Brakmo.
 *
 * $Log: ts_i.h,v $
 * Revision 2.1  1999/12/23  01:28:04  kerr
 * Updated Copyright information for entire source tree.
 *
 * Revision 2.0  1998/06/24  21:43:46  kramer
 * Major Bump: Move all version numbers up to 2.0 for external release.
 * -drew
 *
 * Revision 1.2  1998/06/16  19:19:45  kramer
 * Added copyright information
 *
 * Revision 1.1  1998/03/06  19:14:44  brakmo
 * New Touchscreen driver and modifications to the console.
 *
 *
 */

#ifndef ts_i_h
#define ts_i_h

#include <linux/config.h>


#ifdef MODULE
#include <linux/module.h>
#include <linux/version.h>
#else
#define MOD_INC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif

#include <linux/types.h>
#include <linux/fs.h>
#include <linux/mm.h>    /* for verify_area */
#include <linux/errno.h> /* for -EBUSY */
#include <asm/segment.h> /* for put_user_byte */

#include <asm/arch/sa1100.h>

#include <linux/kernel.h>
#include <asm/io.h>
#include <asm/io_reg.h>
#include <asm/proc/page.h>
/* #include <linux/mm.h> */
/* #include <asm/segment.h> */
#include <asm/proc/pgtable.h>

#include <asm/arch/mmap.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <asm/irq.h>
#include <asm/hardware.h>
/* #include <asm/io.h> */
/* #include <asm/arch/io_reg.h> */
#include <asm/arch/ostimers.h>

#define VirtAdd(PhAdd) (io_p2v(PhAdd))
#include <asm/arch/Itsy.h>

#define TS_MAJOR 44
#define TS_NAME  "ts"


/* Register 9 */
#define TSMX_POW	0x1
#define TSPX_POW	0x2
#define TSMY_POW	0x4
#define TSPY_POW	0x8
#define TSMX_GND	0x10
#define TSPX_GND	0x20
#define TSMY_GND	0x40
#define TSPY_GND	0x80
#define TSC_MODE_INT	0x000
#define TSC_MODE_PRES	0x100
#define TSC_MODE_POS	0x200
#define TSC_BIAS_ENA	0x800
#define TSPX_LOW	0x1000
#define TSMX_LOW	0x2000

/* Register 10 */
#define ADC_SYNC_ENA	0x1
#define ADC_INPUT_TSPX	0x0
#define ADC_INPUT_TSMX	0x4
#define ADC_INPUT_TSPY	0x8
#define ADC_INPUT_TSMY	0xc
#define ADC_INPUT_AD0	0x10
#define ADC_INPUT_AD1	0x14
#define ADC_INPUT_AD2	0x18
#define ADC_INPUT_AD3	0x1c
#define EXT_REF_ENA	0x20
#define ADC_START	0x80
#define ADC_ENA		0x8000

/* Register 11 */
#define ADC_DAT_VAL	0x8000
#define GET_ADC_DATA(x)	(((x)>>5)&0x3ff)

#define ADC_ENA_TYPE (ADC_SYNC_ENA | ADC_ENA)   
/* #define ADC_ENA_TYPE (ADC_ENA)   */

#define DO_DELAY()						\
{								\
   state++;							\
   reset_timer1(ts_stabilize_delay);				\
}

#define SET_NO_POWER()						\
{								\
   mcpCsrWr(9, 0);						\
   mcpCsrWr(10, 0);						\
}

#define SET_PEN_DOWN_INTER()					\
{								\
    mcpCsrWr(10, 0); 						\
    mcpCsrWr(9, (TSPX_POW | TSMX_POW | TSPY_GND | TSMY_GND | 	\
                 TSC_MODE_INT)); 				\
}

#define SET_READ_PRESSURE() 					\
{								\
   mcpCsrWr(10, (Reg10=ADC_ENA_TYPE)); 				\
   mcpCsrWr(9, (TSPX_POW | TSMX_POW | TSPY_GND | TSMY_GND | 	\
		TSC_MODE_PRES | TSC_BIAS_ENA)); 		\
}
				
#define SET_READ_RESISTANCE_X() 				\
{								\
   mcpCsrWr(10, (Reg10=ADC_ENA_TYPE)); 				\
   mcpCsrWr(9, (TSPX_POW | TSMX_GND |				\
		TSC_MODE_PRES | TSC_BIAS_ENA)); 		\
}
				
#define SET_READ_RESISTANCE_Y() 				\
{								\
   mcpCsrWr(10, (Reg10=ADC_ENA_TYPE)); 				\
   mcpCsrWr(9, (TSPY_POW | TSMY_GND |				\
		TSC_MODE_PRES | TSC_BIAS_ENA)); 		\
}

#define SET_READ_X_POS(tsy) 					\
{								\
  mcpCsrWr(10, Reg10=(ADC_ENA_TYPE | tsy)); 			\
  mcpCsrWr(9, (TSPX_POW | TSMX_GND |				\
	       TSC_MODE_POS | TSC_BIAS_ENA));			\
}

#define SET_READ_Y_POS(tsx) 					\
{								\
  mcpCsrWr(10, Reg10=(ADC_ENA_TYPE | tsx)); 			\
  mcpCsrWr(9, (TSPY_POW | TSMY_GND |				\
	       TSC_MODE_POS | TSC_BIAS_ENA));			\
}

#define SET_READ_AD(ad) 					\
{								\
  mcpCsrWr(10, Reg10=(ADC_ENA_TYPE | ad)); 			\
}



#endif
