#ifndef ACORNSCSI_H
#define ACORNSCSI_H

#ifndef ASM
extern int acornscsi_detect (Scsi_Host_Template *);
extern const char *acornscsi_info (struct Scsi_Host *);
extern int acornscsi_queuecmd (Scsi_Cmnd *, void (*done)(Scsi_Cmnd *));
extern int acornscsi_abort (Scsi_Cmnd *);
extern int acornscsi_reset (Scsi_Cmnd *, unsigned int);
extern int acornscsi_proc_info (char *, char **, off_t, int, int, int);
extern int acornscsi_biosparam (Disk *, kdev_t, int []);

#ifndef NULL
#define NULL 0
#endif

#ifndef CMD_PER_LUN
#define CMD_PER_LUN 2
#endif

#ifndef CAN_QUEUE
#define CAN_QUEUE 16
#endif

#ifndef PROC_SCSI_AKA30
#include "linux/proc_fs.h"
#define PROC_SCSI_AKA30 PROC_SCSI_EATA
#endif

#include <scsi/scsicam.h>

#define ACORNSCSI_3 {						\
	NULL,			/* Ptr for modules */		\
	NULL,			/* usage count for modules */	\
	NULL,							\
	acornscsi_proc_info,					\
	"Acorn 16-bit SCSI",					\
	acornscsi_detect,					\
	NULL, 			/* Release */			\
	acornscsi_info,						\
	NULL,							\
	acornscsi_queuecmd,					\
	acornscsi_abort,					\
	acornscsi_reset,					\
	NULL,							\
	scsicam_bios_param,					\
	CAN_QUEUE,		/* can_queue */			\
	7,			/* this id */			\
	SG_ALL,			/* sg_tablesize */		\
	CMD_PER_LUN,		/* cmd_per_lun */		\
	0,			/* number of boards */		\
	0,			/* unchecked isa dma */		\
	DISABLE_CLUSTERING					\
	}

#ifndef HOSTS_C

/* SBIC registers */
#define OWNID			0
#define OWNID_FS1		(1<<7)
#define OWNID_FS2		(1<<6)
#define OWNID_EHP		(1<<4)
#define OWNID_EAF		(1<<3)

#define CTRL			1
#define CTRL_DMAMODE		(1<<7)
#define CTRL_DMADBAMODE		(1<<6)
#define CTRL_DMABURST		(1<<5)
#define CTRL_DMAPOLLED		0
#define CTRL_HHP		(1<<4)
#define CTRL_EDI		(1<<3)
#define CTRL_IDI		(1<<2)
#define CTRL_HA			(1<<1)
#define CTRL_HSP		(1<<0)

#define TIMEOUT			2
#define TOTSECTS		3
#define TOTHEADS		4
#define TOTCYLH			5
#define TOTCYLL			6
#define LOGADDRH		7
#define LOGADDRM2		8
#define LOGADDRM1		9
#define LOGADDRL		10
#define SECTORNUM		11
#define HEADNUM			12
#define CYLH			13
#define CYLL			14
#define TARGETLUN		15
#define TARGETLUN_TLV		(1<<7)
#define TARGETLUN_DOK		(1<<6)

#define CMNDPHASE		16
#define SYNCHTRANSFER		17
#define SYNCHTRANSFER_OF0	0x00
#define SYNCHTRANSFER_OF1	0x01
#define SYNCHTRANSFER_OF2	0x02
#define SYNCHTRANSFER_OF3	0x03
#define SYNCHTRANSFER_OF4	0x04
#define SYNCHTRANSFER_OF5	0x05
#define SYNCHTRANSFER_OF6	0x06
#define SYNCHTRANSFER_OF7	0x07
#define SYNCHTRANSFER_OF8	0x08
#define SYNCHTRANSFER_OF9	0x09
#define SYNCHTRANSFER_OF10	0x0A
#define SYNCHTRANSFER_OF11	0x0B
#define SYNCHTRANSFER_OF12	0x0C
#define SYNCHTRANSFER_8DBA	0x00
#define SYNCHTRANSFER_2DBA	0x20
#define SYNCHTRANSFER_3DBA	0x30
#define SYNCHTRANSFER_4DBA	0x40
#define SYNCHTRANSFER_5DBA	0x50
#define SYNCHTRANSFER_6DBA	0x60
#define SYNCHTRANSFER_7DBA	0x70

#define TRANSCNTH		18
#define TRANSCNTM		19
#define TRANSCNTL		20
#define DESTID			21
#define DESTID_SCC		(1<<7)
#define DESTID_DPD		(1<<6)

#define SOURCEID		22
#define SOURCEID_ER		(1<<7)
#define SOURCEID_ES		(1<<6)
#define SOURCEID_DSP		(1<<5)
#define SOURCEID_SIV		(1<<4)

#define SSR			23
#define CMND			24
#define CMND_RESET		0x00
#define CMND_ABORT		0x01
#define CMND_ASSERTATN		0x02
#define CMND_NEGATEACK		0x03
#define CMND_DISCONNECT		0x04
#define CMND_RESELECT		0x05
#define CMND_SELWITHATN		0x06
#define CMND_SELECT		0x07
#define CMND_SELECTATNTRANSFER	0x08
#define CMND_SELECTTRANSFER	0x09
#define CMND_RESELECTRXDATA	0x0A
#define CMND_RESELECTTXDATA	0x0B
#define CMND_WAITFORSELRECV	0x0C
#define CMND_SENDSTATCMD	0x0D
#define CMND_SENDDISCONNECT	0x0E
#define CMND_SETIDI		0x0F
#define CMND_RECEIVECMD		0x10
#define CMND_RECEIVEDTA		0x11
#define CMND_RECEIVEMSG		0x12
#define CMND_RECEIVEUSP		0x13
#define CMND_SENDCMD		0x14
#define CMND_SENDDATA		0x15
#define CMND_SENDMSG		0x16
#define CMND_SENDUSP		0x17
#define CMND_TRANSLATEADDR	0x18
#define CMND_XFERINFO		0x20
#define CMND_SBT		(1<<7)

#define DATA			25
#define ASR			26
#define ASR_INT			(1<<7)
#define ASR_LCI			(1<<6)
#define ASR_BSY			(1<<5)
#define ASR_CIP			(1<<4)
#define ASR_PE			(1<<1)
#define ASR_DBR			(1<<0)

/* DMAC registers */
#define INIT			0x00
#define INIT_8BIT		(1)

#define CHANNEL			0x80
#define CHANNEL_0		0x00
#define CHANNEL_1		0x01
#define CHANNEL_2		0x02
#define CHANNEL_3		0x03

#define TXCNTLO			0x01
#define TXCNTHI			0x81
#define TXADRLO			0x02
#define TXADRMD			0x82
#define TXADRHI			0x03

#define DEVCON0			0x04
#define DEVCON0_AKL		(1<<7)
#define DEVCON0_RQL		(1<<6)
#define DEVCON0_EXW		(1<<5)
#define DEVCON0_ROT		(1<<4)
#define DEVCON0_CMP		(1<<3)
#define DEVCON0_DDMA		(1<<2)
#define DEVCON0_AHLD		(1<<1)
#define DEVCON0_MTM		(1<<0)

#define DEVCON1			0x84
#define DEVCON1_WEV		(1<<1)
#define DEVCON1_BHLD		(1<<0)

#define MODECON			0x05
#define MODECON_WOED		0x01
#define MODECON_VERIFY		0x00
#define MODECON_READ		0x04
#define MODECON_WRITE		0x08
#define MODECON_AUTOINIT	0x10
#define MODECON_ADDRDIR		0x20
#define MODECON_DEMAND		0x00
#define MODECON_SINGLE		0x40
#define MODECON_BLOCK		0x80
#define MODECON_CASCADE		0xC0

#define STATUS			0x85
#define TEMPLO			0x06
#define TEMPHI			0x86
#define REQREG			0x07
#define MASKREG			0x87
#define MASKREG_M0		0x01
#define MASKREG_M1		0x02
#define MASKREG_M2		0x04
#define MASKREG_M3		0x08

/* miscellaneous internal variables */

#define CMD_READ		0
#define CMD_WRITE		1
#define CMD_MISC		2

#define PHASE_IDLE			0x00
#define PHASE_CONNECTING		0x11
#define PHASE_CONNECTED			0x12
#define PHASE_COMMANDSENT		0x13

#define PHASE_DATAIN			0x26
#define PHASE_DATAOUT			0x27

#define PHASE_STATUSIN			0x3c
#define PHASE_FINISHED			0x4d
#define PHASE_ABORTED			0x4e
#define PHASE_DISCONNECTING		0x4f
#define PHASE_RECONNECTED		0x50

#define PHASE_CONNECTING_FOR_ABORT	0x80
#define PHASE_CONNECTED_FOR_ABORT	0x81

#define INTR_CONNECTED		0
#define INTR_NOTCONNECTED	1
#define INTR_CONTINUE		2

#define POD_SPACE(x)	((x) + 0xd0000)
#define SCpdirection	SCp.phase
#define MASK_ON		(MASKREG_M3|MASKREG_M2|MASKREG_M1|MASKREG_M0)
#define MASK_OFF	(MASKREG_M3|MASKREG_M2|MASKREG_M1)

#define RESET_STATUS(q,x)	({ status_ptr[q] = 0; })
#define ADD_STATUS(q,x)	({ status_array[q][status_ptr[q]] = (x); status_ptr[q] = (status_ptr[q] + 1) & 63; })

#define min(x,y) ((x) < (y) ? (x) : (y))
#define max(x,y) ((x) < (y) ? (y) : (x))

struct as_stats {
    unsigned long queues;
    unsigned long removes;
    unsigned long fins;
    unsigned long reads;
    unsigned long writes;
    unsigned long miscs;
    unsigned long aborts;
    unsigned long resets;
    unsigned long disconnects;
};

struct as_cmnd {
    Scsi_Cmnd *cmd;
    struct as_cmnd *next;			/* next command in queue		 */
    int used;					/* block in use				 */
    unsigned long origlen;			/* origional length			 */
    unsigned long sbiclen;			/* sbic length to transfer		 */
    unsigned long dmaclen;			/* dmac length to transfer		 */
#ifdef FIX_64K
    unsigned long transferred;			/* bytes transferred			 */
#endif
    unsigned int buffer_initialised	:1,	/* buffer is initialised		 */
		 buffer_no		:1,	/* buffer number			 */
		 saved			:1;	/* if saved info			 */
    unsigned short bufaddr[2];
    unsigned short buflen[2];
    unsigned short sdmacaddress;		/* DMAC address				 */
    unsigned short sdmaclen;			/* DMAC length				 */
    unsigned char  scmdptr;			/* command pointer			 */
};

struct as_dev {
    struct as_cmnd *issue_head;
    struct as_cmnd *issue_tail;
    struct as_cmnd *disconnected;		/* commands disconnected		 */
    unsigned char busy;				/* luns on device that are busy		 */
    unsigned char sync_xfer;			/* SYNC transfer register value		 */
    unsigned char sync_stat;			/* SYNC transfer status			 */
    unsigned char disconnect_ok;		/* allow disconnects			 */
#define SDTR_UNSET	2
#define SDTR_SENT	1
#define SDTR_DONE	0
};

typedef struct as_msg {
    struct as_msg *next;
    unsigned char used;
    unsigned char len;
    unsigned char vals[5];
} AS_Msg;
    
typedef struct acornscsi_hostdata {
    int busy;					/* in process of connecting		 */
    unsigned long tmpsbiclen;			/* temporary data			 */
    struct as_cmnd *connecting;			/* command that is connecting		 */
    struct as_cmnd *aborting;			/* command that is aborting		 */
    struct as_cmnd *connected;			/* command that is connected		 */
    struct as_cmnd queuedcommands[CAN_QUEUE];	/* commands 				 */
    struct as_dev dev[8];

    unsigned int  last_message;			/* last message sent			 */
    unsigned char page_reg;			/* current page register value		 */
    unsigned char phase;			/* phase of command			 */
    unsigned char cmdptr;			/* current command pointer		 */
    unsigned char in_msgptr;			/* message received length		 */
    unsigned char in_msg[8];			/* message received			 */
    AS_Msg *out_msg;

    struct as_stats stats;			/* stats				 */
} AS_Host;

#endif

#endif /* ndef ASM */
#endif /* ACORNSCSI_H */
