/*
 * linux/arch/arm/kernel/dma.c
 *
 * Copyright (C) 1995, 1996 Russell King
 */

#include <linux/config.h>
#include <linux/sched.h>
#include <linux/malloc.h>
#include <linux/mman.h>

#include <asm/page.h>
#include <asm/pgtable.h>
#include <asm/irq.h>
#include <asm/hardware.h>
#include <asm/io.h>
#define KERNEL_ARCH_DMA
#include <asm/dma.h>
#ifdef CONFIG_ARCH_BRUTUS
#include <asm/arch/mmap.h>
#endif

#ifndef CONFIG_ARCH_BRUTUS
static unsigned long dma_address[8];
static unsigned long dma_count[8];
static char dma_direction[8] = { -1, -1, -1, -1, -1, -1, -1};
#endif

#if defined(CONFIG_ARCH_A5K) || defined(CONFIG_ARCH_RPC)
#define DMA_PCIO
#endif
#if defined(CONFIG_ARCH_ARC) && defined(CONFIG_BLK_DEV_FD)
#define DMA_OLD
#endif

#ifdef CONFIG_ARCH_BRUTUS
/* SA1100 DMA Register Definitions 
 * There are 6 sets of identical registers, 
 * 1 for each channel. Use the DMA_REG macro 
 * to address a particular register
 */
#define DMA_CHANNELS 6

#define DMA_CH_0  (0xB0000000)
#define DMA_CH_1  (0xB0000020)
#define DMA_CH_2  (0xB0000040)
#define DMA_CH_3  (0xB0000060)
#define DMA_CH_4  (0xB0000080)
#define DMA_CH_5  (0xB00000A0)

volatile unsigned int dma_channel_ba[6];/* = {DMA_CH_0, DMA_CH_1, DMA_CH_2,
				      DMA_CH_3, DMA_CH_4, DMA_CH_5 };*/

unsigned int dma_channel_buffer[6] = {0, 0, 0, 0, 0, 0};
    
#define DDAR      0x0           /* address register */
#define DCSR_SET  0x04          /* control/status */
#define DCSR_CLR  0x08
#define DCSR_RD   0x0C

#define DCSR_RUN   (0x01 << 0)
#define DCSR_IE    (0x01 << 1)
#define DCSR_ERROR (0x01 << 2)
#define DCSR_DONEA (0x01 << 3)
#define DCSR_STRTA (0x01 << 4)
#define DCSR_DONEB (0x01 << 5)
#define DCSR_STRTB (0x01 << 6)
#define DCSR_BIU   (0x01 << 7)

#define DBSA      0x10        /* buffer A start address */
#define DBTA      0x14        /* buffer A transfer count */
#define DBSB      0x18        /* buffer B base address */
#define DBTB      0x1C        /* buffer B transfer count */

#define DMA_REG(base, offset) ( *((volatile unsigned int *) ((base) + (offset))) )

#endif

#ifdef CONFIG_ARCH_BRUTUS
/* 
 * disables a DMA channel, If a DMA is in progress the channel will be
 * paused which may lead to strange things when you
 * go to re-enable it, so try and be sure that no DMAs are in progress
 * when you disable a channel.
 */
void __inline__
disable_dma(unsigned int dmanr)
{
    if ((dmanr >=0)  && (dmanr < DMA_CHANNELS))
    {
	DMA_REG(dma_channel_ba[dmanr], DCSR_CLR) = 0x7F; /*DCSR_RUN | DCSR_IE | 
	    DCSR_DONEA | DCSR_DONEB;*/
/*	printk("%08x\n", DMA_REG2(dma_channel_ba[dmanr], DCSR_CLR));
	
	printk("disable_dma status 0x%02x\n", 
	       DMA_REG(dma_channel_ba[dmanr], DCSR_RD));
	*/
    }
    else
    {
	printk ("disable_dma: dma %d not supported\n", dmanr);
    }
}
#endif

void __inline__
enable_dma (unsigned int dmanr)
{
#ifdef CONFIG_ARCH_BRUTUS
    if ((dmanr >=0)  && (dmanr < DMA_CHANNELS))
    {
	if (DMA_REG(dma_channel_ba[dmanr], DCSR_RD) & DCSR_BIU)
	{
	    /* start buffer b */
	    DMA_REG(dma_channel_ba[dmanr], DCSR_SET) = DCSR_RUN | DCSR_STRTB;
	    dma_channel_buffer[dmanr] = 1;
	}
	else
	{
	    /* start buffer a */
	    DMA_REG(dma_channel_ba[dmanr], DCSR_SET) = DCSR_RUN | DCSR_STRTA;
	    dma_channel_buffer[dmanr] = 0;
	}
    }
    else
    {
	printk ("enable_dma: dma %d not supported\n", dmanr);
    }
#else
	switch (dmanr) {
#ifdef DMA_PCIO
		case 2: {
			void *fiqhandler_start;
			unsigned int fiqhandler_length;
			extern void floppy_fiqsetup (unsigned long len, unsigned long addr,
					unsigned long port);
		    	switch (dma_direction[dmanr]) {
	    		case 1: {
				extern unsigned char floppy_fiqin_start, floppy_fiqin_end;
				fiqhandler_start = &floppy_fiqin_start;
				fiqhandler_length = &floppy_fiqin_end - &floppy_fiqin_start;
				break;
			}
			case 0: {
				extern unsigned char floppy_fiqout_start, floppy_fiqout_end;
				fiqhandler_start = &floppy_fiqout_start;
				fiqhandler_length = &floppy_fiqout_end - &floppy_fiqout_start;
				break;
			}
		    	default:
				printk ("enable_dma: dma%d not initialised\n", dmanr);
				return;
			}
			memcpy ((void *)0x1c, fiqhandler_start, fiqhandler_length);
			flush_page_to_ram(0);
			floppy_fiqsetup (dma_count[dmanr], dma_address[dmanr], (int)PCIO_FLOPPYDMABASE);
			enable_irq (64);
			return;
		}
#endif
#ifdef DMA_OLD
		case 0: { /* Data DMA */
			switch (dma_direction[dmanr]) {
			case 1: /* read */
	    	    	{
				extern unsigned char fdc1772_dma_read, fdc1772_dma_read_end;
				extern void fdc1772_setupdma(unsigned int count,unsigned int addr);
				unsigned long flags;
#ifdef DEBUG
				printk("enable_dma fdc1772 data read\n");
#endif
				save_flags(flags);
				cliIF();
			
				memcpy ((void *)0x1c, (void *)&fdc1772_dma_read,
					&fdc1772_dma_read_end - &fdc1772_dma_read);
				fdc1772_setupdma(dma_count[dmanr],dma_address[dmanr]); /* Sets data pointer up */
				enable_irq (64);
				restore_flags(flags);
			}
			break;

			case 0: /* write */
		        {
				extern unsigned char fdc1772_dma_write, fdc1772_dma_write_end;
				extern void fdc1772_setupdma(unsigned int count,unsigned int addr);
				unsigned long flags;

#ifdef DEBUG
				printk("enable_dma fdc1772 data write\n");
#endif
				save_flags(flags);
				cliIF();
				memcpy ((void *)0x1c, (void *)&fdc1772_dma_write,
					&fdc1772_dma_write_end - &fdc1772_dma_write);
				fdc1772_setupdma(dma_count[dmanr],dma_address[dmanr]); /* Sets data pointer up */
				enable_irq (64);

				restore_flags(flags);
			}
			break;
	    		default:
				printk ("enable_dma: dma%d not initialised\n", dmanr);
				return;
			}
		}
		break;

		case 1: { /* Command end FIQ - actually just sets a flag */
			/* Need to build a branch at the FIQ address */
			extern void fdc1772_comendhandler(void);
			unsigned long flags;

			/*printk("enable_dma fdc1772 command end FIQ\n");*/
			save_flags(flags);
			cliIF();
	
			*((unsigned int *)0x1c)=0xea000000 | (((unsigned int)fdc1772_comendhandler-(0x1c+8))/4); /* B fdc1772_comendhandler */

			restore_flags(flags);
		}
		break;
#endif
		case DMA_0:
		case DMA_1:
		case DMA_2:
		case DMA_3:
		case DMA_S0:
		case DMA_S1:
			arch_enable_dma (dmanr - DMA_0);
			break;

		default:
			printk ("enable_dma: dma %d not supported\n", dmanr);
	}
#endif /* CONFIG_ARCH_BRUTUS */
}

#ifdef CONFIG_ARCH_BRUTUS
/*
 * set the sa1100 DMA controller address register.
 *
 * dmanr  : the DMA channel to set.
 * deviec : the device address to set.
 */
void __inline__
set_dma_mode_sa1100 (unsigned int dmanr, unsigned int device)
{ 
    /* initialise the register addresses. */
    dma_channel_ba[0] = io_p2v(DMA_CH_0);
    dma_channel_ba[1] = io_p2v(DMA_CH_1);
    dma_channel_ba[2] = io_p2v(DMA_CH_2);
    dma_channel_ba[3] = io_p2v(DMA_CH_3);
    dma_channel_ba[4] = io_p2v(DMA_CH_4);
    dma_channel_ba[5] = io_p2v(DMA_CH_5);

    if ((dmanr >=0)  && (dmanr < DMA_CHANNELS))
    {
	DMA_REG(dma_channel_ba[dmanr], DDAR) = device;
    }
    else
    {
	printk ("set_dma_mode_sa1100: dma %d not supported\n", dmanr);
    }
}

/*
 * return the start address for the last used buffer on the 
 * given dma channel. This function relies on the dma_channel_buffer
 * state variable because it is impossible to know at this stage 
 * which buffer was last used. If the DMA finished the BIU says the 
 * opposite buffer, if the DMA only partially completed the BIU
 * indicates the same buffer.
 *
 * dmanr : dma channel.
 */
unsigned int __inline__
get_dma_address_sa1100(unsigned int dmanr)
{
    unsigned int addr = 0;
    if ((dmanr >=0)  && (dmanr < DMA_CHANNELS))
    {
	if (dma_channel_buffer[dmanr])
	{
	    /* read buffer b */
	    addr = DMA_REG(dma_channel_ba[dmanr], DBSB);
	}
	else
	{
	    /* read buffer a */
	    addr = DMA_REG(dma_channel_ba[dmanr], DBSA);
	}
    }
    else
    {
	printk ("get_dma_address_sa1100: dma %d not supported\n", dmanr);
    }
    return addr;
}

#else

void set_dma_mode (unsigned int dmanr, char mode)
{
	if (dmanr < 8) {
		if (mode == DMA_MODE_READ)
			dma_direction[dmanr] = 1;
		else if (mode == DMA_MODE_WRITE)
			dma_direction[dmanr] = 0;
		else
			printk ("set_dma_mode: dma%d: invalid mode %02X not supported\n",
				dmanr, mode);
	} else if (dmanr < MAX_DMA_CHANNELS)
		arch_set_dma_mode (dmanr - DMA_0, mode);
	else
		printk ("set_dma_mode: dma %d not supported\n", dmanr);
}
#endif

/* 
 * Set the physical address of memory to transfer to/from.
 */
void __inline__
set_dma_addr (unsigned int dmanr, unsigned int addr)
{ 
#ifdef CONFIG_ARCH_BRUTUS
    /* We must look at which buffer is going to be used next 
     * before setting the address.
     */
    if ((dmanr >=0)  && (dmanr < DMA_CHANNELS))
    {
	if (DMA_REG(dma_channel_ba[dmanr], DCSR_RD) & DCSR_BIU)
	{
	    /* use buffer b */
	    DMA_REG(dma_channel_ba[dmanr], DBSB) = addr;
	}
	else
	{ 
	    /* use buffer a */
	    DMA_REG(dma_channel_ba[dmanr], DBSA) = addr;
	}
    }
    else
    {
	printk ("set_dma_addr: dma %d not supported\n", dmanr);
    }
#else
	if (dmanr < 8)
		dma_address[dmanr] = (unsigned long)addr;
	else if (dmanr < MAX_DMA_CHANNELS)
		arch_set_dma_addr (dmanr - DMA_0, addr);
	else
		printk ("set_dma_addr: dma %d not supported\n", dmanr);
#endif
}

void __inline__
set_dma_count (unsigned int dmanr, unsigned int count)
{
#ifdef CONFIG_ARCH_BRUTUS

    if ((dmanr >=0)  && (dmanr < DMA_CHANNELS))
    {
	if (DMA_REG(dma_channel_ba[dmanr], DCSR_RD) & DCSR_BIU)
	{
	    /* use buffer b */
	    DMA_REG(dma_channel_ba[dmanr], DBTB) = count;
	}
	else
	{
	    DMA_REG(dma_channel_ba[dmanr], DBTA) = count;
	}
    }
    else
    {
	printk ("set_dma_count: dma %d not supported\n", dmanr);
    }
#else
	if (dmanr < 8)
		dma_count[dmanr] = (unsigned long)count;
	else if (dmanr < MAX_DMA_CHANNELS)
		arch_set_dma_count (dmanr - DMA_0, count);
	else
	    	printk ("set_dma_count: dma %d not supported\n", dmanr);
#endif
}

int __inline__
get_dma_residue (unsigned int dmanr)
{  
#ifdef CONFIG_ARCH_BRUTUS
    /* must look at the dma_channel_buffer state variable
     * to determine which buffer was last used.
     */
    int residue = 0;

    if ((dmanr >=0)  && (dmanr < DMA_CHANNELS))
    {
	if (dma_channel_buffer[dmanr])
	{
	    /* get buffer b */
	    residue = DMA_REG(dma_channel_ba[dmanr], DBTB);
	}
	else
	{
	    /* get buffer a */
	    residue = DMA_REG(dma_channel_ba[dmanr], DBTA);
	}
    }
    else
    {
	printk ("get_dma_residue: dma %d not supported\n", dmanr);
    }
    return residue;
#else
	if (dmanr < 8) {
		switch (dmanr) {
#if defined(CONFIG_ARCH_A5K) || defined(CONFIG_ARCH_RPC)
		case 2: {
			extern int floppy_fiqresidual (void);
			return floppy_fiqresidual ();
		}
#endif
#if defined(CONFIG_ARCH_ARC) && defined(CONFIG_BLK_DEV_FD)
		case 0: {
			extern unsigned int fdc1772_bytestogo;
			return fdc1772_bytestogo;
		}
#endif
		default:
			return -1;
		}
	} else if (dmanr < MAX_DMA_CHANNELS)
		return arch_dma_count (dmanr - DMA_0);
	return -1;
#endif /* CONFIG_ARCH_BRUTUS */
}








