#define SERIAL_ECHO_PORT_P	UART_SERIAL1_BASE_P
#define SERIAL_ECHO_PORT_V	UART_SERIAL1_BASE_V
#define SERIAL_ECHO_DIVISOR	0x17	/* 9600 baud */
/* #define SERIAL_ECHO_DIVISOR	0x5	/* 38400 baud */
/* #define SERIAL_ECHO_DIVISOR	0x1	/* 115200 baud */

/*
 * linux/arch/arm/kernel/head32.S
 *
 * Copyright (C) 1994, 1995, 1996, 1997 Russell King
 *
 * Kernel 32 bit startup code for ARM6 / ARM7 / StrongARM
 */
#include <linux/config.h>
		.text
		.align

		.globl	swapper_pg_dir
		.equ	swapper_pg_dir,	0xc0004000

		.globl	_entry
		.globl	_stext
/*
 * Entry point and restart point.  Entry *must* be called with r0 == 0,
 * MMU off.
 *
 *  r1 = 0 -> ebsa  (Ram @ 0x00000000)
 *  r1 = 1 -> RPC   (Ram @ 0x10000000)
 *  r1 = 2 -> ebsit (???)
 *  r1 = 3 -> brutus (Ram @ 0xc0000000)
 */
_stext:
_entry:
	bl	serial_echo_init_SERIAL1_P

	mov	r0,  #0x61   @ a
	bl	serial_echo_print_wait_P
	
@ do not turn off interrupts (assume already off)

		mrs	r0, cpsr
		bic	r0, r0, #0x1f
		orr	r0, r0, #0x13
		msr	cpsr, r0


		mov	r0, #0  @ these should be done by bootloader
		mov	r1, #3		
	
		teq	r0, #0					@ check for illegal entry...
		bne	Lerror					@ loop indefinitely
		cmp	r1, #4					@ Unknown machine architecture
		bge	Lerror
@
@ First thing to do is to get the page tables set up so that we can call the kernel
@ in the correct place.  This is relocatable code...
@
		mrc	p15, 0, r9, c0, c0			@ get Processor ID
@
@ Read processor ID register (CP#15, CR0).
@ NOTE: ARM2 & ARM250 cause an undefined instruction exception...
@ Values are:
@		XX01XXXX = ARMv4 architecture (StrongARM)
@		XX00XXXX = ARMv3 architecture
@		4156061X = ARM 610
@ 		4156030X = ARM 3
@		4156025X = ARM 250
@		4156020X = ARM 2
@
		adr	r10, LCProcTypes
1:		ldmia	r10!, {r5, r6, r8}			@ Get Set, Mask, MMU Flags
		teq	r5, #0					@ End of list?
		beq	Lerror
		eor	r5, r5, r9
		tst	r5, r6
		addne	r10, r10, #8
		bne	1b

		adr	r4, LCMachTypes
		add	r4, r4, r1, lsl #4
		ldmia	r4, {r4, r5, r6}			@ r4 = page dir in physical ram

		mov	r0, r4
		mov	r1, #0
		add	r2, r0, #0x4000
1:		str	r1, [r0], #4				@ Clear page table
		teq	r0, r2
		bne	1b
@
@ Add enough entries to allow the kernel to be called.
@ It will sort out the real mapping in paging_init
@

@ shouldn't need this, but try...
@	PA	VA
@	c00-c01 000-001

		add	r0, r4, #0
		mov	r1, #0x0000000c				@ SECT_CACHEABLE | SECT_BUFFERABLE
		orr	r1, r1, r8
		add	r1, r1, r5
		str	r1, [r0], #4

@ now do bare minimum of entries for linux: first ram
@	PA	VA
@	c00-c01 c00-c01

		add	r0, r4, #0x3000
		mov	r1, #0x0000000c				@ SECT_CACHEABLE | SECT_BUFFERABLE
		orr	r1, r1, r8
		add	r1, r1, r5
		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		str	r1, [r0], #4
		add	r1, r1, #1 << 20
@
@ Map in IO space
@

@	PA	VA
@	800-87f	e00-e7f
@	900-97f	e80-eff
@	a00-a7f	f00-f7f
@	b00-b7f	f80-fff
	
		add	r0, r4, #0x3800
		orr	r1, r6, r8
#ifndef CONFIG_ARCH_BRUTUS
		add	r2, r0, #0x0800
#else /* CONFIG_ARCH_BRUTUS */
		add	r2, r0, #0x0200
#endif /*  CONFIG_ARCH_BRUTUS */
1:		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		teq	r0, r2
		bne	1b
#ifdef CONFIG_ARCH_BRUTUS
		add	r1, r1, #1 << 27  @ 0x8000000
		add	r2, r0, #0x0200
2:		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		teq	r0, r2
		bne	2b
		add	r1, r1, #1 << 27  @ 0x8000000
		add	r2, r0, #0x0200
3:		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		teq	r0, r2
		bne	3b
		add	r1, r1, #1 << 27  @ 0x8000000
		add	r2, r0, #0x0200
4:		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		teq	r0, r2
		bne	4b
#endif /* CONFIG_ARCH_BRUTUS */

	
@ YYY what are we doing about screen?
@@
@@ Map in screen at 0x02000000 & SCREEN2_BASE
@@
@		teq	r5, #0
@		addne	r0, r4, #0x80				@ 02000000
@		movne	r1, #0x02000000
@		orrne	r1, r1, r8
@		strne	r1, [r0]
@		addne	r0, r4, #0x3600				@ d8000000
@		strne	r1, [r0]
@
@ The following should work on both v3 and v4 implementations
@
	
		mov	lr, pc
		mov	pc, r10					@ Call processor flush (returns ctrl reg)
		adr	r5, _entry
		sub	r10, r10, r5				@ Make r10 PIC

		ldr	lr, Lbranch
		mcr	p15, 0, r0, c1, c0			@ Enable MMU & caches.  In 3 instructions
								@ we loose this page!
		mov	pc, lr

Lerror:		mov	r0, #0x02000000
		mov	r1, #0x11
		orr	r1, r1, r1, lsl #8
		orr	r1, r1, r1, lsl #16
		str	r1, [r0], #4
		str	r1, [r0], #4
		str	r1, [r0], #4
		str	r1, [r0], #4
		b	Lerror

Lbranch:	.long	Lalready_done_mmap			@ Real address of routine

		@ EBSA (pg dir phys, phys ram start, phys i/o)
LCMachTypes:	.long	swapper_pg_dir - 0xc0000000		@ Address of page tables (physical)
		.long	0					@ Address of RAM
		.long	0xe0000000				@ I/O address
		.long	0

		@ RPC
		.long	swapper_pg_dir - 0xc0000000 + 0x10000000
		.long	0x10000000
		.long	0x03000000
		.long	0

		@ EBSIT ???
		.long	swapper_pg_dir - 0xc0000000
		.long	0
		.long	0xe0000000
		.long	0

		@ BRUTUS
		.long	swapper_pg_dir - 0xc0000000 + 0xc0000000
		.long	0xc0000000
		.long	0x80000000			@ off by a half-gig
		.long	0

LCProcTypes:	@ ARM6 / 610
		.long	0x41560600
		.long	0xffffff00
		.long	0x00000c12
		b	Larmv3_flush_early			@ arm v3 flush & ctrl early setup
		mov	pc, lr

		@ ARM7 / 710
		.long	0x41007000
		.long	0xfffff000
		.long	0x00000c12
		b	Larmv3_flush_late			@ arm v3 flush & ctrl late setup
		mov	pc, lr

		@ StrongARM
		.long	0x4401a100
		.long	0xfffffff0
		.long	0x00000c02
		b	Larmv4_flush_early
		b	Lsa_fastclock

		@ BRUTUS
		.long	0x4401a110
		.long	0xfffffff0
		.long	0x00000c02
		b	Larmv4_flush_early
		b	Lsa_fastclock

		.long	0

LC0:		.long	edata
		.long	arm_id
		.long	end
		.long	init_user_stack + 4096
		.align

Larmv3_flush_early:
		mov	r0, #0
		mcr	p15, 0, r0, c7, c0			@ flush caches on v3
		mcr	p15, 0, r0, c5, c0			@ flush TLBs on v3
		mcr	p15, 0, r4, c2, c0			@ load page table pointer
		mov	r0, #5					@ Domains 0, 1 = client
		mcr	p15, 0, r0, c3, c0			@ load domain access register
		mov	r0, #0x3d				@ ....DPWC.M
		mov	pc, lr

Larmv3_flush_late:
		mov	r0, #0
		mcr	p15, 0, r0, c7, c0			@ flush caches on v3
		mcr	p15, 0, r0, c5, c0			@ flush TLBs on v3
		mcr	p15, 0, r4, c2, c0			@ load page table pointer
		mov	r0, #5					@ Domains 0, 1 = client
		mcr	p15, 0, r0, c3, c0			@ load domain access register
		mov	r0, #0x7d				@ ....DPWC.M
		mov	pc, lr

Larmv4_flush_early:
		mov	r0, #0
		mcr	p15, 0, r0, c7, c7, 0			@ flush I,D caches on v4
		mcr	p15, 0, r0, c7, c10, 4			@ drain write buffer on v4
		mcr	p15, 0, r0, c9, c0			@ flush read buffer on v4
		mcr	p15, 0, r0, c8, c7			@ flush I,D TLBs on v4
		mcr	p15, 0, r4, c2, c0			@ load page table pointer
		mov	r0, #5					@ Domains 0, 1 = client
		mcr	p15, 0, r0, c3, c0			@ load domain access register
		mrc	p15, 0, r0, c1, c0			@ get control register v4
		orr	r0, r0, #0x3d				@ ....DPWC.M
		orr	r0, r0, #1 << 12			@ v4 supports separate I cache
		mov	pc, lr

Lsa_fastclock:	mcr	p15, 0, r4, c15, c1, 2			@ Enable clock switching
		mov	pc, lr

Lalready_done_mmap:
		adr	r5, _entry				@ Add base back in
		add	r10, r10, r5

		adr	r5, LC0
		ldmia	r5, {r5, r6, r8, sp}			@ Setup stack
		mov	r4, #0
1:		cmp	r5, r8					@ Clear BSS
		strcc	r4, [r5],#4
		bcc	1b

		str	r9, [r6]				@ Save processor ID
		mov	lr, pc
		add	pc, r10, #4				@ Call post-processor init

		mov	fp, #0
		b	start_kernel

#if 0
/*
 * Useful debugging routines
 */
		.globl	printhex8
printhex8:	mov	r1, #8
		b	printhex

		.globl	printhex4
printhex4:	mov	r1, #4
		b	printhex

		.globl	printhex2
printhex2:	mov	r1, #2
printhex:	ldr	r2, =hexbuf
		add	r3, r2, r1
		mov	r1, #0
		strb	r1, [r3]
1:		and	r1, r0, #15
		mov	r0, r0, lsr #4
		cmp	r1, #10
		addlt	r1, r1, #'0'
		addge	r1, r1, #'a' - 10
		strb	r1, [r3, #-1]!
		teq	r3, r2
		bne	1b
		mov	r0, r2

		.globl	printascii
printascii:
#ifdef CONFIG_ARCH_RPC
		mov	r3, #0xe0000000
		orr	r3, r3, #0x00010000
		orr	r3, r3, #0x00000fe0
#else
		mov	r3, #0xf0000000
		orr	r3, r3, #0x0be0
#endif
		b	3f
1:		ldrb	r2, [r3, #0x18]
		tst	r2, #0x10
		beq	1b
		strb	r1, [r3]
2:		ldrb	r2, [r3, #0x14]
		and	r2, r2, #0x60
		teq	r2, #0x60
		bne	2b
		teq	r1, #'\n'
		moveq	r1, #'\r'
		beq	1b
3:		teq	r0, #0
		ldrneb	r1, [r0], #1
		teqne	r1, #0
		bne	1b
		mov	pc, lr

		.globl	printch
printch:
#ifdef CONFIG_ARCH_RPC
		mov	r3, #0xe0000000
		orr	r3, r3, #0x00010000
		orr	r3, r3, #0x00000fe0
#else
		mov	r3, #0xf0000000
		orr	r3, r3, #0x0be0
#endif
		mov	r1, r0
		mov	r0, #0
		b	1b

		.bss
hexbuf:		.space 16

#endif


#define UART_SERIAL3_BASE_P     0x80050000
#define UART_SERIAL3_BASE_V     0xe0050000
#define UART_SERIAL1_BASE_P     0x80010000
#define UART_SERIAL1_BASE_V     0xe0010000

#define UTCR0               0x00
#define UTCR1               0x04
#define UTCR2               0x08
#define UTCR3               0x0c
#define UTDR                0x14
#define UTSR0               0x1C   
#define UTSR1               0x20

#define UART_RX             UTDR
#define UART_TX             UTDR

#define UTCR3_RXE            1     /* Receiver enable */
#define UTCR3_TXE            2     /* Transmit enable */
#define UTSR1_TNF            4     /* transmit fifo non full */


#define GAFR_P          0x9004001c    /* GPIO alternative function register */
#define GPDR_P          0x90040004    /* GPIO pin direction register */
#define PPAR_P          0x90060008    /* PPAR pin assignment register */
#define GPSR_P		0x90040008    /* GPSR pin set register */
#define SDCR0_P		0x80020060    /* SDCR0 configuration register */
		
serial_echo_init_SERIAL1_P:	

	ldr	r1, =SDCR0_P
	ldr	r2, =1
	str	r2, [r1, #0]
	
	ldr     r1, =SERIAL_ECHO_PORT_P

	@ serial_echo_outl (0x08, UTCR0); /* No parity, 8 bits, 1 stop */
	ldr	r2, =8
	str	r2, [r1, #UTCR0]

	@ serial_echo_outl (SERIAL_ECHO_DIVISOR >> 8, UTCR1)
	ldr	r2, =SERIAL_ECHO_DIVISOR
	mov	r3, r2, LSR #8
	str	r3, [r1, #UTCR1]
	@ serial_echo_outl (SERIAL_ECHO_DIVISOR & 0xff, UTCR2);
	and	r3, r2, #0xff
	str	r3, [r1, #UTCR2]

	@ serial_echo_outl (0, UTCR3);            /* Disable all interrupts */
	ldr	r2, =0
	str	r2, [r1, #UTCR3]

	@ serial_echo_outl (UTCR3_TXE, UTCR3);    /* Turn on transmitter */
	ldr	r2, =UTCR3_TXE
	str	r2, [r1, #UTCR3]

	@ /* clear out any errors by writing the line status register */
	@ serial_echo_outl(0xff, UTSR0) ;
	ldr	r2, =0xff
	str	r2, [r1, #UTSR0]

	mov	pc, lr



serial_echo_init_SERIAL3_P:	

	@ outl(0x01800000, GPSR_P);   /* bits 23 and 24 = 1 */
	ldr	r1, =GPSR_P
	ldr	r2, =0x01800000
	str	r2, [r1, #0]

	ldr     r1, =SERIAL_ECHO_PORT_P

	@ serial_echo_outl (0x08, UTCR0); /* No parity, 8 bits, 1 stop */
	ldr	r2, =8
	str	r2, [r1, #UTCR0]

	@ serial_echo_outl (SERIAL_ECHO_DIVISOR >> 8, UTCR1)
	ldr	r2, =SERIAL_ECHO_DIVISOR
	mov	r3, r2, LSR #8
	str	r3, [r1, #UTCR1]
	@ serial_echo_outl (SERIAL_ECHO_DIVISOR & 0xff, UTCR2);
	and	r3, r2, #0xff
	str	r3, [r1, #UTCR2]

	@ serial_echo_outl (0, UTCR3);            /* Disable all interrupts */
	ldr	r2, =0
	str	r2, [r1, #UTCR3]

	@ serial_echo_outl (UTCR3_TXE, UTCR3);    /* Turn on transmitter */
	ldr	r2, =UTCR3_TXE
	str	r2, [r1, #UTCR3]

	@ /* clear out any errors by writing the line status register */
	@ serial_echo_outl(0xff, UTSR0) ;
	ldr	r2, =0xff
	str	r2, [r1, #UTSR0]

	mov	pc, lr



# takes one character in r0
# assumes transmitter is not busy
serial_echo_print_P:	
	ldr     r1, =SERIAL_ECHO_PORT_P
	@ serial_echo_outl (*s, UART_TX)
	str	r0, [r1, #UART_TX]
	mov	pc, lr

# takes one character in r0
serial_echo_print_wait_P:	
	ldr	r1, =SERIAL_ECHO_PORT_P
WAIT_FOR_XMITR1:	
	ldr	r2, [r1, #UTSR1]
	tst	r2, #UTSR1_TNF
	beq     WAIT_FOR_XMITR1

	@ serial_echo_outl (*s, UART_TX)
	str	r0, [r1, #UART_TX]
	mov	pc, lr

# takes one character in r0
# assumes transmitter is not busy
serial_echo_print_V:	
	ldr     r1, =SERIAL_ECHO_PORT_V
	@ serial_echo_outl (*s, UART_TX)
	str	r0, [r1, #UART_TX]
	mov	pc, lr

# takes one character in r0
	.globl serial_echo_print_wait_V
serial_echo_print_wait_V:	
	ldr	r1, =SERIAL_ECHO_PORT_V
WAIT_FOR_XMITR2:	
	ldr	r2, [r1, #UTSR1]
	tst	r2, #UTSR1_TNF
	beq     WAIT_FOR_XMITR2

	@ serial_echo_outl (*s, UART_TX)
	str	r0, [r1, #UART_TX]
	mov	pc, lr


# takes one character in r0
	.globl serial_echo_print_wait_C_V
serial_echo_print_wait_C_V:	
WAIT_FOR_XMITR3:	
	ldr	r1, =SERIAL_ECHO_PORT_V
	ldr	r1, [r1, #UTSR1]
	tst	r1, #UTSR1_TNF
	beq     WAIT_FOR_XMITR3

	@ serial_echo_outl (*s, UART_TX)
	ldr	r1, =SERIAL_ECHO_PORT_V
	str	r0, [r1, #UART_TX]

	cmp	r0, #10
	moveq	r0, #13
	beq	WAIT_FOR_XMITR3

	mov	pc, lr





/*
 * Useful debugging routines
 */
		.globl	printhex8
printhex8:
		ldr	r1, =fakestack
		stmfa	r1!, {r1-r4, lr}
		ldr	r2, =fakestack
		str	r1, [r2]
		mov	r1, #8
		mov	r4, r1
		ldr	r2, =hexbuf
		add	r3, r2, r1
		mov	r1, #0
		strb	r1, [r3]
1:		and	r1, r0, #15
		mov	r0, r0, lsr #4
		cmp	r1, #10
		addlt	r1, r1, #'0'
		addge	r1, r1, #'a' - 10
		strb	r1, [r3, #-1]!
		teq	r3, r2
		bne	1b

@ r4 is size
@ r2 is location

	add	r3, r4, r2
	mov	r4, #0
1:
	ldrb	r0, [r2], #1
	bl	serial_echo_print_wait_C_V
	teq	r3, r2
	bne	1b

	ldr	r0, =fakestack
	ldr	r0, [r0]
	ldmfa	r0!, {r1-r4, pc}
	
	
		.bss
hexbuf:		.space 16


fakestack:	.space 1024
