/*
 * SA1100 Sleep Mode	
 * Copyright (c) Compaq Computer Corporation, 1998, 1999
 *
 * Author: Deborah A. Wallach
 * 03-25-98
 *
 * Suspend/resume from SA1100 sleep mode (higher level), 
 * save and restore general purpose registers.
 *	
 */

#include <linux/linkage.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <asm/io_reg.h>
#include <linux/mm.h>
#include <asm/pgtable.h>
#include <asm/arch/mmu.h>
#include <linux/powermgr.h>
#include <asm/arch/Itsy.h>
#include <linux/itsy_version.h>

unsigned long sleep_mode_save_area[1024];

extern void rtc_set_alarm(int seconds);
extern void rtc_disable_alarm(void);
extern int rtc_alarm_happened(void);

void sa1100_sleep_mode_enter(void)
{
  extern void sa1100_sleep_mode(void *, unsigned long, unsigned long);
  extern void entry_from_sleep(void);
  extern unsigned long coproc_data;
  unsigned long the_sp;

  if (is_itsy_v2()) {
    /* do not turn off DRAM once in sleep mode */
    outl((inl(PGSR_V) & (~GPIO_2nDRAMEN)), PGSR_V);
  }

  rtc_set_alarm(45);

  {
    int delay;
    if (powermgr_suspend_check(0, &delay)) {
      powermgr_suspend();
    }
    else {
      printk("suspend_check failed with delay %d\n", delay);
      return;
    }
  }

  outl(PCFR_ClkStp + PCFR_PCMCIANeg + PCFR_StMemNeg, PCFR_V);

  /* get the stack pointer */
  __asm__ __volatile__("mov  %0, sp"                                    \
			 : "=r" (the_sp)				\
			 :);
  the_sp = virt_to_phys(the_sp);

  sa1100_sleep_mode(&coproc_data,  /* where to write some info */
		    virt_to_phys((unsigned long)entry_from_sleep),
		    the_sp
		    );

  return;  /* should never reach here */
}

void sa1100_deep_sleep_mode_enter(void)
{
  extern void sa1100_sleep_mode(void *, unsigned long, unsigned long);
  extern void entry_from_sleep(void);
  extern void lcdOff(void);

  if (is_itsy_v1()) sa1100_sleep_mode_enter();  /* YYY interface for V1 not implemented yet in monitor */
  /* remove this check once implemented in monitor and it should be fine */

  {
    int delay;
    if (powermgr_suspend_check(0, &delay)) {
      powermgr_suspend();
    }
    else {
      printk("suspend_check failed with delay %d\n", delay);
      return;
    }
  }

  /* Turn off lcd screen  LSB */
  lcdOff(); 

  if (is_itsy_v2()) {
    /* turn off DRAM once in sleep mode */
    outl((inl(PGSR_V) | GPIO_2nDRAMEN), PGSR_V);
  }

  outl(PCFR_ClkStp + PCFR_PCMCIANeg + PCFR_StMemNeg, PCFR_V);

  outl(1, PSPR_V);
  outl(1, PMCR_V);
  while (1);

  return;  /* should never reach here */
}

/* called after make the phys-to-virt transition */
asmlinkage void wakeup_from_sleep_mode(void) 
{
  extern void wakeup_done(void);
  extern void sa1100_sleep_mode_final(void);

  if (rtc_alarm_happened()) {
    extern void entry_from_sleep(void);
    extern int powerbutton_suspend(void *);
    extern void lcdOff(void);

    /* turn off lcd screen */
    lcdOff();

    /* rewrite address to jump to on stack for monitor */
    /* get stack address from PSPR (scratch pad register)
     * it's physical, so convert to virtual so we can write it
     */
    outl(virt_to_phys((unsigned long)entry_from_sleep),
	 phys_to_virt(inl(PSPR_V)));

    /* hmm, can't test if we woke up due to button press? */

    /* reinitialize button to wake us up */
    powerbutton_suspend(0);

    printk("woke up due to rtc alarm... going back to sleep\n");
    rtc_disable_alarm();
    sa1100_sleep_mode_final();
    return; /* should never reach here */
  }

  powermgr_resume(inl(PSSR_V));

  /* wakeup is done and clean; clear the reset and power status bits */
  outl(0x1f,PSSR_V);
  outl(0xf, RCSR_V);   

  wakeup_done();

  return;  /* won't actually reach here */
}


/* This is here so flush_cache_all can be called from assembly */
asmlinkage void call_flush_cache_all(void) 
{
  flush_cache_all();
}


/* power management functions */
static unsigned long general_regs[4];  /* storage of general registers during sleep */

static int general_suspend_check(void *unused, int idle_jiffies)
{
  return(0);
}

static int general_suspend(void *cookie)
{
  unsigned long *general_regs = (unsigned long *)cookie;
/*  general_regs[0] = inl(IRQ_ICMR_V); */
  general_regs[1] = inl(IRQ_ICLR_V);
  general_regs[2] = inl(IRQ_ICCR_V) & 1;  /* only real bit */

  general_regs[3] = inl(OSCR_V);

  return(1); /* YYY - values ignored for now */
}

static int general_resume(void *cookie, int resume_flags)
{
  unsigned long *general_regs = (unsigned long *)cookie;

  outl(0, IRQ_ICMR_V); /* reset ICMR */
  /* outl(general_regs[0], IRQ_ICMR_V); *****XXX**** not until everyone masks properly */
  outl(general_regs[1], IRQ_ICLR_V);
  outl(general_regs[2], IRQ_ICCR_V);

  outl(general_regs[3], OSCR_V); /* restore old value of OS free-running counter */

  outl(0xf, RCSR_V);   /* clear all reset status bits */
  return(1); /* YYY - values ignored for now */
}

void general_powermgr_register(void)
{
  powermgr_client client;
  powermgr_id general_powermgr_id; 

  client.suspend_check = general_suspend_check;
  client.suspend = general_suspend;
  client.resume = general_resume;
  client.cookie = general_regs;
  client.name = "General registers";
  client.uamps_idle = 0; /* YYY ? */
  client.uamps_sleep = 0; /* YYY ? */
  general_powermgr_id = powermgr_register(&client);
  if (general_powermgr_id == POWERMGR_ID_INVALID) {
    printk("Failed to register %s module for power management!!!\n", client.name);
    return;
  }

  /* clear the reset and power status bits */
  outl(0x1f,PSSR_V);
  outl(0xf, RCSR_V);   
}
