/*
 * SA1100 Sleep Mode	
 * Copyright (c) Compaq Computer Corporation, 1998, 1999
 *
 * Author: Deborah A. Wallach
 * 03-25-98
 *
 * Enter SA1100 sleep mode
 *	
 */

#define PMCR_V          0xe8020000  /* Power manager control register */
#define PSSR_V          0xe8020004  /* Power manager sleep status register */
#define PSPR_V          0xe8020008  /* Power manager scratch pad register */
#define PWER_V          0xe802000c  /* Power manager wakeup enable register */
#define PCFR_V          0xe8020010  /* Power manager general configuration register */
#define PPCR_V          0xe8020014  /* Power manager PLL configuration register */
#define PGSR_V          0xe8020018  /* Power manager GPIO sleep state register */
#define POSR_V          0xe802001c  /* Power manager oscillator status register */
#define GFER_V          0xe8040014  /* GPIO Falling-Edge detect Register */
#define GEDR_V          0xe8040018  /* GPIO Edge Detect status Register */
#define GPDR_V          0xe8040004  /* GPIO pin direction register */

@ r0 is &coproc_data  /* where to write some info */
@ r1 is the physical address of entry_from_sleep (where to jump when wake up)
@ r2 is the physical address of the stack pointer in sleep_mode_enter
	
	.text
	.align
	.globl sa1100_sleep_mode
	.globl sa1100_sleep_mode_final
sa1100_sleep_mode:	
	@ save coprocessor 15 register 1 (control)
	mrc	p15, 0, r3, c1, c0
	bic	r3, r3, #0x0c00  @ these bits are not valid
	bic	r3, r3, #0xc000
	bic	r3, r3, #0x00ff0000
	bic	r3, r3, #0xff000000
	str	r3, [r0, #0]

	@ save coprocessor 15 register 2 (tlb base)
	mrc	p15, 0, r3, c2, c0
	bic	r3, r3, #0x00ff  @ these bits are not valid
	bic	r3, r3, #0x3f00
	str	r3, [r0, #4]
	@ save stack pointer
	str	sp, [r0, #8]
	
	@ interface between monitor and linux:
	@  store wakeup routine on stack for monitor
	@  put stack address monitor can use in scratchpad register
	stmfd	sp!, {r1}
	@
	sub	r2, r2, #4    @ make r2 (phys addr of sp) match...
	ldr	r0, =PSPR_V
	str	r2, [r0]

	ldr	r0, =sleep_mode_save_area	
	stmea	r0!, {r4 - r9, fp, sp, lr}	@ Store most regs 
	mrs	ip, cpsr
	stmea	r0!, {ip}			@ Save cpsr_SVC
sa1100_sleep_mode_final:	
	bl	call_flush_cache_all
	
        @ PMCR = PMCR_SF  (go to sleep!)
	mov	r0, #1
	ldr	r1, =PMCR_V
	str	r0, [r1]

@	mov	r0,  #'x'
@	bl	serial_echo_print_wait_V
	b .    @ better not get here


	
@ this function should be called at the end of wakeup;
@ it will restore the previous registers and stack and jump to the place
@ the code was at when it went to sleep
	.globl	wakeup_done
wakeup_done:	
	ldr	sp, =sleep_mode_save_area
	add	sp, sp, #40 @ number of registers saved
	ldmea	sp!, {ip}
	msr	spsr, ip			@ Save tasks CPSR into SPSR for this return
	ldmea	sp!, {r4 - r9, fp, sp, pc}^	@ Load all regs saved previously

