/*
 *  linux/arch/arm/kernel/setup.c
 *
 *  Copyright (C) 1995, 1996 Russell King
 */

/*
 * This file obtains various parameters about the system that the kernel
 * is running on.
 */

#include <linux/config.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/stddef.h>
#include <linux/unistd.h>
#include <linux/ptrace.h>
#include <linux/malloc.h>
#include <linux/ldt.h>
#include <linux/user.h>
#include <linux/a.out.h>
#include <linux/tty.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/major.h>
#include <linux/utsname.h>
#include <linux/blk.h>

#include <asm/segment.h>
#include <asm/system.h>
#include <asm/hardware.h>
#include <asm/pgtable.h>
#include <asm/arch/mmu.h>
#include <asm/arch/mmap.h>
#include <asm/procinfo.h>

#include <asm/proc/unaligned.h>

#include <linux/itsy_version.h>

struct drive_info_struct { char dummy[32]; } drive_info;

struct screen_info screen_info = {
	0, 0,
	{ 0, 0 },
	0,
	0,
	100,
	0, 0, 0,
	75
};

#include "params.h"

struct param_struct *params = (struct param_struct *)PARAMS_BASE;

extern const struct processor arm2_processor_functions;
extern const struct processor arm250_processor_functions;
extern const struct processor arm3_processor_functions;
extern const struct processor arm6_processor_functions;
extern const struct processor arm7_processor_functions;
extern const struct processor sa110_processor_functions;
extern const struct processor sa1100_processor_functions;

struct armversions armidlist[] = {
#if defined(__arm2__) || defined(__arm3__)
	{ 0x41560200, 0xfffffff0, F_MEMC	, "ARM/VLSI",	"arm2"		, &arm2_processor_functions   },
	{ 0x41560250, 0xfffffff0, F_MEMC	, "ARM/VLSI",	"arm250"	, &arm250_processor_functions },
	{ 0x41560300, 0xfffffff0, F_MEMC|F_CACHE, "ARM/VLSI",	"arm3"		, &arm3_processor_functions   },
#endif
#if defined(__arm6__) || defined(__sa110__) || defined(__sa1100__)
	{ 0x41560600, 0xfffffff0, F_MMU|F_32BIT	, "ARM/VLSI",	"arm6"		, &arm6_processor_functions   },
	{ 0x41560610, 0xfffffff0, F_MMU|F_32BIT	, "ARM/VLSI",	"arm610"	, &arm6_processor_functions   },
	{ 0x41007000, 0xffffff00, F_MMU|F_32BIT , "ARM/VLSI",   "arm7"		, &arm7_processor_functions   },
	{ 0x41007100, 0xffffff00, F_MMU|F_32BIT , "ARM/VLSI",   "arm710"	, &arm7_processor_functions   },
	{ 0x4401a100, 0xfffffff0, F_MMU|F_32BIT	, "DEC",	"sa110"		, &sa110_processor_functions  },
	{ 0x4401a110, 0xfffffff0, F_MMU|F_32BIT	, "DEC",	"sa1100"	, &sa1100_processor_functions  },
#endif
	{ 0x00000000, 0x00000000, 0		, "***",	"*unknown*"	, NULL }
};

#ifdef CONFIG_BLK_DEV_IMG
#if defined(CONFIG_BLK_DEV_IDE) || defined(CONFIG_BLK_DEV_HD)
#define DFLT_MAJOR IDE0_MAJOR
#elif defined(CONFIG_BLK_DEV_XD)
#define DFLT_MAJOR XT_DISK_MAJOR
#else
#define DFLT_MAJOR IDE0_MAJOR
#endif

static kdev_t image_dev[8] = {
    MKDEV(DFLT_MAJOR, 0x01),
    MKDEV(DFLT_MAJOR, 0x01),
    MKDEV(DFLT_MAJOR, 0x01),
    MKDEV(DFLT_MAJOR, 0x01),
    MKDEV(DFLT_MAJOR, 0x41),
    MKDEV(DFLT_MAJOR, 0x41),
    MKDEV(DFLT_MAJOR, 0x41),
    MKDEV(DFLT_MAJOR, 0x41)
};

extern void adfsimg_add_image (int slot, kdev_t dev, char *path);

static inline void img_setupdevs(struct param_struct *params)
{
	if (params->u.s.magic != 0xfeedface) {
		int i;
		for (i = 0; i < 8; i++)
			adfsimg_add_image(i, image_dev[i], params->u.paths[i]);
	} else {
		int i, p = 1;

		for (i = 0; i < params->u.s.n[0]; i++) {
			kdev_t dev;

			dev = MKDEV(params->u.s.n[p+1], params->u.s.n[p]);
			p += 2;
			if (MAJOR(dev) != UNNAMED_MAJOR)
				adfsimg_add_image(i, dev, &params->u.s.n[p]);
			p += strlen(&params->u.s.n[p]) + 1;
		}
	}
}
#else
#define img_setupdevs(x)
#endif

#if defined(CONFIG_ARCH_RPC) || defined(CONFIG_ARCH_BRUTUS)
struct dram_bank drambank[NR_BANKS];
#endif
#if defined(CONFIG_ARCH_BRUTUS)
/* points to free part of mapable memory */
unsigned long current_mapable_pointer = MAPABLE_MEM_BASE;
unsigned long io_pgtable[MAPABLE_MEM_SIZE>>PGDIR_SHIFT] = {0};
#endif

#define COMMAND_LINE_SIZE 256

unsigned long arm_id;
unsigned long sleep_function_phys;
unsigned long sleep_function_virt = 0;
unsigned int vram_half_sam;
int armidindex;
int ioebpresent;
int memc_ctrl_reg;
int number_ide_drives;
int number_mfm_drives;

extern int bytes_per_char_h;
extern int bytes_per_char_v;
extern int root_mountflags;
extern int _etext, _edata, _end;
extern unsigned long real_end_mem;

unsigned int dram_info_pointer;
unsigned int staticmem_info_pointers[4];
unsigned int params_version;
unsigned int itsy_version;

#ifdef CONFIG_BLK_DEV_RAM
extern int rd_doload;		/* 1 = load ramdisk, 0 = don't load */
extern int rd_prompt;		/* 1 = prompt for ramdisk, 0 = don't prompt */
extern int rd_image_start;	/* starting block # of image */

static inline void setup_ramdisk (void)
{
	rd_image_start	= 0;
	rd_prompt	= 1;
	rd_doload	= 1;
}
#else
#define setup_ramdisk()	
#endif

#ifndef CONFIG_ARCH_BRUTUS
static void check_ioeb_present(void)
{
	if (((*IOEB_BASE) & 15) == 5)
		armidlist[armidindex].features |= F_IOEB;
}
#endif /* CONFIG_ARCH_BRUTUS */

static char command_line[COMMAND_LINE_SIZE] = { 0, };
       char saved_command_line[COMMAND_LINE_SIZE];

struct processor processor;

void setup_arch(char **cmdline_p,
	unsigned long * memory_start_p, unsigned long * memory_end_p)
{
	unsigned long memory_start, memory_end;
	char c = ' ', *to = command_line, *from;
	int len = 0;

	armidindex = 0;
	while (1) {
		if (!((armidlist[armidindex].id ^ arm_id) &
			armidlist[armidindex].mask))
			break;
		armidindex += 1;
	}

	if (armidlist[armidindex].id == 0) {
		int i;
		for (i = 0; i < 3200; i++)
			((unsigned long *)SCREEN2_BASE)[i] = 0x77113322;
		while (1);
	}

	processor = *armidlist[armidindex].proc;

	processor._proc_init ();

	memory_start = MAPTOPHYS((unsigned long)&_end);
	memory_end = GET_MEMORY_END(params);

	bytes_per_char_h  = params->bytes_per_char_h;
	bytes_per_char_v  = params->bytes_per_char_v;

	from		  = params->commandline;
	ROOT_DEV	  = MKDEV(RAMDISK_MAJOR,0);  /* params->rootdev; */ /* ZZZ */
	ORIG_X		  = params->video_x;
	ORIG_Y		  = params->video_y;
	ORIG_VIDEO_COLS	  = params->video_num_cols;
	ORIG_VIDEO_LINES  = params->video_num_rows;
	memc_ctrl_reg	  = params->memc_control_reg;   /* unused */
	number_ide_drives = 0;  /* (params->adfsdrives >> 6) & 3; */ /* ZZZ */
	number_mfm_drives = 0;  /* (params->adfsdrives >> 3) & 3; */ /* ZZZ */
	setup_ramdisk();

#if defined(CONFIG_ARCH_RPC) || defined(CONFIG_ARCH_BRUTUS)
	{
		int i;
		unsigned int bytes;

		for (i = 0, bytes = 0; i < NR_BANKS; i++) {
			drambank[i].v2p_offset = PAGE_OFFSET + bytes -
#ifdef CONFIG_ARCH_RPC
							(0x10000000 + (i << 26));
#else /* CONFIG_ARCH_BRUTUS */
                                                        (0xc0000000 + (i << 27));
#endif
			bytes += params->pages_in_bank[i] * PAGE_SIZE;
			drambank[i].virt_offset = PAGE_OFFSET + bytes;
		}
#ifdef CONFIG_ARCH_RPC
		drambank[5].v2p_offset = 0xd6000000;
		drambank[5].virt_offset = 0xd8000000;
#endif /* CONFIG_ARCH_RPC */
	}
	switch (params->pages_in_vram) {  /* ZZZ */
	case 256:
		vram_half_sam = 1024;
		break;
	case 512:
	default:
		vram_half_sam = 2048;
	}

#endif

	if (params->mountrootrdonly)
		root_mountflags |= MS_RDONLY;

	img_setupdevs(params);  /* should be empty function */

	init_task.mm->start_code = TASK_SIZE;
	init_task.mm->end_code	 = TASK_SIZE + (unsigned long) &_etext;
	init_task.mm->end_data	 = TASK_SIZE + (unsigned long) &_edata;
	init_task.mm->brk	 = TASK_SIZE + (unsigned long) &_end;

#if defined(CONFIG_ARCH_BRUTUS)
	{
	  int i;

	  params_version = params->version;
	  for (i=0; i<4; i++) {
	    staticmem_info_pointers[i] = params->staticmem_info_pointers[i];
	  }
	  dram_info_pointer = params->dram_info_pointer;

	  itsy_version = params->itsy_version;

	  printk("params info:\n"
		 "  staticmem_info_pointers[0] = %08lx\n"
		 "  staticmem_info_pointers[1] = %08lx\n"
		 "  staticmem_info_pointers[2] = %08lx\n"
		 "  staticmem_info_pointers[3] = %08lx\n"
		 "  dram_info_pointer = %08x\n"
		 "  itsy_version = %08x\n"
		 "  mon_version  = %08x\n"
		 "  class_id[0] = %08lx\n"
		 "  class_id[1] = %08lx\n"
		 "  class_id[2] = %08lx\n"
		 "  class_id[3] = %08lx\n"
		 "  features = %08x\n",
		 params->staticmem_info_pointers[0],
		 params->staticmem_info_pointers[1],
		 params->staticmem_info_pointers[2],
		 params->staticmem_info_pointers[3],
		 params->dram_info_pointer,
		 params->itsy_version,
		 params->mon_version,
		 params->class_id[0],
		 params->class_id[1],
		 params->class_id[2],
		 params->class_id[3],
		 params->features);
	}
#endif

	/* Save unparsed command line copy for /proc/cmdline */
	memcpy(saved_command_line, from, COMMAND_LINE_SIZE);
	saved_command_line[COMMAND_LINE_SIZE-1] = '\0';

	for (;;) {
		if (c == ' ' &&
		    from[0] == 'm' &&
		    from[1] == 'e' &&
		    from[2] == 'm' &&
		    from[3] == '=') {
			memory_end = simple_strtoul(from+4, &from, 0);
			if (*from == 'K' || *from == 'k') {
				memory_end = memory_end << 10;
				from++;
			} else if (*from == 'M' || *from == 'm') {
				memory_end = memory_end << 20;
				from++;
			}
			memory_end = memory_end + PAGE_OFFSET;
		}
		c = *from++;
		if (!c)
			break;
		if (COMMAND_LINE_SIZE <= ++len)
			break;
		*to++ = c;
	}

	*to = '\0';
	*cmdline_p = command_line;
	*memory_start_p = memory_start;
	*memory_end_p = memory_end;
	strcpy (system_utsname.machine, armidlist[armidindex].name);
}

#ifdef CONFIG_ARCH_BRUTUS
void setup_arch_after_paging(void)
{
#ifdef CONFIG_ITSY       
  /* map in FLASH memory */
  initrd_start = allocate_vspace(FLASH1, 4<<PGDIR_SHIFT, 0, 0)+params->ramdisk_offset;
  initrd_end   = initrd_start-params->ramdisk_offset+(4<<PGDIR_SHIFT)-400;  /* make sure don't read into Mark's section */
#endif /* CONFIG_ITSY */
}
#endif /* CONFIG_ARCH_BRUTUS */

#define ISSET(bit) (armidlist[armidindex].features & bit)

int get_cpuinfo(char * buffer)
{
	int len;
	extern STATS_MISALIGNED Tmisalignstat_stats;
	extern int arm_mm_alignment_enabled;
	char *alignstr;

	switch ( arm_mm_alignment_enabled ) {
	case 0:
	  alignstr = "Alignment, no reporting";
	  break;
	case 1:
	  alignstr = "Alignment, reporting";
	  break;
	case 2:
	  alignstr = "No alignment";
	  break;
	case3:
	  alignstr = "Unknown alignment status?";
	}

	len = sprintf (buffer,  "CPU:\n"
				"Type\t\t: %s\n"
				"Revision\t: %d\n"
				"Manufacturer\t: %s\n"
				"32bit modes\t: %s\n"
				"BogoMips\t: %lu.%02lu\n"
		       		"Alignment handler: %s\n"
		                "\nMisaligned Accesses\t: %d (sys: %d) (usr: %d)\n"
		                "  LDR/STR accesses\t: %d\n"
		                "  LDRH/STRH accesses\t: %d\n"
		                "  LDM/STM accesses\t: %d\n"
		                "  SWP accesses    \t: %d\n"
		                "  Invalid accesses\t: %d\n",


				armidlist[armidindex].name,
				(int)arm_id & 15,
				armidlist[armidindex].manu,
				ISSET (F_32BIT) ? "yes" : "no",
				(loops_per_sec+2500) / 500000,
				((loops_per_sec+2500) / 5000) % 100,
		       		alignstr,
		                (Tmisalignstat_stats.UI_user+Tmisalignstat_stats.UI_sys),
		                Tmisalignstat_stats.UI_sys,
		                Tmisalignstat_stats.UI_user,
		                Tmisalignstat_stats.UI_word,
		                Tmisalignstat_stats.UI_short,
		                Tmisalignstat_stats.UI_multiple,
		                Tmisalignstat_stats.UI_swap,
		                Tmisalignstat_stats.UI_unsupported
		       );
	  
	len += sprintf (buffer + len,
				"\nHardware:\n"
				"Mem System\t: %s\n"
				"IOEB\t\t: %s\n",
				ISSET(F_MEMC)  ? "MEMC" : 
				ISSET(F_MMU)   ? "MMU"  : "*unknown*",
				ISSET(F_IOEB)  ? "present" : "absent"
				);
	return len;
}

#ifdef	CONFIG_MODULES
#include <linux/module.h>
#include <linux/version.h>
#endif
#ifdef CONFIG_POWERMGR
#include <linux/powermgr.h>
#endif
extern void clock_call_dram_setup(int, int, void *);
/* DS2437 battery monitor interface routines */
extern int battmon_send_reset(int);
extern void battmon_write_one(int);
extern void battmon_write_zero(int);
extern int battmon_read_bit(int);


#ifdef	CONFIG_MODULES
static struct symbol_table setup_syms = {
#include <linux/symtab_begin.h>
        X(current_mapable_pointer),
        X(io_pgtable),
        X(sleep_function_phys),
        X(sleep_function_virt),
        X(clock_call_dram_setup),
	X(dram_info_pointer),
	X(itsy_version),
	X(staticmem_info_pointers),
	X(params_version),
        X(battmon_send_reset),
        X(battmon_write_one),
        X(battmon_write_zero),
        X(battmon_read_bit),
#include <linux/symtab_end.h>
};
#endif

void setup_ksyms_arch(void) 
{
#ifdef	CONFIG_POWERMGR
  powermgr_init_ksyms();
  powermgr_init_procfs();
#endif	/* CONFIG_POWERMGR */
#ifdef	CONFIG_MODULES
  /* export symbols */
  (void) register_symtab(&setup_syms);
#endif	/* CONFIG_MODULES */
}
  
