/*
 * linux/arch/arm/lib/fp_support.c
 *
 * Copyright (C) 1995, 1996 Russell King
 */

#include <linux/sched.h>

extern void (*fp_save)(void *);

    /*
    Special hack for Russel's FP emulator.
    It expects to find a variable "fp_current", which would be the same
    as kernel's "current".
    I can not make this association working (compiler seems to be to smart???)
    As such I declare here a dummy variable fp_current, just to make insmod
    happy, and then inside /kernel/exit.c in send_signal function there is
    a special case added - if the sig is SIGFPE, we ignore passed "fp_current"
    and replace it arbitrary with "current".
    The nwfpe does not need the send_signal hack, as it is already passing
    "current" correctly.
    The Russel's fpem.o can not be modified, and as such needs the hack.

    If U know, how to convince the compiler to accept fp_current = current, 
    pls let me know...

    woody@corelcomputer.com
    */

//struct task_struct *fp_current;

/* export weak alias, just so that these 3 have something to
be initialized with; when the fp_emulator module binds, it will
overload these entries anyway */




#ifdef __ELF__
extern int fp_printk   __attribute__ ((weak, alias ("printk"  )));
int fp_printk;
extern int fp_current  __attribute__ ((weak, alias ("current" )));
extern int fp_send_sig __attribute__ ((weak, alias ("send_sig")));
#else
extern int fp_printk(void);
extern int fp_send_sig(unsigned long sig,struct task_struct * p,int priv);
__asm__(
	".stabs \"fp_printk\", 11, 0, 0, 0\n\t"
	".stabs \"printk\", 1, 0, 0, 0\n\t"
	".stabs \"fp_current\", 11, 0, 0, 0\n\t"
	".stabs \"current\", 1, 0, 0, 0\n\t"
	".stabs \"fp_send_sig\", 11, 0, 0, 0\n\t"
	".stabs \"send_sig\", 1, 0, 0, 0\n\t");
#endif


void fp_setup(void)
{
	struct task_struct **p;

	p = task;
	do {
		if(*p)
			fp_save((*p)->tss.fpe_save);
	}
	while (++p < task + NR_TASKS);
}
