/*
 * linux/arch/arm/lib/irqs.S
 *
 * Copyright (C) 1995, 1996 Russell King. (rmk92@ecs.soton.ac.uk)
 *
 * Interrupt wrappers - handles disabling & re-enabling interrupts
 */

#include <asm/assembler.h>

@ IRQ stubs

@ IRQ stubs entered with:
@ r0 = IRQ number
@ r1 = regs
@ r4 = IRQ_ICMR_V
@ return 0 for normal irq

		.text
/*
 * Disable interrupt & print a message
 */
LC0:		.long	intr_count

		.globl	bad_IRQ_interrupt
bad_IRQ_interrupt:
		mov	r9, lr
		ldr	r5, [r4]        @ load the mask register
		mov	r2, #1
		bic	r2, r5, r2, lsl r0  @ set irq mask bit to 0
		str	r2, [r4]
		bl	bad_IRQ
		mov	r0, #0
		mov	pc, r9

		.globl	fast_IRQ_interrupt
fast_IRQ_interrupt:
		mov	r9, lr
		ldr	r5, [r4]        @ load the mask register
		mov	r2, #1
		bic	r2, r5, r2, lsl r0  @ set irq mask bit to 0
		str	r2, [r4]
		ldr	r6, LC0
		ldr	r7, [r6]
		add	r2, r7, #1
		str	r2, [r6]
		bl	do_IRQ
		mrs	r2, cpsr
		orr	r2, r2, #I_BIT
		msr	cpsr, r2
		str	r7, [r6]
		str	r5, [r4]
		mov	r0, #1
		mov	pc, r9

		.globl	IRQ_interrupt
IRQ_interrupt:
		mov	r9, lr
		ldr	r5, [r4]        @ load the mask register
		mov	r2, #1
		bic	r2, r5, r2, lsl r0  @ set irq mask bit to 0
		str	r2, [r4]			@ Disable interrupt source
		ldr	r6, LC0
		ldr	r7, [r6]
		add	r2, r7, #1
		str	r2, [r6]
		mrs	r8, cpsr			@ Enable interrupts
		bic	r2, r8, #I_BIT
		msr	cpsr, r2
		bl	do_IRQ
		msr	cpsr, r8			@ Restore interrupts
		str	r7, [r6]
		str	r5, [r4]			@ Re-enable interrupt source
		mov	r0, #0
		mov	pc, r9

		.globl	timer_IRQ_interrupt
timer_IRQ_interrupt:
		mov	r9, lr
		ldr	r5, [r4]        @ load the mask register
		mov	r2, #1
		bic	r2, r5, r2, lsl r0  @ set irq mask bit to 0
		str	r2, [r4]			@ Disable interrupt source
		ldr	r6, LC0
		ldr	r7, [r6]
		add	r2, r7, #1
		str	r2, [r6]
		mrs	r8, cpsr
		bl	do_IRQ
		msr	cpsr, r8			@ Restore interrupts
		str	r7, [r6]
		str	r5, [r4]			@ Re-enable interrupt source
		mov	r0, #0
		mov	pc, r9

@ XXX not sure if this is doing what it used to
		.globl	probe_IRQ_interrupt
probe_IRQ_interrupt:
		mov	r9, lr
		ldr	r5, [r4]        @ load the mask register
		mov	r2, #1
		bic	r2, r5, r2, lsl r0  @ set irq mask bit to 0
		mov	r6, r4
		str	r2, [r6]
		ldr	r6, LC0+4
		ldr	r4, [r6]
		mov	r2, #1
		bic	r4, r4, r2, lsl r0
		str	r4, [r6]
		mov	pc, r9

		.global	bad_IRQ
bad_IRQ:	mov	r1, r0
		adr	r0, Lmsg
		b	printk

Lmsg:		.ascii	"Bad interrupt %d received!\n\0"
		.align
