/*
 *  linux/fs/adfs/map.c
 *
 * Copyright (C) 1997 Russell King
 */

#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/adfs_fs.h>

static int lookup_zone (struct super_block *sb, int zone, int frag_id, int *offset)
{
	unsigned int mapptr, idlen, mapsize;
	unsigned long *map;

	map     = (unsigned long *)sb->u.adfs_sb.s_map[zone]->b_data;
	zone    =
	mapptr  = (zone == 0 ? ADFS_DR_SIZE + 4 : 4) << 3;
	idlen   = sb->u.adfs_sb.s_idlen;
	mapsize = sb->u.adfs_sb.s_zonesize;

	do {
		unsigned long v1, v2;
		unsigned int start;

		__asm__(
		"	add	%1, %2, %3, lsr #3\n"
		"	bic	%1, %1, #3\n"
		"	ldr	%0, [%1], #4\n"
		"	ldr	%1, [%1]\n"
		: "=r" (v1), "=r" (v2) : "r" (map), "r" (mapptr));
		v1 = (v1 >> (mapptr & 31)) | (v2 << (32 - (mapptr & 31)));
		start = mapptr;
		mapptr += idlen;

		v2 = map[mapptr >> 5] >> (mapptr & 31);
		do {
			if (!v2) {
				mapptr = (mapptr + 32) & ~31;
				v2 = map[mapptr >> 5];
			} else {
				for (; (v2 & 255) == 0; v2 >>= 8, mapptr += 8);
				for (; (v2 & 1) == 0; v2 >>= 1, mapptr += 1);
				break;
			}
		} while (mapptr < mapsize);
		mapptr += 1;

		if ((v1 & ((1 << idlen) - 1)) == frag_id) {
			int length = mapptr - start;
			if (*offset >= length)
				*offset -= length;
			else
				return start + *offset - zone;
		}
	} while (mapptr < mapsize);
	return -1;
}

int adfs_map_lookup (struct super_block *sb, int frag_id, int offset)
{
	unsigned int start_zone, zone, max_zone;

	zone = start_zone = frag_id / sb->u.adfs_sb.s_ids_per_zone;
	max_zone = sb->u.adfs_sb.s_map_size;

	if (start_zone >= max_zone)
		return 0;

	do {
		int result;

		result = lookup_zone (sb, zone, frag_id, &offset);
		if (result != -1) {
			result += zone ? (zone * sb->u.adfs_sb.s_zonesize) - (ADFS_DR_SIZE << 3): 0;
			return result;
		}
		zone ++;
		if (zone >= max_zone)
			zone = 0;
	} while (zone != start_zone);
	return 0;
}
