
/*
 *	FILE    	Am29xxx.h
 *
 *	Version 	1.2
 *	Author  	Marc A. Viredaz
 *	        	Compaq Western Research Laboratory, Palo Alto, CA
 *	        	(DEC Western Research Laboratory, Palo Alto, CA)
 *	Date    	January 2000 (January 1998)
 *	System  	Portable
 *	Language	C or ARM Assembly
 *	Purpose 	Definition of constants related to the flash memories
 *	        	of the AMD Am29LVxxx and AMD Am29DLxxx families (this
 *	        	file replaces the file "Am29LVxxx.h").
 *	Copyright	(c) Compaq Computer Corporation, 1998, 2000.
 *
 * Description
 *    This file defines the constants used to erase and program flash
 *    memories of the AMD Am29LVxxx and AMD Am29DLxxx families (see
 *    "supported devices", below).  Addresses are defined for byte-mode
 *    operations (i.e., pin BYTE# set to 0) and should be shifted left by 1
 *    bit for word-mode operations (i.e., pin BYTE# set to 1). The macros
 *    "F29_8Add" and "F29_16Add" should be used for that purpose. Addresses
 *    corresponds to the address pins of the flash memories and should be
 *    appropriately shifted to account for the processor addresses. All data
 *    constants are defined for word-mode operations (i.e., pin BYTE# set
 *    to 1) and should be cropped to 8 bits for byte-mode operations (i.e.,
 *    pin BYTE# set to 0). The macros "F29_8Data" and "F29_16Data" should be
 *    used for that purpose.
 *
 *    Supported devices
 *
 *       A29_Am29LV160B 	AMD Am29LV160DB70EC/I/E
 *                      	AMD Am29LV160DB70FC/I/E
 *                      	AMD Am29LV160DB70WCC/I/E
 *                      	AMD Am29LV160BB70REC
 *                      	AMD Am29LV160BB70RFC
 *                      	AMD Am29LV160BB70RWCC
 *                      	AMD Am29LV160BB80REC
 *                      	AMD Am29LV160BB80RFC
 *                      	AMD Am29LV160BB80RWCC
 *                      	AMD Am29LV160BB80EC/I/E
 *                      	AMD Am29LV160BB80FC/I/E
 *                      	AMD Am29LV160BB80WCC/I/E
 *                      	AMD Am29LV160DB90EC/I/E
 *                      	AMD Am29LV160DB90FC/I/E
 *                      	AMD Am29LV160DB90WCC/I/E
 *                      	AMD Am29LV160BB90EC/I/E
 *                      	AMD Am29LV160BB90FC/I/E
 *                      	AMD Am29LV160BB90WCC/I/E
 *                      	AMD Am29LV160DB120EC/I/E
 *                      	AMD Am29LV160DB120FC/I/E
 *                      	AMD Am29LV160DB120WCC/I/E
 *                      	AMD Am29LV160BB120EC/I/E
 *                      	AMD Am29LV160BB120FC/I/E
 *                      	AMD Am29LV160BB120WCC/I/E
 *       A29_Am29LV160T 	AMD Am29LV160DT70EC/I/E
 *                      	AMD Am29LV160DT70FC/I/E
 *                      	AMD Am29LV160DT70WCC/I/E
 *                      	AMD Am29LV160BT70REC
 *                      	AMD Am29LV160BT70RFC
 *                      	AMD Am29LV160BT70RWCC
 *                      	AMD Am29LV160BT80REC
 *                      	AMD Am29LV160BT80RFC
 *                      	AMD Am29LV160BT80RWCC
 *                      	AMD Am29LV160BT80EC/I/E
 *                      	AMD Am29LV160BT80FC/I/E
 *                      	AMD Am29LV160BT80WCC/I/E
 *                      	AMD Am29LV160DT90EC/I/E
 *                      	AMD Am29LV160DT90FC/I/E
 *                      	AMD Am29LV160DT90WCC/I/E
 *                      	AMD Am29LV160BT90EC/I/E
 *                      	AMD Am29LV160BT90FC/I/E
 *                      	AMD Am29LV160BT90WCC/I/E
 *                      	AMD Am29LV160DT120EC/I/E
 *                      	AMD Am29LV160DT120FC/I/E
 *                      	AMD Am29LV160DT120WCC/I/E
 *                      	AMD Am29LV160BT120EC/I/E
 *                      	AMD Am29LV160BT120FC/I/E
 *                      	AMD Am29LV160BT120WCC/I/E
 *       A29_Am29LV800B 	AMD Am29LV800BB70EC/I
 *                      	AMD Am29LV800BB70FC/I
 *                      	AMD Am29LV800BB70WBC/I
 *                      	AMD Am29LV800BB70REC/I/E
 *                      	AMD Am29LV800BB70RFC/I/E
 *                      	AMD Am29LV800BB70RWBC/I
 *                      	AMD Am29LV800BB80EC/I/E
 *                      	AMD Am29LV800BB80FC/I/E
 *                      	AMD Am29LV800BB80WBC/I/E
 *                      	AMD Am29LV800BB90EC/I/E
 *                      	AMD Am29LV800BB90FC/I/E
 *                      	AMD Am29LV800BB90WBC/I/E
 *                      	AMD Am29LV800B-90REC/I
 *                      	AMD Am29LV800B-90RFC/I
 *                      	AMD Am29LV800B-100EC/I/E
 *                      	AMD Am29LV800B-100FC/I/E
 *                      	AMD Am29LV800BB120EC/I/E
 *                      	AMD Am29LV800BB120FC/I/E
 *                      	AMD Am29LV800BB120WBC/I/E
 *                      	AMD Am29LV800B-120EC/I/E
 *                      	AMD Am29LV800B-120FC/I/E
 *                      	AMD Am29LV800B-150EC/I/E
 *                      	AMD Am29LV800B-150FC/I/E
 *       A29_Am29LV800T 	AMD Am29LV800BT70EC/I
 *                      	AMD Am29LV800BT70FC/I
 *                      	AMD Am29LV800BT70WBC/I
 *                      	AMD Am29LV800BT70REC/I/E
 *                      	AMD Am29LV800BT70RFC/I/E
 *                      	AMD Am29LV800BT70RWBC/I
 *                      	AMD Am29LV800BT80EC/I/E
 *                      	AMD Am29LV800BT80FC/I/E
 *                      	AMD Am29LV800BT80WBC/I/E
 *                      	AMD Am29LV800BT90EC/I/E
 *                      	AMD Am29LV800BT90FC/I/E
 *                      	AMD Am29LV800BT90WBC/I/E
 *                      	AMD Am29LV800T-90REC/I
 *                      	AMD Am29LV800T-90RFC/I
 *                      	AMD Am29LV800T-100EC/I/E
 *                      	AMD Am29LV800T-100FC/I/E
 *                      	AMD Am29LV800BT120EC/I/E
 *                      	AMD Am29LV800BT120FC/I/E
 *                      	AMD Am29LV800BT120WBC/I/E
 *                      	AMD Am29LV800T-120EC/I/E
 *                      	AMD Am29LV800T-120FC/I/E
 *                      	AMD Am29LV800T-150EC/I/E
 *                      	AMD Am29LV800T-150FC/I/E
 *       A29_Am29LV400B 	AMD Am29LV400BB55REC/I
 *                      	AMD Am29LV400BB55RFC/I
 *                      	AMD Am29LV400BB55RWAC/I
 *                      	AMD Am29LV400BB70EC/I/E
 *                      	AMD Am29LV400BB70FC/I/E
 *                      	AMD Am29LV400BB70WAC/I/E
 *                      	AMD Am29LV400BB80EC/I/E
 *                      	AMD Am29LV400BB80FC/I/E
 *                      	AMD Am29LV400BB80WAC/I/E
 *                      	AMD Am29LV400BB90EC/I/E
 *                      	AMD Am29LV400BB90FC/I/E
 *                      	AMD Am29LV400BB90WAC/I/E
 *                      	AMD Am29LV400B-90REC/I
 *                      	AMD Am29LV400B-90RFC/I
 *                      	AMD Am29LV400B-90RWAC
 *                      	AMD Am29LV400B-100EC/I/E
 *                      	AMD Am29LV400B-100FC/I/E
 *                      	AMD Am29LV400B-100WAC/I/E
 *                      	AMD Am29LV400BB120EC/I/E
 *                      	AMD Am29LV400BB120FC/I/E
 *                      	AMD Am29LV400BB120WAC/I/E
 *                      	AMD Am29LV400B-120EC/I/E
 *                      	AMD Am29LV400B-120FC/I/E
 *                      	AMD Am29LV400B-120WAC/I/E
 *                      	AMD Am29LV400B-150EC/I/E
 *                      	AMD Am29LV400B-150FC/I/E
 *                      	AMD Am29LV400B-150WAC/I/E
 *       A29_Am29LV400T 	AMD Am29LV400BT55REC/I
 *                      	AMD Am29LV400BT55RFC/I
 *                      	AMD Am29LV400BT55RWAC/I
 *                      	AMD Am29LV400BT70EC/I/E
 *                      	AMD Am29LV400BT70FC/I/E
 *                      	AMD Am29LV400BT70WAC/I/E
 *                      	AMD Am29LV400BT80EC/I/E
 *                      	AMD Am29LV400BT80FC/I/E
 *                      	AMD Am29LV400BT80WAC/I/E
 *                      	AMD Am29LV400BT90EC/I/E
 *                      	AMD Am29LV400BT90FC/I/E
 *                      	AMD Am29LV400BT90WAC/I/E
 *                      	AMD Am29LV400T-90REC/I
 *                      	AMD Am29LV400T-90RFC/I
 *                      	AMD Am29LV400T-90RWAC
 *                      	AMD Am29LV400T-100EC/I/E
 *                      	AMD Am29LV400T-100FC/I/E
 *                      	AMD Am29LV400T-100WAC/I/E
 *                      	AMD Am29LV400BT120EC/I/E
 *                      	AMD Am29LV400BT120FC/I/E
 *                      	AMD Am29LV400BT120WAC/I/E
 *                      	AMD Am29LV400T-120EC/I/E
 *                      	AMD Am29LV400T-120FC/I/E
 *                      	AMD Am29LV400T-120WAC/I/E
 *                      	AMD Am29LV400T-150EC/I/E
 *                      	AMD Am29LV400T-150FC/I/E
 *                      	AMD Am29LV400T-150WAC/I/E
 *       A29_Am29LV200B 	AMD Am29LV200BB55REC/I
 *                      	AMD Am29LV200BB55RFC/I
 *                      	AMD Am29LV200BB70EC/I
 *                      	AMD Am29LV200BB70FC/I
 *                      	AMD Am29LV200BB80EC/I/E
 *                      	AMD Am29LV200BB80FC/I/E
 *                      	AMD Am29LV200BB90EC/I/E
 *                      	AMD Am29LV200BB90FC/I/E
 *                      	AMD Am29LV200B-90REC/I
 *                      	AMD Am29LV200B-90RFC/I
 *                      	AMD Am29LV200B-100EC/I/E
 *                      	AMD Am29LV200B-100FC/I/E
 *                      	AMD Am29LV200BB120EC/I/E
 *                      	AMD Am29LV200BB120FC/I/E
 *                      	AMD Am29LV200B-120EC/I/E
 *                      	AMD Am29LV200B-120FC/I/E
 *                      	AMD Am29LV200B-150EC/I/E
 *                      	AMD Am29LV200B-150FC/I/E
 *       A29_Am29LV200T 	AMD Am29LV200BT55REC/I
 *                      	AMD Am29LV200BT55RFC/I
 *                      	AMD Am29LV200BT70EC/I
 *                      	AMD Am29LV200BT70FC/I
 *                      	AMD Am29LV200BT80EC/I/E
 *                      	AMD Am29LV200BT80FC/I/E
 *                      	AMD Am29LV200BT90EC/I/E
 *                      	AMD Am29LV200BT90FC/I/E
 *                      	AMD Am29LV200T-90REC/I
 *                      	AMD Am29LV200T-90RFC/I
 *                      	AMD Am29LV200T-100EC/I/E
 *                      	AMD Am29LV200T-100FC/I/E
 *                      	AMD Am29LV200BT120EC/I/E
 *                      	AMD Am29LV200BT120FC/I/E
 *                      	AMD Am29LV200T-120EC/I/E
 *                      	AMD Am29LV200T-120FC/I/E
 *                      	AMD Am29LV200T-150EC/I/E
 *                      	AMD Am29LV200T-150FC/I/E
 *       A29_Am29DL322B 	AMD Am29DL322CB70REC/I
 *                      	AMD Am29DL322CB70RWDC/I
 *                      	AMD Am29DL322CB90EI
 *                      	AMD Am29DL322CB90WDI
 *                      	AMD Am29DL322CB120EI/E
 *                      	AMD Am29DL322CB120WDI/E
 *       A29_Am29DL322T 	AMD Am29DL322CB70REC/I
 *                      	AMD Am29DL322CB70RWDC/I
 *                      	AMD Am29DL322CB90EI
 *                      	AMD Am29DL322CB90WDI
 *                      	AMD Am29DL322CB120EI/E
 *                      	AMD Am29DL322CB120WDI/E
 *       A29_Am29DL323B 	AMD Am29DL323CB70REC/I
 *                      	AMD Am29DL323CB70RWDC/I
 *                      	AMD Am29DL323CB90EI
 *                      	AMD Am29DL323CB90WDI
 *                      	AMD Am29DL323CB120EI/E
 *                      	AMD Am29DL323CB120WDI/E
 *       A29_Am29DL323T 	AMD Am29DL323CB70REC/I
 *                      	AMD Am29DL323CB70RWDC/I
 *                      	AMD Am29DL323CB90EI
 *                      	AMD Am29DL323CB90WDI
 *                      	AMD Am29DL323CB120EI/E
 *                      	AMD Am29DL323CB120WDI/E
 *       A29_Am29DL162B 	AMD Am29DL162CB70EI
 *                      	AMD Am29DL162CB70WCI
 *                      	AMD Am29DL162CB70REC/I
 *                      	AMD Am29DL162CB70RWCC/I
 *                      	AMD Am29DL162CB90EI
 *                      	AMD Am29DL162CB90WCI
 *                      	AMD Am29DL162CB120EI/E
 *                      	AMD Am29DL162CB120WCI/E
 *       A29_Am29DL162T 	AMD Am29DL162CT70EI
 *                      	AMD Am29DL162CT70WCI
 *                      	AMD Am29DL162CT70REC/I
 *                      	AMD Am29DL162CT70RWCC/I
 *                      	AMD Am29DL162CT90EI
 *                      	AMD Am29DL162CT90WCI
 *                      	AMD Am29DL162CT120EI/E
 *                      	AMD Am29DL162CT120WCI/E
 *       A29_Am29DL163B 	AMD Am29DL163CB70EI
 *                      	AMD Am29DL163CB70WCI
 *                      	AMD Am29DL163CB70REC/I
 *                      	AMD Am29DL163CB70RWCC/I
 *                      	AMD Am29DL163CB90EI
 *                      	AMD Am29DL163CB90WCI
 *                      	AMD Am29DL163CB120EI/E
 *                      	AMD Am29DL163CB120WCI/E
 *       A29_Am29DL163T 	AMD Am29DL163CT70EI
 *                      	AMD Am29DL163CT70WCI
 *                      	AMD Am29DL163CT70REC/I
 *                      	AMD Am29DL163CT70RWCC/I
 *                      	AMD Am29DL163CT90EI
 *                      	AMD Am29DL163CT90WCI
 *                      	AMD Am29DL163CT120EI/E
 *                      	AMD Am29DL163CT120WCI/E
 *       A29_AM29DL800B 	AMD Am29DL800BB70EC/I
 *                      	AMD Am29DL800BB70FC/I
 *                      	AMD Am29DL800BB70WBC/I
 *                      	AMD Am29DL800BB90EC/I/E
 *                      	AMD Am29DL800BB90FC/I/E
 *                      	AMD Am29DL800BB90WBC/I/E
 *                      	AMD Am29DL800BB120EC/I/E
 *                      	AMD Am29DL800BB120FC/I/E
 *                      	AMD Am29DL800BB120WBC/I/E
 *       A29_AM29DL800T 	AMD Am29DL800BT70EC/I
 *                      	AMD Am29DL800BT70FC/I
 *                      	AMD Am29DL800BT70WBC/I
 *                      	AMD Am29DL800BT90EC/I/E
 *                      	AMD Am29DL800BT90FC/I/E
 *                      	AMD Am29DL800BT90WBC/I/E
 *                      	AMD Am29DL800BT120EC/I/E
 *                      	AMD Am29DL800BT120FC/I/E
 *                      	AMD Am29DL800BT120WBC/I/E
 *       A29_AM29DL400B 	AMD Am29DL400BB70EC/I
 *                      	AMD Am29DL400BB70FC/I
 *                      	AMD Am29DL400BB80EC/I/E
 *                      	AMD Am29DL400BB80FC/I/E
 *                      	AMD Am29DL400BB90EC/I/E
 *                      	AMD Am29DL400BB90FC/I/E
 *                      	AMD Am29DL400BB120EC/I/E
 *                      	AMD Am29DL400BB120FC/I/E
 *       A29_AM29DL400T 	AMD Am29DL400BT70EC/I
 *                      	AMD Am29DL400BT70FC/I
 *                      	AMD Am29DL400BT80EC/I/E
 *                      	AMD Am29DL400BT80FC/I/E
 *                      	AMD Am29DL400BT90EC/I/E
 *                      	AMD Am29DL400BT90FC/I/E
 *                      	AMD Am29DL400BT120EC/I/E
 *                      	AMD Am29DL400BT120FC/I/E
 */


#ifndef AM29XXX
#define AM29XXX

#ifdef __linux__
#include <asm/arch/F29xxx.h>
#else
#include <F29xxx.h>
#endif /* __linux__ */


/*
 * General definitions
 */

#define A29_Am29XX32xSz	0x00400000	/* Am29XX32xX Size [byte]          */
#define A29_Am29XX16xSz	0x00200000	/* Am29XX16xX Size [byte]          */
#define A29_Am29XX800Sz	0x00100000	/* Am29XX800X Size [byte]          */
#define A29_Am29XX400Sz	0x00080000	/* Am29XX400X Size [byte]          */
#define A29_Am29XX200Sz	0x00040000	/* Am29XX200X Size [byte]          */
#define A29_Am29LV160Sz	A29_Am29XX16xSz	/* Am29LV160BX/DX Size [byte]      */
#define A29_Am29LV800Sz	A29_Am29XX800Sz	/* Am29LV800X/BX Size [byte]       */
#define A29_Am29LV400Sz	A29_Am29XX400Sz	/* Am29LV400X/BX Size [byte]       */
#define A29_Am29LV200Sz	A29_Am29XX200Sz	/* Am29LV200X/BX Size [byte]       */
#define A29_Am29DL32xSz	A29_Am29XX32xSz	/* Am29DL32xCX Size [byte]         */
#define A29_Am29DL322Sz	A29_Am29DL32xSz	/* Am29DL322CX Size [byte]         */
#define A29_Am29DL323Sz	A29_Am29DL32xSz	/* Am29DL323CX Size [byte]         */
#define A29_Am29DL16xSz	A29_Am29XX16xSz	/* Am29DL16xCX Size [byte]         */
#define A29_Am29DL162Sz	A29_Am29DL16xSz	/* Am29DL162CX Size [byte]         */
#define A29_Am29DL163Sz	A29_Am29DL16xSz	/* Am29DL163CX Size [byte]         */
#define A29_Am29DL800Sz	A29_Am29XX800Sz	/* Am29DL800BX Size [byte]         */
#define A29_Am29DL400Sz	A29_Am29XX400Sz	/* Am29DL400BX Size [byte]         */
#define A29_Am29DL322B1	0x00080000	/* Am29DL322CX Bank 1 size [byte]  */
#define A29_Am29DL322B2	        	/* Am29DL322CX Bank 2 size [byte]  */ \
                	(A29_Am29DL322Sz - A29_Am29DL322B1)
#define A29_Am29DL323B1	0x00100000	/* Am29DL323CX Bank 1 size [byte]  */
#define A29_Am29DL323B2	        	/* Am29DL323CX Bank 2 size [byte]  */ \
                	(A29_Am29DL323Sz - A29_Am29DL323B1)
#define A29_Am29DL162B1	0x00040000	/* Am29DL162CX Bank 1 size [byte]  */
#define A29_Am29DL162B2	        	/* Am29DL162CX Bank 2 size [byte]  */ \
                	(A29_Am29DL162Sz - A29_Am29DL162B1)
#define A29_Am29DL163B1	0x00080000	/* Am29DL163CX Bank 1 size [byte]  */
#define A29_Am29DL163B2	        	/* Am29DL163CX Bank 2 size [byte]  */ \
                	(A29_Am29DL163Sz - A29_Am29DL163B1)
#define A29_Am29DL800B1	0x00020000	/* Am29DL800BX Bank 1 size [byte]  */
#define A29_Am29DL800B2	        	/* Am29DL800BX Bank 2 size [byte]  */ \
                	(A29_Am29DL800Sz - A29_Am29DL800B1)
#define A29_Am29DL400B1	0x00020000	/* Am29DL400BX Bank 1 size [byte]  */
#define A29_Am29DL400B2	        	/* Am29DL400BX Bank 2 size [byte]  */ \
                	(A29_Am29DL400Sz - A29_Am29DL400B1)
#define A29_NlSctSz	F29_NlSctSz	/* Normal Sector Size [byte]       */
#define A29_SpSct0Sz	F29_SpSct0Sz	/* Special Sector 0 Size [byte]    */
                	        	/* [Am29LVxxxX only]               */
#define A29_SpSct1Sz	F29_SpSct1Sz	/* Special Sector 1 Size [byte]    */
                	        	/* [Am29LVxxxX only]               */
#define A29_SpSct2Sz	F29_SpSct2Sz	/* Special Sector 2 Size [byte]    */
                	        	/* [Am29LVxxxX only]               */
#define A29_SpSct3Sz	F29_SpSct3Sz	/* Special Sector 3 Size [byte]    */
                	        	/* [Am29LVxxxX only]               */


/*
 * Identification, erase, and program definitions
 */

#define A29_RstCmd	0x00F0  	/* Reset Command                   */
#define A29_Unlck1Add	0x00000AAA	/* Unlock cycle 1 Address          */
#define A29_Unlck1Cmd	0x00AA  	/* Unlock cycle 1 Command          */
#define A29_Unlck2Add	0x00000555	/* Unlock cycle 2 Address          */
#define A29_Unlck2Cmd	0x0055  	/* Unlock cycle 2 Command          */
#define A29_AutoSelAdd	0x00000AAA	/* Auto-Select mode Address        */
#define A29_AutoSelCmd	F29_IDCmd	/* Auto-Select mode Command        */
                	        	/* auto-select mode:               */
#define A29_ManufIDAdd	F29_ManufIDAdd	/*  Manufacturer IDentification    */
                	        	/*  Address                        */
#define A29_AMD 	0x0001  	/*   AMD                           */
#define A29_DevIDAdd	F29_DevIDAdd	/*  Device IDentification Address  */
#define A29_Am29LV160B	0x2249  	/*   Am29LV160BB/DB identification */
#define A29_Am29LV160T	0x22C4  	/*   Am29LV160BT/DT identification */
#define A29_Am29LV800B	0x225B  	/*   Am29LV800B/BB identification  */
#define A29_Am29LV800T	0x22DA  	/*   Am29LV800T/BT identification  */
#define A29_Am29LV400B	0x22BA  	/*   Am29LV400B/BB identification  */
#define A29_Am29LV400T	0x22B9  	/*   Am29LV400T/BT identification  */
#define A29_Am29LV200B	0x22BF  	/*   Am29LV200B/BB identification  */
#define A29_Am29LV200T	0x223B  	/*   Am29LV200T/BT identification  */
#define A29_Am29DL322B	0x2256  	/*   Am29DL322CB identification    */
#define A29_Am29DL322T	0x2255  	/*   Am29DL322CT identification    */
#define A29_Am29DL323B	0x2253  	/*   Am29DL323CB identification    */
#define A29_Am29DL323T	0x2250  	/*   Am29DL323CT identification    */
#define A29_Am29DL162B	0x222E  	/*   Am29DL162CB identification    */
#define A29_Am29DL162T	0x222D  	/*   Am29DL162CT identification    */
#define A29_Am29DL163B	0x222B  	/*   Am29DL163CB identification    */
#define A29_Am29DL163T	0x2228  	/*   Am29DL163CT identification    */
#define A29_Am29DL800B	0x22CB  	/*   Am29DL800BB identification    */
#define A29_Am29DL800T	0x224A  	/*   Am29DL800BT identification    */
#define A29_Am29DL400B	0x220F  	/*   Am29DL400BB identification    */
#define A29_Am29DL400T	0x220C  	/*   Am29DL400BT identification    */
#define A29_SctVerAdd	0x00000004	/*  Sector protect Verify Address  */
                	        	/*  offset (+ base sector address) */
#define A29_UnprSct	0x0000  	/*   Unprotected Sector            */
#define A29_PrSct	0x0001  	/*   Protected Sector              */
#define A29_SecSiFctAdd	0x00000006	/*  Security on Silicon Factory    */
                	        	/*  protect Address                */
                	        	/*  [Am29DLxxxCX only]             */
#define A29_SecSiUsr	0x0000  	/*   Security on Silicon User      */
                	        	/*   programmed sector             */
                	        	/*   [Am29DLxxxCX only]            */
#define A29_SecSiFct	0x0080  	/*   Security on Silicon Factory   */
                	        	/*   protected sector              */
                	        	/*   [Am29DLxxxCX only]            */
#define A29_PrgmAdd	0x00000AAA	/* Program Address                 */
#define A29_PrgmCmd	0x00A0  	/* Program Command                 */
#define A29_BypAdd	0x00000AAA	/* unlock-Bypass mode Address      */
                	        	/* [Am29LVxxxBX only]              */
#define A29_BypCmd	0x0020  	/* unlock-Bypass mode Command      */
                	        	/* [Am29LVxxxBX only]              */
                	        	/* unlock-Bypass mode:             */
#define A29_BypPrgmCmd	0x00A0  	/*  Program Command                */
                	        	/*  [Am29XXxxxBX/CX/DX only]       */
#define A29_BypRst1Cmd	0x0090  	/*  Reset cycle 1 Command          */
                	        	/*  [Am29XXxxxBX/CX/DX only]       */
#define A29_BypRst2Cmd	0x0000  	/*  Reset cycle 2 Command          */
                	        	/*  [Am29XXxxxBX/CX/DX only]       */
#define A29_ErsAdd	0x00000AAA	/* Erase Address                   */
#define A29_ErsCmd	0x0080  	/* Erase Command                   */
#define A29_ChipErsAdd	0x00000AAA	/* Chip Erase Address              */
#define A29_ChipErsCmd	0x0010  	/* Chip Erase Command              */
#define A29_SctErsCmd	0x0030  	/* Sector Erase Command            */
#define A29_ErsSusCmd	0x00B0  	/* Erase Suspend Command           */
#define A29_ErsResCmd	0x0030  	/* Erase Resume Command            */
#define A29_SecSiEntAdd	0x00000AAA	/* Security on Silicon Enter       */
                	        	/* Address [Am29DLxxxCX only]      */
#define A29_SecSiEntCmd	0x0088  	/* Security on Silicon Enter       */
                	        	/* Command [Am29DLxxxCX only]      */
#define A29_SecSiEx1Add	0x00000AAA	/* Security on Silicon Exit 1      */
                	        	/* Address [Am29DLxxxCX only]      */
#define A29_SecSiEx1Cmd	0x0090  	/* Security on Silicon Exit 1      */
                	        	/* Command [Am29DLxxxCX only]      */
#define A29_SecSiEx2Cmd	0x0000  	/* Security on Silicon Exit 2      */
                	        	/* Command [Am29DLxxxCX only]      */
#define A29_CFIQryAdd	0x000000AA	/* Common Flash memory Interface   */
                	        	/* (CFI) Query Address             */
                	        	/* [Am29XX16xX & Am29XX32xX only]  */
#define A29_CFIQryCmd	0x0098  	/* Common Flash memory Interface   */
                	        	/* (CFI) Query Command             */
                	        	/* [Am29XX16xX & Am29XX32xX only]  */


/*
 * Status polling definitions
 */

#define A29_ErsTgl	0x0004  	/* Erase Toggle bit                */
#define A29_SctErsTim	0x0008  	/* Sector Erase Timer bit          */
#define A29_ErrTim	0x0020  	/* Error Timer bit                 */
#define A29_AlgTgl	0x0040  	/* embedded Algorithm Toggle bit   */
#define A29_BsyInvData	0x0080  	/* Busy Inverted Data bit          */


#endif /* !defined (AM29XXX) */
