
/*
 *	FILE    	F29xxx.h
 *
 *	Version 	1.2
 *	Author  	Marc A. Viredaz
 *	        	Compaq Western Research Laboratory, Palo Alto, CA
 *	        	(DEC Western Research Laboratory, Palo Alto, CA)
 *	Date    	January 2000 (June 1998)
 *	System  	Portable
 *	Language	C or ARM Assembly
 *	Purpose 	Definition of constants related to the flash memories
 *	        	of the Hitachi HN29WXxxx and Motorola M29Fxxx
 *	        	families.
 *	Copyright	(c) Compaq Computer Corporation, 1998, 2000.
 *
 * Description
 *    This file defines the constants used to erase and program flash
 *    memories of the Hitachi HN29WXxxx and Motorola M29Fxxx families (see
 *    "supported devices", below). Addresses are defined for byte-mode
 *    operations (i.e., pin /BYTE set to 0) and should be shifted left by 1
 *    bit for word-mode operations (i.e., pin /BYTE set to 1). The macros
 *    "F29_8Add" and "F29_16Add" should be used for that purpose. Addresses
 *    corresponds to the address pins of the flash memories and should be
 *    appropriately shifted to account for the processor addresses. All data
 *    constants are defined for word-mode operations (i.e., pin /BYTE set to
 *    1) and should be cropped to 8 bits for byte-mode operations (i.e., pin
 *    /BYTE set to 0). The macros "F29_8Data" and "F29_16Data" should be used
 *    for that purpose.
 *
 *    Supported devices
 *
 *       F29_HN29WB800  	Hitachi HN29WB800T-8
 *                      	Hitachi HN29WB800R-8
 *                      	Hitachi HN29WB800T-10
 *                      	Hitachi HN29WB800R-10
 *                      	Hitachi HN29WB800T-12
 *                      	Hitachi HN29WB800R-12
 *       F29_HN29WT800  	Hitachi HN29WT800T-8
 *                      	Hitachi HN29WT800R-8
 *                      	Hitachi HN29WT800T-10
 *                      	Hitachi HN29WT800R-10
 *                      	Hitachi HN29WT800T-12
 *                      	Hitachi HN29WT800R-12
 *       F29_M29F800A3B 	Motorola M29F800A3BT80
 *                      	Motorola M29F800A3BR80
 *                      	Motorola M29F800A3BT10
 *                      	Motorola M29F800A3BR10
 *                      	Motorola M29F800A3BT12
 *                      	Motorola M29F800A3BR12
 *       F29_M29F800A3T 	Motorola M29F800A3UT80
 *                      	Motorola M29F800A3UR80
 *                      	Motorola M29F800A3UT10
 *                      	Motorola M29F800A3UR10
 *                      	Motorola M29F800A3UT12
 *                      	Motorola M29F800A3UR12
 */

#ifndef F29XXX
#define F29XXX


/*
 * General definitions
 */

#define F29_8Add(Val)	(Val)   	/* byte-mode (8-bit) Address       */
#define F29_16Add(Val)	((Val) >> 1)	/* word-mode (16-bit) Address      */
#define F29_8Data(Val)	        	/* byte-mode (8-bit) Data          */ \
                	((Val) BAnd ((1 << ByteWdth) - 1))
#define F29_16Data(Val)	        	/* word-mode (16-bit) Data         */ \
                	((Val) BAnd ((1 << 2*ByteWdth) - 1))

#define F29_HN29W800Sz	0x00100000	/* HN29WX800 Size [byte]           */
#define F29_M29F800A3Sz	0x00100000	/* M29F800A3X Size [byte]          */
#define F29_PgSz	0x00000100	/* Page Size [byte]                */
#define F29_NlSctSz	0x00010000	/* Normal Sector Size [byte]       */
#define F29_SpSct0Sz	(F29_NlSctSz/4)	/* Special Sector 0 Size [byte]    */
#define F29_SpSct1Sz	(F29_NlSctSz/8)	/* Special Sector 1 Size [byte]    */
#define F29_SpSct2Sz	(F29_NlSctSz/8)	/* Special Sector 2 Size [byte]    */
#define F29_SpSct3Sz	(F29_NlSctSz/2)	/* Special Sector 3 Size [byte]    */


/*
 * Identification, erase, and program definitions
 */

#define F29_MemRdCmd	0x00FF  	/* flash-Memory Read Command       */
#define F29_IDCmd	0x0090  	/* IDentification mode Command     */
                	        	/* identification mode:            */
#define F29_ManufIDAdd	0x00000000	/*  Manufacturer IDentification    */
                	        	/*  Address                        */
#define F29_Hitachi	0x0707  	/*   Hitachi                       */
#define F29_Motorola	0x1C1C  	/*   Motorola                      */
#define F29_DevIDAdd	0x00000002	/*  Device IDentification Address  */
#define F29_HN29WB800	0x8686  	/*   HN29WB800 identification      */
#define F29_HN29WT800	0x8585  	/*   HN29WT800 identification      */
#define F29_M29F800A3B	0x5E5E  	/*   M29F800A3B identification     */
#define F29_M29F800A3U	0x5D5D  	/*   M29F800A3U identification     */
#define F29_StatRdCmd	0x0070  	/* Status register Read Command    */
#define F29_StatClrCmd	0x0050  	/* Status register Clear Command   */
#define F29_CnfrmCmd	0x00D0  	/* Confirm Command                 */
#define F29_PgPrgmCmd	0x0041  	/* Page Program Command            */
#define F29_ChipErsCmd	0x00A7  	/* Chip Erase Command (unlocked    */
                	        	/* sectors)                        */
#define F29_SctErsCmd	0x0020  	/* Sector Erase Command            */
#define F29_LckRdCmd	0x0071  	/* Locked status Read Command      */
#define F29_LckPrgmCmd	0x0077  	/* Locked status Program Command   */
#define F29_SusCmd	0x00B0  	/* erase/program Suspend Command   */
#define F29_ResCmd	0x00D0  	/* erase/program Resume Command    */
#define F29_SleepCmd	0x00F0  	/* Sleep Command [M29FxxxA3X only] */


/*
 * Status register definitions
 */

#define F29_Sleep	0x01    	/* Sleep bit [M29FxxxA3X only]     */
#define F29_SctPrgmErr	0x08    	/* Sector Program Error bit        */
#define F29_PrgmErr	0x10    	/* Program Error bit               */
#define F29_ErsErr	0x20    	/* Erase Error bit                 */
#define F29_Sus 	0x40    	/* Suspended bit                   */
#define F29_Rdy 	0x80    	/* Ready bit                       */


#endif /* !defined (F29XXX) */
