/* Copyright (c) Compaq Computer Corporation, 1999 */

#ifndef __ASM_ARCH_DMA_H
#define __ASM_ARCH_DMA_H

#ifdef CONFIG_ARCH_BRUTUS
/* The SA1100 has its own DMA Controller that supports the on-chip 
 * serial controllers only. These are the addresses of each peripheral
 * for programming the DMA controller. You may select the endianess with
 * the DMA_DEVICE_ADDRESS macro below.
 */
#define DMA_UDC_TRANSMIT                0x80000A04
#define DMA_UDC_RECEIVE                 0x80000A15
#define DMA_SDLC_TRANSMIT               0x80801E20
#define DMA_SDLC_RECEIVE                0x80801E31
#define DMA_SERIAL1_UART_TRANSMIT       0x80400540
#define DMA_SERIAL1_UART_RECEIVE        0x80400551
#define DMA_SERIAL2_UART_TRANSMIT       0x81001B60
#define DMA_SERIAL2_UART_RECEIVE        0x81001B71
#define DMA_HSSP_TRANSMIT               0x80C00564
#define DMA_HSSP_RECEIVE                0x80C00575
#define DMA_SERIAL3_UART_TRANSMIT       0x81400580
#define DMA_SERIAL3_UART_RECEIVE        0x81400591
#define DMA_MCP_TRANSMIT_AUDIO          0x818002A8
#define DMA_MCP_RECEIVE_AUDIO           0x818002B9
#define DMA_MCP_TRANSMIT_TELECON        0x818003C8
#define DMA_MCP_RECEIVE_TELECON         0x818003D9
#define DMA_SSP_TRANSMIT                0x81C01BE8
#define DMA_SSP_RECEIVE                 0x81C01BF9

#define DMA_DEVICE_ADDRESS(address, endianness) (endianess) ? \
                 ((address) | 0x02) : \
                 (address)

#endif

/* enable/disable a specific DMA channel */
extern void enable_dma(unsigned int dmanr);

#ifdef CONFIG_ARCH_BRUTUS
extern void disable_dma(unsigned int dmanr);
extern void set_dma_mode_sa1100 (unsigned int dmanr, unsigned int device);
extern unsigned int get_dma_address_sa1100(unsigned int dmanr);
#else
static __inline__ void disable_dma(unsigned int dmanr)
{
    printk("disable_dma: dma %d not supported\n", dmanr);
}

/* set mode (above) for a specific DMA channel */
extern void set_dma_mode(unsigned int dmanr, char mode);
#endif

/* Clear the 'DMA Pointer Flip Flop'.
 * Write 0 for LSB/MSB, 1 for MSB/LSB access.
 * Use this once to initialize the FF to a known state.
 * After that, keep track of it. :-)
 * --- In order to do that, the DMA routines below should ---
 * --- only be used while interrupts are disabled! ---
 */
static __inline__ void clear_dma_ff(unsigned int dmanr)
{
    printk("clear_dma_ff: dma %d not supported\n", dmanr);
}

/* Set only the page register bits of the transfer address.
 * This is used for successive transfers when we know the contents of
 * the lower 16 bits of the DMA current address register, but a 64k boundary
 * may have been crossed.
 */
static __inline__ void set_dma_page(unsigned int dmanr, char pagenr)
{
    printk("set_dma_page: dma %d not supported\n", dmanr);
}


/* Set transfer address & page bits for specific DMA channel.
 * Assumes dma flipflop is clear.
 */
extern void set_dma_addr(unsigned int dmanr, unsigned int addr);

/* Set transfer size for a specific DMA channel.
 */
extern void set_dma_count(unsigned int dmanr, unsigned int count);

/* Get DMA residue count. After a DMA transfer, this
 * should return zero. Reading this while a DMA transfer is
 * still in progress will return unpredictable results.
 * If called before the channel has been used, it may return 1.
 * Otherwise, it returns the number of _bytes_ left to transfer.
 *
 * Assumes DMA flip-flop is clear.
 */
extern int get_dma_residue(unsigned int dmanr);

/* YYY don't know why I needed to add these, but I did */

static __inline__ void arch_enable_dma(unsigned int dmanr)
{
    printk("arch_enable_dma: dma %d not supported\n", dmanr);
}
static __inline__ void arch_set_dma_mode(unsigned int dmanr, char mode)
{
  printk("arch_set_dma_mode: dma %d not supported\n", dmanr);
}
static __inline__ void arch_set_dma_addr(unsigned int dmanr, unsigned int addr)
{
  printk("arch_set_dma_addr: dma %d not supported\n", dmanr);
}
static __inline__ void arch_set_dma_count(unsigned int dmanr, unsigned int count)
{
  printk("arch_set_dma_count: dma %d not supported\n", dmanr);
}
static __inline__ int arch_dma_count(unsigned int dmanr)
{
  printk("arch_dma_count: dma %d not supported\n", dmanr);
  return(0);
}

#endif /* _ASM_ARCH_DMA_H */

