/*
 * linux/include/asm-arm/arch-brutus/hardware.h
 *
 * Copyright (c) Compaq Computer Corporation, 1998
 *
 * Author: Deborah A. Wallach
 * Based on arch-rpc/hardware.h written by Russell King.
 *
 * This file contains the hardware definitions of Brutus.
 */

#ifndef __ASM_ARCH_HARDWARE_H
#define __ASM_ARCH_HARDWARE_H

/*
 * What hardware must be present
 */

#ifndef __ASSEMBLER__

/*
 * for use with inb/outb
 */
/*  totally wrong, who knows what they should be?  is this page table defn? */
#define VIDC_BASE		0x80100000  /* used in drivers/char/conso*.c  YYY */


/*
 * IO definitions
 */
/* all in io_reg.h now */

/*
 * Mapping areas
 */
/* io_base/io_end need to be a _lot_ bigger to hold our io space 
 * we finagle this by compressing down the space.
 * note that mapping isn't 1-1 anymore
 */
/* base, end are virtual; start is physical */
#define IO_END			0xffffffff
#define IO_BASE			0xe0000000
#define IO_SIZE_V		(IO_END - IO_BASE)
#define IO_SIZE_P		(2*(IO_END - IO_BASE))
#define IO_START		0x80000000

#define MAPABLE_MEM_END         0xdfe00000 /* exclusive (top page already mapped?) */
#define MAPABLE_MEM_BASE        0xd0000000
#define MAPABLE_MEM_SIZE        (MAPABLE_MEM_END - MAPABLE_MEM_BASE)


/* comments pertain to itsy. otherwise just static memory bank mappings */
#define FLASH0                  0x00000000 /* mirror of flash 1 or flash 2 */          
#define FLASH1                  0x08000000 /* mother board flash */
#define FLASH2                  0x10000000 /* daughter card flash */
#define FLASH3                  0x18000000 /* unused */

/*
 * Screen mapping information
 */
/* YYY should only matter if HAVE_MAP_VID_MEM is defined */
/* used in mmap.h, drivers/char/console.c */
#define SCREEN2_END		0xe0000000
#define SCREEN2_BASE		0xd8000000
#define SCREEN1_END		SCREEN2_BASE
#define SCREEN1_BASE		0xd0000000
#define SCREEN_START		0x02000000	/* VRAM */

/*
 * RAM definitions
 */
#define MAPTOPHYS(x)		(x) 
#define KERNTOPHYS(x)		((unsigned long)(&x))
#define GET_MEMORY_END(p)	(PAGE_OFFSET + p->page_size * \
						(p->pages_in_bank[0] + \
						 p->pages_in_bank[1] + \
						 p->pages_in_bank[2] + \
						 p->pages_in_bank[3]))

#define PARAMS_BASE		(PAGE_OFFSET + 0x0100)

#else

#define VIDC_BASE		0xe0400000 /* YYY */
#define IO_BASE			0xe0000000 /* YYY should this be the same as kernel? */

#endif
#endif

