/*
 * linux/include/asm-arm/arch/io_reg.h
 *
 * Copyright (c) Compaq Computer Corporation, 1997
 *
 * Author: Deborah A. Wallach
 *
 * Changelog:
 *  03-12-1997	kerr	Created
 */
#ifndef __ASM_ARCH_IO_REG_H
#define __ASM_ARCH_IO_REG_H

#define GAFR_P          0x9004001c    /* GPIO alternative function register */
#define GPDR_P          0x90040004    /* GPIO pin direction register */
#define PPAR_P          0x90060008    /* PPAR pin assignment register */
#define GAFR_V          0xe804001c    /* GPIO alternative function register */
#define GPDR_V          0xe8040004    /* GPIO pin direction register */
#define PPAR_V          0xe8060008    /* PPAR pin assignment register */
#define SDCR0_P		0x80020060    /* SDCR0 configuration register */
#define SDCR0_V		0xe0020060    /* SDCR0 configuration register */
#define GPSR_P		0x90040008    /* GPSR pin set register */
#define GPSR_V		0xe8040008    /* GPSR pin set register */
#define GPCR_P		0x9004000c    /* GPCR pin clear register */
#define GPCR_V		0xe804000c    /* GPCR pin clear register */



/* physical */
#define IRQ_ICIP_P   	((volatile unsigned long *) 0x90050000)	/* IC IRQ Pending Reg */
#define IRQ_ICMR_P   	((volatile unsigned long *) 0x90050004)	/* IC Mask Reg  */
#define IRQ_ICLR_P   	((volatile unsigned long *) 0x90050008)	/* IC Level Reg */
#define IRQ_ICCR_P   	((volatile unsigned long *) 0x9005000C)	/* IC Control Reg */
#define IRQ_ICFP_P   	((volatile unsigned long *) 0x90050010)	/* IC FIQ Pending Reg */
#define IRQ_ICPR_P   	((volatile unsigned long *) 0x90050020)	/* IC Pending Reg */

/* virtual */
#define IRQ_ICIP_V   	((volatile unsigned long *) 0xe8050000)	/* IC IRQ Pending Reg */
#define IRQ_ICMR_V    	((volatile unsigned long *) 0xe8050004)	/* IC Mask Reg  */
#define IRQ_ICLR_V   	((volatile unsigned long *) 0xe8050008)	/* IC Level Reg */
#define IRQ_ICCR_V   	((volatile unsigned long *) 0xe805000C)	/* IC Control Reg */
#define IRQ_ICFP_V   	((volatile unsigned long *) 0xe8050010)	/* IC FIQ Pending Reg */
#define IRQ_ICPR_V   	((volatile unsigned long *) 0xe8050020)	/* IC Pending Reg */


/* base addresses for UARTs for serial ports */

#define UART_SERIAL3_BASE_P     ((volatile unsigned long *) 0x80050000)
#define UART_SERIAL1_BASE_P     ((volatile unsigned long *) 0x80010000)

#define UART_SERIAL3_BASE_V     ((volatile unsigned long *) 0xe0050000)
#define UART_SERIAL1_BASE_V     ((volatile unsigned long *) 0xe0010000)


/* addresses for OS Timer */
#define OSMR0_P         0x90000000 /* OS Timer Match Register 0 */ 
#define OSMR1_P         0x90000004 /* OS Timer Match Register 1 */ 
#define OSMR2_P         0x90000008 /* OS Timer Match Register 2 */ 
#define OSMR3_P         0x9000000c /* OS Timer Match Register 3 */ 
#define OSCR_P          0x90000010 /* OS Timer Counter Register */ 
#define OSSR_P          0x90000014 /* OS Timer Status Register */ 
#define OWER_P          0x90000018 /* OS Timer Watchdog Enable Register */ 
#define OIER_P          0x9000001c /* OS Timer Interrupt Enable Register */ 

#define OSMR0_V         0xe8000000 /* OS Timer Match Register 0 */ 
#define OSMR1_V         0xe8000004 /* OS Timer Match Register 1 */ 
#define OSMR2_V         0xe8000008 /* OS Timer Match Register 2 */ 
#define OSMR3_V         0xe800000c /* OS Timer Match Register 3 */ 
#define OSCR_V          0xe8000010 /* OS Timer Counter Register */ 
#define OSSR_V          0xe8000014 /* OS Timer Status Register */ 
#define OWER_V          0xe8000018 /* OS Timer Watchdog Enable Register */ 
#define OIER_V          0xe800001c /* OS Timer Interrupt Enable Register */ 


/* addresses for RTC */
#define RCNR_P          0x90010004 /* RTC Count Register */
#define RTAR_P          0x90010000 /* RTC Alarm Register */
#define RTSR_P          0x90010010 /* RTC Status Register */
#define RTTR_P          0x90010008 /* RTC Timer Trim Register */

#define RCNR_V          0xe8010004 /* RTC Count Register */
#define RTAR_V          0xe8010000 /* RTC Alarm Register */
#define RTSR_V          0xe8010010 /* RTC Status Register */
#define RTTR_V          0xe8010008 /* RTC Timer Trim Register */

/* addresses for reset controller */
#define RSRR_P          0x90030000  /* Reset controller software reset register */ 
#define RCSR_P          0x90030004  /* Reset controller status register */ 

#define RSRR_V          0xe8030000  /* Reset controller software reset register */ 
#define RCSR_V          0xe8030004  /* Reset controller status register */ 

/* addresses for power management */

#define PMCR_P          0x90020000  /* Power manager control register */
#define PSSR_P          0x90020004  /* Power manager sleep status register */
#define PSPR_P          0x90020008  /* Power manager scratch pad register */
#define PWER_P          0x9002000c  /* Power manager wakeup enable register */
#define PCFR_P          0x90020010  /* Power manager general configuration register */
#define PPCR_P          0x90020014  /* Power manager PLL configuration register */
#define PGSR_P          0x90020018  /* Power manager GPIO sleep state register */
#define POSR_P          0x9002001c  /* Power manager oscillator status register */

#define PMCR_V          0xe8020000  /* Power manager control register */
#define PSSR_V          0xe8020004  /* Power manager sleep status register */
#define PSPR_V          0xe8020008  /* Power manager scratch pad register */
#define PWER_V          0xe802000c  /* Power manager wakeup enable register */
#define PCFR_V          0xe8020010  /* Power manager general configuration register */
#define PPCR_V          0xe8020014  /* Power manager PLL configuration register */
#define PGSR_V          0xe8020018  /* Power manager GPIO sleep state register */
#define POSR_V          0xe802001c  /* Power manager oscillator status register */

#define PPCR_MASK       0x1f        /* Mask representing non-reserved portion of PPCR */

#define GPLR_P          0x90040000  /* GPIO Pin level Register */
#define GRER_P          0x90040010  /* GPIO Rising-Edge detect Register */
#define GFER_P          0x90040014  /* GPIO Falling-Edge detect Register */
#define GEDR_P          0x90040018  /* GPIO Edge Detect status Register */

#define GPLR_V          0xe8040000  /* GPIO Pin level Register */
#define GRER_V          0xe8040010  /* GPIO Rising-Edge detect Register */
#define GFER_V          0xe8040014  /* GPIO Falling-Edge detect Register */
#define GEDR_V          0xe8040018  /* GPIO Edge Detect status Register */

/* addresses for memory configuration */

#define MDCNFG_P        0xa0000000 /* DRAM configuration register */
#define MDCAS0_P        0xa0000004 /* DRAM CAS waveform shift register 0 */
#define MDCAS1_P        0xa0000008 /* DRAM CAS waveform shift register 1 */
#define MDCAS2_P        0xa000000c /* DRAM CAS waveform shift register 2 */
#define MSC0_P          0xa0000010 /* Static memory control register 0 */
#define MSC1_P          0xa0000014 /* Static memory control register 1 */

#define MDCNFG_V        0xf0000000 /* DRAM configuration register */
#define MDCAS0_V        0xf0000004 /* DRAM CAS waveform shift register 0 */
#define MDCAS1_V        0xf0000008 /* DRAM CAS waveform shift register 1 */
#define MDCAS2_V        0xf000000c /* DRAM CAS waveform shift register 2 */
#define MSC0_V          0xf0000010 /* Static memory control register 0 */
#define MSC1_V          0xf0000014 /* Static memory control register 1 */

#define MDCNFG_DE_MASK  0x0000000f /* Mask of DE field of MDCNFG */
#define MSC_BANK_MASK   0x0000ffff /* Mask for a single bank of static memory */
#define MSC_INVALID     0x0000fffc /* Pattern for an invalid MSC entry */

#endif
