/*
 * linux/include/asm-arm/arch-brutus/mmap.h
 *
 * Copyright (c) Compaq Computer Corporation, 1998
 *
 * Author: Deborah A. Wallach
 */

/* #define HAVE_MAP_VID_MEM ZZZ */

#ifndef mmap_h
#define mmap_h

#include <linux/config.h>
#include <asm/pgtable.h>

#define SAFE_ADDR	0xe0000000	/* SA1100 Zero Bank */

#ifdef	CONFIG_ARCH_BRUTUS
/*
 * I/O mappings
 *
 */

/* points to current free part of mapable memory */
extern unsigned long current_mapable_pointer; 
/* mapping for use of io_p2v and io_v2p */
extern unsigned long io_pgtable[MAPABLE_MEM_SIZE>>PGDIR_SHIFT];

static inline unsigned long allocate_vspace(unsigned long p_orig,
					    unsigned long size,
					    int write, 
					    int buffer)
{
  unsigned long p_end, p;
  int access = PMD_SECT_AP_READ;
  int spi = current_mapable_pointer>>PGDIR_SHIFT;

  /* round down to nearest section */
  p = p_orig & PGDIR_MASK;
  p_end = p + size;

  /* must allocate in units of PGDIR_SIZE */
  if (size & ~PGDIR_MASK) return(0); 

  if ((current_mapable_pointer + size) >= MAPABLE_MEM_END) return(0);
  if (write)  access |= PMD_SECT_AP_WRITE;
  if (buffer) access |= PMD_SECT_BUFFERABLE;

  while (p < p_end) {
    pgd_val(swapper_pg_dir[spi++]) = p |
	PMD_TYPE_SECT | PMD_DOMAIN(DOMAIN_KERNEL) |
	    access;
    io_pgtable[(current_mapable_pointer-MAPABLE_MEM_BASE)>>PGDIR_SHIFT] = p;
    p += PGDIR_SIZE;
    current_mapable_pointer += PGDIR_SIZE;
  }	

  /* ensure modifications visible: flush cache, write buffer */
  flush_cache_all();

  return(current_mapable_pointer+(p_orig&(~PGDIR_MASK))-size);
}



static inline unsigned long io_v2p(unsigned long v)
{
  if(v>=0xe0000000) {
    /* IO Register Space */
    switch ((v >> 27) & 3) {
    case 0:
      return(v - 0x60000000);
    case 1:
      return(v - 0x58000000);
    case 2:
      return(v - 0x50000000);
    case 3:
      return(v - 0x48000000);
    default:
      return(SAFE_ADDR);
    }
  }
  else {
    if ((v >= MAPABLE_MEM_BASE) && (v <= MAPABLE_MEM_END)) {
      return(io_pgtable[(v-MAPABLE_MEM_BASE)>>PGDIR_SHIFT] | (v&(~PGDIR_MASK)));
    }
    return(SAFE_ADDR);
  }
}

static inline unsigned long io_p2v(unsigned long p)
{
  if((p>=IO_START) && (p < IO_START + IO_SIZE_P)) {
    /* IO Register Space */
    switch ((p >> 28) & 3) {
    case 0:
      return(p + 0x60000000);
    case 1:
      return(p + 0x58000000);
    case 2:
      return(p + 0x50000000);
    case 3:
      return(p + 0x48000000);
    default:
      /* virtual address of SAFE_ADDR */
      return(0xdf000000);
    }
  }
  else {
    unsigned long spi = 0;
    unsigned long cp = p & PGDIR_MASK;
    while((spi<<PGDIR_SHIFT) < current_mapable_pointer) {
      if (io_pgtable[spi] == cp) return(MAPABLE_MEM_BASE+(spi<<PGDIR_SHIFT)+(p & ~PGDIR_MASK));
      spi++;
    }
    /* virtual address of SAFE_ADDR */
    return(0xdf000000);
  }
}
#endif	/* CONFIG_ARCH_BRUTUS */

#endif
