/*
 * SA1100 Registers Header File
 *
 * Copyright (c) Compaq Computer Corporation, 1998
 *
 * Author: Lawrence S. Brakmo
 *
 * History:
 *
 * 	6-Jan-98	First version started, taken from "brutus.h"
 */

#ifndef sa1100_h
#define sa1100_h

#define EXT(d, b) ((((unsigned int)(d))>>(b))&1)
#define EXTV(d, hb, lb)\
 (((((unsigned int)(d))>>(lb))&((((unsigned int)0xffffffff)<<(31-(hb)))>>((lb)+(31-(hb))))) & 0xffffffff)

/*
 * XBOX GPIO Control Registers (Brutus specific?)
 *
 */
#define XGP_BASE    0x100A0000 /* XBOX GP Base         */
#define XGP_SET_REG 0x100A0000 /* XBOX GP Set Reg      */
#define XGP_RD_CNT  0x100A0004 /* XBOX GP Count to Set */
#define XGP_RD_SET  0x100A0008 /* XBOX GP Read Set     */
#define XGP_RD_GP   0x100A000C /* XBOX Read Current GP */
#define XGP_SET_PUL 0x100A0010 /* XBOX Set GP Pulse    */
 
/*
 * System Control Module GPIO Register Locations
 *
 */
#define GP_BASE_P     0x90040000 /*  SCM GP BASE         */
#define GP_PIN_LVL_P  0x90040000 /*  Pin Level           */
#define GP_PIN_DIR_P  0x90040004 /*  Direction           */
#define GP_PIN_SET_P  0x90040008 /*  Output Set          */
#define GP_PIN_CLR_P  0x9004000C /*  Output Clr          */
#define GP_PIN_RED_P  0x90040010 /*  Rising Edge Detect  */
#define GP_PIN_FED_P  0x90040014 /*  Falling Edge Detect */
#define GP_PIN_EDS_P  0x90040018 /*  Edge Detect Status  */

/*
 * System Control Module IC Register Locations
 *
 */
#define IC_BASE_P     0x90050000 /*  SCM IC Base      */
#define IC_PEND_P     0x90050000 /*  Pending          */
#define IC_MASK_P     0x90050004 /*  Mask             */
#define IC_LEVEL_P    0x90050008 /* Level 0-IRQ 1-FIQ */

/*
 * PPC Registers
 *
 */
#define PPC_PIN_DIR_P		0x90060000
#define PPC_PIN_STATE_P		0x90060004
#define PPC_PIN_ASSIGN_P	0x90060008
#define PPC_SLEEP_DIR_P		0x9006000C
#define PPC_PIN_FLAG_P		0x90060010
#define PPC_MCP_REG1_P		0x90060030

#define CLEAR       0xFFFFFFFF /*  Clear Reg Pattern  */

#define GPIO_LOWEST_OR  11                    /* GP Lowest OR    	*/
#define GPIO_INTOR_MASK (1<<GPIO_LOWEST_OR)   /* GP[27:GPIOLOWESTOR] Int Num */
#define GPIO_INT_MASK \
            (0xFFFFFFFF>>(31-GPIO_LOWEST_OR)) /* GP Interrupt Mask  */

#define IRQ         0x1
#define FIQ         0x2

struct int_struct {
  int ic_pending1;    // RW
  int ic_pending2;    // RW
  int qdone;          // RW
  int gp_reset_state; // RW
  int gp_state;       // RW
};

struct xbox_gp_regs {
  int xgpsr;        // W
  int xgprdcnt;     // R
  int xgprdset;     // R
  int xgprdgp;      // R
  int xgpsetpul;    // W
};

struct scm_gpio_regs {
  int gp_pinlevel;  // R
  int gp_pindir;    // RW
  int gp_pinset;    // W
  int gp_pinclr;    // W
  int gp_pinred;    // RW
  int gp_pinfed;    // RW
  int gp_pineds;    // RW
  int gp_altfunc;   // RW
};

struct scm_ic_regs {
  int ic_irq_pending; // R
  int ic_mask;        // RW
  int ic_level;       // RW
  int reserved;       // --
  int ic_fiq_pending; // R
};

#define VIDEO_BASE_P	0xB0100000

struct lcd_regs {
  int Control0;     /* LCCR0 */
  int Status;       /* LCSR  */
  int Reserved1;    /* Pad   */
  int Reserved2;    /* Pad   */
  int DMA1Base;     /* DBAR1 */
  int DMA1Current;  /* DCAR1 */
  int DMA2Base;     /* DBAR2 */
  int DMA2Current;  /* DCAR2 */
  int Control1;     /* LCCR1 */
  int Control2;     /* LCCR2 */
  int Control3;     /* LCCR3 */
};

/*
 * LCD Control Register 0 Bits
 *
 */
#define LCD0_M_LEN 0x00000001
#define LCD0_M_CMS 0x00000002
#define LCD0_M_SDS 0x00000004
#define LCD0_M_PAS 0x00000080
#define LCD0_M_BLE 0x00000100
#define LCD0_M_DPD 0x00000200
#define LCD0_M_FDD 0x000FF000
#define LCD0_V_LEN 0
#define LCD0_V_CMS 1
#define LCD0_V_SDS 2
#define LCD0_V_PAS 7
#define LCD0_V_BLE 8
#define LCD0_V_DPD 9
#define LCD0_V_FDD 12

/*
 * LCD control Register 1 Bits
 *
 */
#define LCD1_M_PPL 0x000003FF
#define LCD1_M_HSW 0x0000FC00
#define LCD1_M_ELW 0x00FF0000
#define LCD1_M_BLW 0xFF000000
#define LCD1_V_PPL 0
#define LCD1_V_HSW 10
#define LCD1_V_ELW 16
#define LCD1_V_BLW 24

/*
 * LCD control Register 2 Bits
 *
 */
#define LCD2_M_LPP 0x000003FF
#define LCD2_M_VSW 0x0000FC00
#define LCD2_M_EFW 0x00FF0000
#define LCD2_M_BFW 0xFF000000
#define LCD2_V_LPP 0
#define LCD2_V_VSW 10
#define LCD2_V_EFW 16
#define LCD2_V_BFW 24

/*
 * LCD control Register 2 Bits
 *
 */
#define LCD3_M_PCD 0x000000FF
#define LCD3_M_ACB 0x0000FF00
#define LCD3_M_API 0x000F0000
#define LCD3_M_VSP 0x00100000
#define LCD3_M_HSP 0x00200000
#define LCD3_M_PCP 0x00400000
#define LCD3_M_OEP 0x00800000
#define LCD3_V_PCD 0
#define LCD3_V_ACB 8
#define LCD3_V_API 16
#define LCD3_V_VSP 20
#define LCD3_V_HSP 21
#define LCD3_V_PCP 22
#define LCD3_V_OEP 23

struct sspreg {
	int sscr0;
	int sscr1;
	int ssppad0;
	int ssdr;
	int ssppad1;
	int sssr;
};

/*
 * SSP CR0
 *
 */
#define SSPCR0_M_DSS 0x0000000F
#define SSPCR0_M_FRF 0x00000030
#define SSPCR0_M_SSE 0x00000080
#define SSPCR0_M_SCR 0x0000FF00
#define SSPCR0_V_DSS 0
#define SSPCR0_V_FRF 4
#define SSPCR0_V_SSE 7
#define SSPCR0_V_SCR 8

/*
 * SSP CR1
 *
 */
#define SSPCR1_M_RIM 0x00000001
#define SSPCR1_M_TIN 0x00000002
#define SSPCR1_M_LBM 0x00000004
#define SSPCR1_V_RIM 0
#define SSPCR1_V_TIN 1
#define SSPCR1_V_LBM 2

/*
 * SSP status
 *
 */
#define SSPSR_M_TNF 0x00000002
#define SSPSR_M_RNE 0x00000004
#define SSPSR_M_BSY 0x00000008
#define SSPSR_M_TFS 0x00000010
#define SSPSR_M_RFS 0x00000020
#define SSPSR_M_ROR 0x00000040
#define SSPSR_V_TNF 1
#define SSPSR_V_RNE 2
#define SSPSR_V_BSY 3
#define SSPSR_V_TFS 4
#define SSPSR_V_RFS 5
#define SSPSR_V_ROR 6

#define SSP_BASE_P 0x80070060


struct mcpreg {
	int mccr;
	int rsv0;
	int mcdr0;
	int mcdr1;
	int mcdr2;
	int rsv1;
	int mcsr;
};

#define MCPBASE_P 0x80060000

#define MCCR_V_ASD 0
#define MCCR_V_TSD 8
#define MCCR_M_MCE 0x010000
#define MCCR_M_ECS 0x020000
#define MCCR_M_ADM 0x040000
#define MCCR_M_TTM 0x080000
#define MCCR_M_TRM 0x100000
#define MCCR_M_ATM 0x200000
#define MCCR_M_ARM 0x400000
#define MCCR_M_LBM 0x800000

#define MCDR2_V_RN 17
#define MCDR2_M_nRW 0x10000

#define MCSR_M_ATS 0x00000001
#define MCSR_M_ARS 0x00000002
#define MCSR_M_TTS 0x00000004
#define MCSR_M_TRS 0x00000008
#define MCSR_M_ATU 0x00000010
#define MCSR_M_ARO 0x00000020
#define MCSR_M_TTU 0x00000040
#define MCSR_M_TRO 0x00000080
#define MCSR_M_ANF 0x00000100
#define MCSR_M_ANE 0x00000200
#define MCSR_M_TNF 0x00000400
#define MCSR_M_TNE 0x00000800
#define MCSR_M_CWC 0x00001000
#define MCSR_M_CRC 0x00002000
#define MCSR_M_ACE 0x00004000
#define MCSR_M_TCE 0x00008000

#endif
