/*
 * bulkmem_itsy.h
 * Implementation of the PCMCIA Card Services "bulk memory"
 * functionality for the Itsy onboard and daughtercard flash, but
 * without the Card Services infrastructure.
 * Tim Mann 3-18-99
 *
 * Copyright (c) Compaq Computer Corporation, 1999
 */

/* Debugging compiled in but turned off;
   turn on with module parameter ftl_itsy_debug= */
#define FTL_ITSY_DEBUG 0

/* Itsy has up to 3 real flash banks, number 1 to 3. */
#define FIRST_BANK 1
#define LAST_BANK 3
#define N_BANKS (LAST_BANK - FIRST_BANK + 1)

/* Region sizes in an Itsy flash bank.  There are at most MAX_REGION.
   The regions have start and size defined by the region sizes array,
   an array of struct region_desc_t's of size MAX_REGION that is
   stored at REGION_SIZES_OFFSET from the end of the bank.  The zeroth
   region is always the entire bank (except that the last erase sector
   is accessible only through the raw character device, not the ftl
   block device); its descriptor is ignored.  If a region with size
   0xffff is found, this terminates the list. */
#define MAX_REGION 4
#define REGION_SIZES_OFFSET (0x100 - 0x98)  /* byte offset from end of bank */
typedef struct region_desc_t {
  __u16 start;  /* number of erase sectors from start of bank */
  __u16 size;   /* number of erase sectors in region */
} region_desc_t;

/* We use client_handle_t to hold the StrongARM bank number */
typedef void* client_handle_t;
/* We use memory_handle_t to hold a bank's base virtual address */
typedef void* memory_handle_t;
/* We just use the corresponding client_handle_t here */
typedef void* eraseq_handle_t;

typedef struct event_callback_args_t {
    client_handle_t client_handle;
    void        *info;
    void        *client_data;
} event_callback_args_t;

typedef struct client_reg_t {
    int (*event_handler)(event_callback_args_t *);
    event_callback_args_t event_callback_args;
} client_reg_t;

#include "bulkmem.h"

int BMIRegisterClient(client_handle_t client, client_reg_t *reg);
int BMIDeregisterClient(client_handle_t client);
int BMIGetFirstRegion(client_handle_t handle, region_info_t *region);
int BMIGetNextRegion(client_handle_t handle, region_info_t *region);
int BMIOpenMemory(client_handle_t *handle, open_mem_t *req);
int BMICloseMemory(memory_handle_t handle);
int BMIReadMemory(memory_handle_t handle, mem_op_t *req, caddr_t buf);
int BMIWriteMemory(memory_handle_t handle, mem_op_t *req, caddr_t buf);
int BMIRegisterEraseQueue(client_handle_t *handle, eraseq_hdr_t *header);
int BMIDeregisterEraseQueue(eraseq_handle_t handle);
int BMICheckEraseQueue(eraseq_handle_t handle);
