/*
 * SA1100 GPIO Driver
 * Copyright (c) Compaq Computer Corporation, 1999
 *
 * Author: Jason Flinn
 * 
 * This interfaces with the "unused" pins of the SA1100 GPIO.  These can
 * be used for various purposes, such as supplementing profile information. 
 *
 * Implementation borrowed from clkspeed-sa100.h.
 */

#ifndef	gpio_sa1100_h
#define	gpio_sa1100_h

/* Pins 11-13 on the SA-1100 are essentially unused, and can be wired
 * up for various purposes (such as assisting profiling).  This driver supports
 * I/O using these pins.
 *
 * Note: This shares the GPIO 11-27 interrupt with rocknscroll, which
 * should not be a problem.  
 *
 * Getting information:
 *   GPIO_GET_PIN_VALUES -- returns the current value of these pins.
 *
 * Setting information
 *   GPIO_SET_PIN_DIRECTION -- configures the pins for input or output.  If
 *   the value passed in for a particular pin is not equal to GPIO_DIR_INPUT
 *   or GPIO_DIR_OUTPUT, then the pin direction is not modified.
 *   
 *   GPIO_SET_PIN_VALUE -- sets the value of the pins.  If the value passed
 *   in for a particular pin is not 0 or 1, then the value of the pin is not
 *   modified.
 *
 *   GPIO_WAIT_FOR_PIN_VALUES -- blocks until the pin values match the 
 *   specified values.  If a value other than 0 or 1 is specified for a 
 *   particular pin, then that pin is not checked before returning.
 */

struct gpio_t {
  char gpio_11;
  char gpio_12;
  char gpio_13;
};
  
#define GPIO_DIR_INPUT     0
#define GPIO_DIR_OUTPUT    1
#define GPIO_DIR_DONT_CARE 2

/* driver ioctl type */
#define	GPIO_IOCTL_TYPE	'g'

/* ioctl commands */
#define	GPIO_SET_PIN_DIRECTION      _IOW(GPIO_IOCTL_TYPE, 0, struct gpio_t)
#define	GPIO_SET_PIN_VALUES	    _IOW(GPIO_IOCTL_TYPE, 1, struct gpio_t)
#define	GPIO_GET_PIN_VALUES         _IOR(GPIO_IOCTL_TYPE, 2, struct gpio_t)
#define	GPIO_WAIT_FOR_PIN_VALUES    _IOW(GPIO_IOCTL_TYPE, 3, struct gpio_t)

#endif	/* gpio_sa1100_h */
