/*
 * Itsy (SA1100+UCB1200) Codec Header File
 * Copyright (c) Compaq Computer Corporation, 1998, 1999
 *
 * Author: Carl Waldspurger
 *
 * $Log: itsy_codec.h,v $
 * Revision 2.5  1999/12/23  01:28:24  kerr
 * Updated Copyright information for entire source tree.
 *
 * Revision 2.4  1999/11/08  22:36:45  caw
 * Updated CODEC_ITSY_V2_IO_BUTTONS to include encoder wheel motion.
 *
 * Revision 2.3  1999/10/05 01:29:09  caw
 * Updated itsy-specific codec constants (for both V1 and V2).
 *
 * Revision 2.2  1999/04/13 19:42:58  brakmo
 * Added a flag, foreground session id and register respectively.
 *
 * Revision 2.1  1999/03/29  23:37:04  caw
 * Initial revision, based on now-obsolete "mcp-sa1100.h".
 *
 */

#ifndef	ITSY_CODEC_H
#define	ITSY_CODEC_H

/*
 * types
 *
 */

/* memory-mapped GPIO registers */
typedef struct {
  volatile uint pin_level;
  volatile uint pin_direction;
  volatile uint pin_out_set;
  volatile uint pin_out_clear;
  volatile uint rising_edge;
  volatile uint falling_edge;
  volatile uint edge_detect; 
  volatile uint alternate_fun;
} gpio_reg;

/* memory-mapped MCP registers */
typedef struct {
  volatile uint mccr;
  volatile uint reserved0;
  volatile uint mcdr0;
  volatile uint mcdr1;
  volatile uint mcdr2;
  volatile uint reserved1;
  volatile uint mcsr;
} mcp_reg;

/*
 * GPIO constants
 *
 */

#define	GPIO_BASE_P	((gpio_reg *) GPLR_P)
#define	GPIO_BASE_V	((gpio_reg *) GPLR_V)

/*
 * MCP constants
 *
 */

#define	MCP_IRQ		(18)

#define MCP_BASE_P	((mcp_reg *) 0x80060000)
#define MCP_BASE_V	((mcp_reg *) 0xe0060000)

#define MCCR_V_ASD	0
#define MCCR_V_TSD	8
#define MCCR_M_MCE	0x010000
#define MCCR_M_ECS	0x020000
#define MCCR_M_ADM	0x040000
#define MCCR_M_TTM	0x080000
#define MCCR_M_TRM	0x100000
#define MCCR_M_ATE	0x200000
#define MCCR_M_ARE	0x400000
#define MCCR_M_LBM	0x800000

#define MCDR2_V_RN	17
#define MCDR2_M_nRW	0x10000

#define MCSR_M_ATS	0x00000001
#define MCSR_M_ARS	0x00000002
#define MCSR_M_TTS	0x00000004
#define MCSR_M_TRS	0x00000008
#define MCSR_M_ATU	0x00000010
#define MCSR_M_ARO	0x00000020
#define MCSR_M_TTU	0x00000040
#define MCSR_M_TRO	0x00000080
#define MCSR_M_ANF	0x00000100
#define MCSR_M_ANE	0x00000200
#define MCSR_M_TNF	0x00000400
#define MCSR_M_TNE	0x00000800
#define MCSR_M_CWC	0x00001000
#define MCSR_M_CRC	0x00002000
#define MCSR_M_ACE	0x00004000
#define MCSR_M_TCE	0x00008000

/*
 * UCB1200 constants
 *
 */

/* shared codec irq */
#define	CODEC_IRQ			(6)

/* codec registers */
#define	CODEC_REG_IO_DATA		(0)
#define	CODEC_REG_IO_DIRECTION		(1)
#define	CODEC_REG_RISE_INT_ENABLE	(2)
#define	CODEC_REG_FALL_INT_ENABLE	(3)
#define	CODEC_REG_INT_STATUS		(4)
#define	CODEC_REG_TELECOM_CTL_A		(5)
#define	CODEC_REG_TELECOM_CTL_B		(6)
#define	CODEC_REG_AUDIO_CTL_A		(7)
#define	CODEC_REG_AUDIO_CTL_B		(8)
#define CODEC_REG_TS_CTL		(9)
#define	CODEC_REG_ADC_CTL		(10)
#define	CODEC_REG_ADC_DATA		(11)
#define	CODEC_REG_ID			(12)
#define	CODEC_REG_MODE			(13)

/* itsyV1-specific codec register values */
#define	CODEC_ITSY_V1_IO_DIRECTIONS	(0x8200)
#define	CODEC_ITSY_V1_IO_DATA_INIT	(0x0000)
#define	CODEC_ITSY_V1_IO_BUTTONS	(0x01ff)

/* itsyV2-specific codec register values */
#define	CODEC_ITSY_V2_IO_DIRECTIONS	(0x8007)
#define	CODEC_ITSY_V2_IO_DATA_INIT	(0x0001)
#define	CODEC_ITSY_V2_IO_BUTTONS_OLD	(0x03c0)
#define	CODEC_ITSY_V2_IO_BUTTONS	(0x03f0)
#define	CODEC_ITSY_V2_IO_BUTTONS_MUX	(0x0006)
#define	CODEC_ITSY_V2_IO_BUTTONS_MUX0	(0x0002)
#define	CODEC_ITSY_V2_IO_BUTTONS_MUX1	(0x0004)

/* 
 * itsy-specific touchscreen and adc register defines
 *
 */

/* Codec registers 2, 3 and 4: interrupt related registers */
#define ADC_INT			(1 << 11)
#define TSPX_INT		(1 << 12)
#define TSMX_INT		(1 << 13)

/* Codec register 9: Touchscreen control register */
#define TSMX_POW		(1 << 0)
#define TSPX_POW		(1 << 1)
#define TSMY_POW		(1 << 2)
#define TSPY_POW		(1 << 3)
#define TSMX_GND		(1 << 4)
#define TSPX_GND		(1 << 5)
#define TSMY_GND		(1 << 6)
#define TSPY_GND		(1 << 7)
#define TSC_MODE_MASK		(3 << 8)
#define TSC_MODE_INT		(0 << 8)
#define TSC_MODE_PRESSURE	(1 << 8)
#define TSC_MODE_POSITION	(1 << 9)
#define TSC_BIAS_ENA		(1 << 11)
#define TSPX_LOW		(1 << 12)
#define TSMX_LOW		(1 << 13)

/* Codec register 10: ADC control register */
#define ADC_SYNC_ENA		(1 << 0)
#define ADC_INPUT_MASK		(7 << 2)
#define ADC_INPUT_TSPX		(0 << 2)
#define ADC_INPUT_TSMX		(1 << 2)
#define ADC_INPUT_TSPY		(2 << 2)
#define ADC_INPUT_TSMY		(3 << 2)
#define ADC_INPUT_AD0		(4 << 2)
#define ADC_INPUT_AD1		(5 << 2)
#define ADC_INPUT_AD2		(6 << 2)
#define ADC_INPUT_AD3		(7 << 2)
#define ADC_START		(1 << 7)
#define ADC_ENA			(1 << 15)

/* Codec register 11: ADC data register */
#define ADC_DATA_SHIFT_VAL	(5)
#define GET_ADC_DATA(x)		(((x)>>ADC_DATA_SHIFT_VAL)&0x3ff)
#define ADC_DAT_VAL		(1 << 15)
#define ADC_ENA_TYPE 		(ADC_SYNC_ENA | ADC_ENA)   

/*
 * useful operations
 *
 */

static inline void mcp_mcsr_wait_read(mcp_reg *mcp)
{
  /* wait for outstanding read to complete */
  while ((mcp->mcsr & MCSR_M_CRC) == 0)  { /* spin */ }
}

static inline void mcp_mcsr_wait_write(mcp_reg *mcp)
{
  /* wait for outstanding write to complete */
  while ((mcp->mcsr & MCSR_M_CWC) == 0)  { /* spin */ }
}

static inline ushort codec_read(mcp_reg *mcp, ushort addr)
{
  ushort data; 
  ulong flags;

  /* critical section */
  save_flags_cli(flags);
  {
    mcp_mcsr_wait_write(mcp);
    mcp->mcdr2 = ((addr & 0xf) << MCDR2_V_RN);
    mcp_mcsr_wait_read(mcp);
    data = mcp->mcdr2 & 0xffff;
  }
  restore_flags(flags);

  return(data);
}

static inline void codec_write(mcp_reg *mcp, ushort addr, ushort data)
{
  ulong flags;

  /* critical section */
  save_flags_cli(flags);
  {
    mcp_mcsr_wait_write(mcp);
    mcp->mcdr2 = ((addr & 0xf) << MCDR2_V_RN) | MCDR2_M_nRW | (data & 0xffff);
  }
  restore_flags(flags);
}

#endif	/* ITSY_CODEC_H */
