/*
 * Itsy frame buffer device
 *
 * Copyright (c) Compaq Computer Corporation, 1998, 1999
 *
 * Author: Lawrence S. Brakmo
 *
 * $Log: itsy_fb.h,v $
 * Revision 2.3  1999/12/23  01:28:26  kerr
 * Updated Copyright information for entire source tree.
 *
 * Revision 2.2  1999/10/09  02:59:50  brakmo
 * Added support for backlight (ioctl to turn on/off and fix colormap).
 *
 * Revision 2.1  1999/05/12 18:01:10  caw
 * Included changes from itsy_sessions branch.
 *
 * Revision 1.1.2.1  1999/04/21  01:23:40  brakmo
 * Renamed files from *brutus* into itsy_*. Added itsy_adc.h
 *
 * Revision 2.2  1999/01/20  20:53:35  brakmo
 * Added new ioctls and function declarations.
 *
 * Revision 2.1  1998/07/02  01:50:13  brakmo
 * Added kernel support for new ggi graphics library.
 *
 * Revision 1.2  1998/06/16  19:20:06  kramer
 * Added copyright information
 *
 * Revision 1.1  1998/03/11  16:39:19  brakmo
 * Created brutus frame buffer include file (ioctl ops).
 *
 * Revision 2.0  1998/06/25  18:38:51  kramer
 * Major Bump: Update all files to version 2.0 for external release
 * -drew
 *
 */

#ifndef itsy_fb_h
#define itsy_fb_h

#include <linux/itsy_console.h>

struct FbLcdParamsStruct {
  int  screenSizeH;		/* in pixels */
  int  screenSizeV;
  int  frameBufferSizeH;	/* May be > than screenSizeH due to padding*/
  				/* in bytes */
  int  frameBufferSize;		/* Size of framebuffer in bytes (in page-size */
  				/* multiples */
  int  pixelDataOffset;		/* Offset from beginning of framebuffer to */
  				/* pixel data */
  int  bitsPerPixel;
  int  reserveTopRows;
  int  reserveBottomRows;
  int  paletteSize;
#define FB_LCD_GRAY   	1
#define FB_LCD_COLOR8	2
#define FB_LCD_COLOR12	3
#define FB_LCD_COLOR16	4
#define FB_LCD_COLOR24	5
  int  colorType;
  int  bitsPerColor;
#define FB_LCD_FLAG_PALETTE_FIXED 	1
  int  flags;
};
  
/* driver ioctl type */
#define FB_IOCTL_TYPE	'F'

#if 0
/* I will keep the numbers the same for a while */
#define FB_IO(type,no) 		no
#define FB_IOW(type,no,size) 	no
#define FB_IOR(type,no,size) 	no
#define FB_IOWR(type,no,size) 	no

/* Later I will change them to the recomemded numbers */
#else
#define FB_IO(type,no) 		_IO(type,no)
#define FB_IOW(type,no,size) 	_IOW(type,no,size)
#define FB_IOR(type,no,size) 	_IOR(type,no,size)
#define FB_IOWR(type,no,size) 	_IOWR(type,no,size)
#endif

/* ioctl commands */
#define FB_LCD_ON 		FB_IO(FB_IOCTL_TYPE, 1)
#define FB_LCD_OFF		FB_IO(FB_IOCTL_TYPE, 2)
#define FB_LCD_STATIC		FB_IO(FB_IOCTL_TYPE, 3)
#define FB_LCD_SWITCH  		FB_IO(FB_IOCTL_TYPE, 4)
#define FB_LCD_PARAMS 		FB_IOR(FB_IOCTL_TYPE, 5, struct FbLcdParamsStruct)
#define FB_LCD_ALLOC    	FB_IOR(FB_IOCTL_TYPE, 6, int)
#define FB_LCD_FREE     	FB_IO(FB_IOCTL_TYPE, 7)
#define FB_LCD_GET      	FB_IOR(FB_IOCTL_TYPE, 8, int)
#define FB_LCD_MMAP_SET 	FB_IO(FB_IOCTL_TYPE, 9)
#define FB_LCD_STATIC_WCB 	FB_IO(FB_IOCTL_TYPE, 10)
#define FB_LCD_CB_OFF		FB_IO(FB_IOCTL_TYPE, 11)
#define FB_LCD_CB_ON		FB_IO(FB_IOCTL_TYPE, 12)
#define FB_LCD_GET_FBN		FB_IOR(FB_IOCTL_TYPE, 13, int)
#define FB_LCD_SHOW		FB_IO(FB_IOCTL_TYPE, 14)
#define FB_LCD_SET_COLORTABLE	FB_IOW(FB_IOCTL_TYPE, 15, (sizeof(short)*PALETTE_NUM_ENTRIES))
#define FB_LCD_GET_COLORTABLE 	FB_IOR(FB_IOCTL_TYPE, 16, (sizeof(short)*PALETTE_NUM_ENTRIES))
#define FB_LCD_SET_BACKLIGHT	FB_IO(FB_IOCTL_TYPE, 17)
#define FB_LCD_GET_BACKLIGHT	FB_IOR(FB_IOCTL_TYPE, 18, int)

#endif
