/*
 * Itsy SA1100 Statistical Profiler
 * Copyright (c) Compaq Computer Corporation, 1999
 *
 * Author: Carl Waldspurger
 *
 * $Log: itsy_prof.h,v $
 * Revision 2.5  1999/12/23  01:28:27  kerr
 * Updated Copyright information for entire source tree.
 *
 * Revision 2.4  1999/08/23  22:10:51  caw
 * Added documentation.
 *
 * Revision 2.3  1999/08/17 01:04:13  caw
 * Defined special sample values.
 *
 * Revision 2.2  1999/08/13 00:46:22  caw
 * Changed itsy_prof_ctl, improving ITSY_PROF_{GET,SET}_CTL interface.
 * Added more documentation.
 *
 * Revision 2.1  1999/08/10  01:15:21  caw
 * Initial revision.
 *
 */

#ifndef	ITSY_PROF_H
#define	ITSY_PROF_H

/*
 * constants
 *
 */

/* valid sampling freq in [10, 1000] */
#define	ITSY_PROF_FREQ_MIN	(10)
#define	ITSY_PROF_FREQ_MAX	(1000)

/* sampling freq defaults */
#define	ITSY_PROF_FREQ_DEFAULT	(256)
#define	ITSY_PROF_RND_DEFAULT	(1)

/* special sample values */
#define	ITSY_PROF_DEVICE_NONE	(0)
#define	ITSY_PROF_INODE_NONE	(0)
#define	ITSY_PROF_INODE_KERNEL	(1)
#define	ITSY_PROF_PID_NONE	(0xffff)

/*
 * types
 *
 */

typedef unsigned char uchar;

/* profiling sample */
typedef struct {
  ulong  pid;	    /* process identifier */
  ulong  offset;    /* offset into executable file */
  ulong  inode;     /* executable file inode */
  ushort device;    /* executable file device */
  uchar  mode;	    /* processor mode */
  uchar  count;	    /* sample count */
}  __attribute__ ((packed)) itsy_prof_sample;

/* statistics */
typedef struct {
  ulong interrupts; /* total interrupts */
  ulong samples;    /* total samples */
  ulong wakeups;    /* client wakeups */
  ulong user;       /* user samples */
  ulong kernel;     /* kernel samples */
  ulong nomap;      /* pc -> image, offset failed */
  ulong unused[2];  /* reserved for future use */
} itsy_prof_stats;

/* control parameters */
typedef struct {
  ulong frequency;  /* samples per sec */
  ulong randomize;  /* boolean flag */
  ulong unused[4];  /* reserved for future use */
} itsy_prof_ctl;

/* stream position (64 bits) */
typedef long long itsy_prof_pos;

/*
 * ioctl interface
 *
 */

/* driver ioctl type */
#define	ITSY_PROF_IOCTL_TYPE	'p'
#define	ITSY_PROF_IOCTL_MAXNR	(6)

/* ioctl commands */
#define	ITSY_PROF_GET_CTL	_IOR(ITSY_PROF_IOCTL_TYPE,  0, itsy_prof_ctl)
#define	ITSY_PROF_SET_CTL	_IOW(ITSY_PROF_IOCTL_TYPE,  1, itsy_prof_ctl)

#define	ITSY_PROF_GET_POSITION	_IOR(ITSY_PROF_IOCTL_TYPE,  2, itsy_prof_pos)
#define	ITSY_PROF_SET_POSITION	_IOWR(ITSY_PROF_IOCTL_TYPE, 3, itsy_prof_pos)

#define	ITSY_PROF_GET_DROPPED	_IOR(ITSY_PROF_IOCTL_TYPE,  4, itsy_prof_pos)
#define	ITSY_PROF_SET_DROPPED	_IOW(ITSY_PROF_IOCTL_TYPE,  5, itsy_prof_pos)
#define	ITSY_PROF_SET_DROP_FAIL	_IO(ITSY_PROF_IOCTL_TYPE,   6)

#endif	/* ITSY_PROF_H */
