/*
 * itsy_rocknscroll.h
 *
 * Rock 'n' Scroll driver for Itsy V1.5 and V2.x with ADXL202 directly 
 * connected to the StrongARM.
 *
 * Copyright (c) Compaq Computer Corporation, 1999. 
 *
 * $Log: itsy_rocknscroll.h,v $
 * Revision 2.3  1999/12/23  01:28:28  kerr
 * Updated Copyright information for entire source tree.
 *
 * Revision 2.2  1999/10/09  01:32:56  bartlett
 * Added support for Itsy V2
 *
 * Revision 2.1  1999/07/14  03:32:23  gback
 * initial version
 *
 */

#ifndef itsy_rocknscroll_h
#define itsy_rocknscroll_h

#define ADXL_SAMPLE_COUNT 1000

struct ADXL_SAMPLE {
   int  free;                   /* Index of first free entry */
   int  overflow;               /* Buffer full so sample had to be flushed */
   struct {
      unsigned int  time;       /* OS timer value */
      unsigned int  gpios;      /* GPIO input values */
   } samples[ ADXL_SAMPLE_COUNT ];
};

/* Device information */

struct ADXL_INFO {
   int  gpio_xaxis;		/* GPIO for ADXL202 x-axis */
   int  gpio_yaxis;		/* GPIO for ADXL202 y-axis */
   int  pitch_x;		/* Co-efficients for converting xy to pitch */
   int  pitch_y;
   int  roll_x;			/* Co-efficients for converting xy to roll */
   int  roll_y;
};

#ifdef MODULE

struct ADXL_INFO adxl_info[ ] = { {  0,  0, 0, 0, 0, 0 },	
				  { 14, 15, 0, 1, 1, 0 },  /* Version 1.x */
				  {  7,  8, 1, 0, 0, 1 }   /* Version 2.x */
				   };

#define ROCKNSCROLL_MAJOR 45
#define ROCKNSCROLL_NAME "rocknscroll"

#endif /* MODULE */

/*
 * ioctl interface
 *
 */

/* driver ioctl type */
#define ROCKNSCROLL_IOCTL_TYPE      'r'
#define ROCKNSCROLL_IOCTL_MAXNR     (2)

/* ioctl commands */

/* minimum number of samples before read() returns, default is 1 */
#define ROCKNSCROLL_SET_MINSAMPLE        _IO(ROCKNSCROLL_IOCTL_TYPE, 0)
#define ROCKNSCROLL_GET_MINSAMPLE        _IOR(ROCKNSCROLL_IOCTL_TYPE, 1, ulong)

#define ROCKNSCROLL_MINSAMPLE_DEFAULT	1

/* returns major hardware version # */
#define ROCKNSCROLL_GET_ADXL_INFO        _IOR(ROCKNSCROLL_IOCTL_TYPE, 2,\
					      struct ADXL_INFO)

#endif /* itsy_rocknscroll_h */
