/*
 * itsy_ts.h
 *
 * Touchscreen device include file
 *
 * Copyright (c) Compaq Computer Corporation, 1998, 1999
 *
 * Author: Lawrence S. Brakmo
 *
 * $Log: itsy_ts.h,v $
 * Revision 2.3  1999/12/23  01:28:28  kerr
 * Updated Copyright information for entire source tree.
 *
 * Revision 2.2  1999/06/10  20:16:48  brakmo
 * Added new ioctls to allow rate change per session.
 *
 * Revision 2.1  1999/05/12 18:01:11  caw
 * Included changes from itsy_sessions branch.
 *
 * Revision 1.1.2.1  1999/04/21  01:23:41  brakmo
 * Renamed files from *brutus* into itsy_*. Added itsy_adc.h
 *
 * 
 */

#ifndef itsy_ts_h
#define itsy_ts_h

/*
 * includes
 */

#include <linux/itsy_event.h> 

/*
 * constants
 */

#define ITSY_EVENT_TOUCHSCREEN		1
#define TS_MAX_RATE			200	/* per second */

/*
 * types
 */

typedef struct {
  ushort x;
  ushort y;
} ts_point;

typedef struct {
  itsy_event_header common;
  int sid;
  ushort pressure;
  ts_point point;
} ts_event;

typedef struct {
  int xscale;
  int xtrans;
  int yscale;
  int ytrans;
  int xyswap;
  int minp;
} ts_calibration;

typedef struct {
  int	 id;
  ts_point p0;
  ts_point p1;
} ts_grab_area;

/*
 * raw to calibrated is achieved by:
 *
 * ts_calibration *cal;
 * ts_point *from, *to;
 *
 *  if (cal->xyswap) {
 *    to->x = ((y * cal->xscale) >> 8) + cal->xtrans;
 *    to->y = ((x * cal->yscale) >> 8) + cal->ytrans;
 *  }
 *  else {
 *    to->x = ((x * cal->xscale) >> 8) + cal->xtrans;
 *    to->y = ((y * cal->yscale) >> 8) + cal->ytrans;
 *  }
 *
 * Given raw (raw touchscreen data) point "raw" and screen (lcd)
 * point "screen" , the calibration parameters are:
 *
 */


/*
 * ioctl interface
 */

/* driver ioctl type */
#define TS_IOCTL_TYPE		'e'
#define TS_IOCTL_MAXNR		(14)	/* there first 4 are defined in
					 * itsy_adc.h */

/* ioctl commands */
#define TS_SET_DEFAULT_CALIBRATION	_IOW(TS_IOCTL_TYPE, 4, ts_calibration)
#define TS_GET_DEFAULT_CALIBRATION	_IOR(TS_IOCTL_TYPE, 5, ts_calibration)
#define TS_SET_CALIBRATION		_IOW(TS_IOCTL_TYPE, 6, ts_calibration)
#define TS_GET_CALIBRATION		_IOR(TS_IOCTL_TYPE, 7, ts_calibration)
#define TS_GET_GRABBER			_IOR(TS_IOCTL_TYPE, 8, long)
#define TS_ADD_GRAB_AREA		_IOWR(TS_IOCTL_TYPE, 9, ts_grab_area)
#define TS_DELETE_GRAB_AREA		_IOW(TS_IOCTL_TYPE, 10, long)
#define TS_SET_RATE			_IO(TS_IOCTL_TYPE, 11)
#define TS_GET_RATE			_IOR(TS_IOCTL_TYPE, 12, long)
#define TS_SET_DEFAULT_RATE		_IO(TS_IOCTL_TYPE, 13)
#define TS_GET_DEFAULT_RATE		_IOR(TS_IOCTL_TYPE, 14, long)

#endif
