/*
 * include/linux/serial.h
 *
 * Copyright (C) 1992, 1994 by Theodore Ts'o.
 * 
 * Redistribution of this file is permitted under the terms of the GNU 
 * Public License (GPL)
 * 
 * These are the UART port assignments, expressed as offsets from the base
 * register.  These assignments should hold for any serial port based on
 * a 8250, 16450, or 16550(A).
 */

#ifndef _LINUX_SERIAL_REG_H
#define _LINUX_SERIAL_REG_H

#include <asm/arch/SA-1100.h>

/*
 * Register offsets.
 */
#define UTCR0               0x00   /* UART_LCR  Line control register */
#define UTCR1               0x04   /* UART_DLLSB */
#define UTCR2               0x08   /* UART_DLMSB */
#define UTCR3               0x0c   /* UART_IER */
#define UTDR                0x14   /* UART_RX, UART_TX */
#define UTSR0               0x1C   
#define UTSR1               0x20   /* UART_LSR  Line Status register */



#define UART_RX             UTDR      /* Receive port, read only */
#define UART_TX             UTDR      /* transmit port, write only */

/*
 * Line control register 
 */
#define UTCR0_EP              UTCR0_OES      /* Even parity */
#define UTCR0_SB              UTCR0_SBS      /* Stop bit */
#define UTCR0_DB              UTCR0_DSS      /* Data bits in transmission (0 = 7, 1 = 8) */


/*
 * Line status bits.
 */
#define UTSR1_ERROR           (UTSR1_PRE | UTSR1_FRE | UTSR1_ROR)   /* LSR_ERROR */


#endif /* _LINUX_SERIAL_REG_H */

